// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/http/tap/v3/tap.proto

package tapv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/common/tap/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Top level configuration for the tap filter.
type Tap struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Common configuration for the HTTP tap filter.
	CommonConfig *v3.CommonExtensionConfig `protobuf:"bytes,1,opt,name=common_config,json=commonConfig,proto3" json:"common_config,omitempty"`
	// Indicates whether HTTP tap filter records the time stamp for request/response headers.
	// Request headers time stamp is stored after receiving request headers.
	// Response headers time stamp is stored after receiving response headers.
	RecordHeadersReceivedTime bool `protobuf:"varint,2,opt,name=record_headers_received_time,json=recordHeadersReceivedTime,proto3" json:"record_headers_received_time,omitempty"`
	// Indicates whether report downstream connection info
	RecordDownstreamConnection bool `protobuf:"varint,3,opt,name=record_downstream_connection,json=recordDownstreamConnection,proto3" json:"record_downstream_connection,omitempty"`
	// If enabled, upstream connection information will be reported.
	RecordUpstreamConnection bool `protobuf:"varint,4,opt,name=record_upstream_connection,json=recordUpstreamConnection,proto3" json:"record_upstream_connection,omitempty"`
	unknownFields            protoimpl.UnknownFields
	sizeCache                protoimpl.SizeCache
}

func (x *Tap) Reset() {
	*x = Tap{}
	mi := &file_envoy_extensions_filters_http_tap_v3_tap_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Tap) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Tap) ProtoMessage() {}

func (x *Tap) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_tap_v3_tap_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Tap.ProtoReflect.Descriptor instead.
func (*Tap) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDescGZIP(), []int{0}
}

func (x *Tap) GetCommonConfig() *v3.CommonExtensionConfig {
	if x != nil {
		return x.CommonConfig
	}
	return nil
}

func (x *Tap) GetRecordHeadersReceivedTime() bool {
	if x != nil {
		return x.RecordHeadersReceivedTime
	}
	return false
}

func (x *Tap) GetRecordDownstreamConnection() bool {
	if x != nil {
		return x.RecordDownstreamConnection
	}
	return false
}

func (x *Tap) GetRecordUpstreamConnection() bool {
	if x != nil {
		return x.RecordUpstreamConnection
	}
	return false
}

var File_envoy_extensions_filters_http_tap_v3_tap_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDesc = "" +
	"\n" +
	".envoy/extensions/filters/http/tap/v3/tap.proto\x12$envoy.extensions.filters.http.tap.v3\x1a+envoy/extensions/common/tap/v3/common.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xdd\x02\n" +
	"\x03Tap\x12d\n" +
	"\rcommon_config\x18\x01 \x01(\v25.envoy.extensions.common.tap.v3.CommonExtensionConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\fcommonConfig\x12?\n" +
	"\x1crecord_headers_received_time\x18\x02 \x01(\bR\x19recordHeadersReceivedTime\x12@\n" +
	"\x1crecord_downstream_connection\x18\x03 \x01(\bR\x1arecordDownstreamConnection\x12<\n" +
	"\x1arecord_upstream_connection\x18\x04 \x01(\bR\x18recordUpstreamConnection:/\x9aň\x1e*\n" +
	"(envoy.config.filter.http.tap.v2alpha.TapB\x9b\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"2io.envoyproxy.envoy.extensions.filters.http.tap.v3B\bTapProtoP\x01ZQgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/tap/v3;tapv3b\x06proto3"

var (
	file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDescData []byte
)

func file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDesc), len(file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDescData
}

var file_envoy_extensions_filters_http_tap_v3_tap_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_http_tap_v3_tap_proto_goTypes = []any{
	(*Tap)(nil),                      // 0: envoy.extensions.filters.http.tap.v3.Tap
	(*v3.CommonExtensionConfig)(nil), // 1: envoy.extensions.common.tap.v3.CommonExtensionConfig
}
var file_envoy_extensions_filters_http_tap_v3_tap_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.http.tap.v3.Tap.common_config:type_name -> envoy.extensions.common.tap.v3.CommonExtensionConfig
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_http_tap_v3_tap_proto_init() }
func file_envoy_extensions_filters_http_tap_v3_tap_proto_init() {
	if File_envoy_extensions_filters_http_tap_v3_tap_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDesc), len(file_envoy_extensions_filters_http_tap_v3_tap_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_http_tap_v3_tap_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_http_tap_v3_tap_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_http_tap_v3_tap_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_http_tap_v3_tap_proto = out.File
	file_envoy_extensions_filters_http_tap_v3_tap_proto_goTypes = nil
	file_envoy_extensions_filters_http_tap_v3_tap_proto_depIdxs = nil
}
