// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/formatter/metadata/v3/metadata.proto

package metadatav3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// %METADATA(TYPE:NAMESPACE:KEY):Z%
//
//	  :ref:`Metadata <envoy_v3_api_msg_config.core.v3.Metadata>` info,
//	  where TYPE is type of metadata (see above for supported types),
//	  NAMESPACE is the filter namespace used when setting the metadata, KEY is an optional
//	  lookup key in the namespace with the option of specifying nested keys separated by ':',
//	  and Z is an optional parameter denoting string truncation up to Z characters long.
//	  The data will be logged as a JSON string. For example, for the following ROUTE metadata:
//
//	  ``com.test.my_filter: {"test_key": "foo", "test_object": {"inner_key": "bar"}}``
//
//	  * %METADATA(ROUTE:com.test.my_filter)% will log: ``{"test_key": "foo", "test_object": {"inner_key": "bar"}}``
//	  * %METADATA(ROUTE:com.test.my_filter:test_key)% will log: ``foo``
//	  * %METADATA(ROUTE:com.test.my_filter:test_object)% will log: ``{"inner_key": "bar"}``
//	  * %METADATA(ROUTE:com.test.my_filter:test_object:inner_key)% will log: ``bar``
//	  * %METADATA(ROUTE:com.unknown_filter)% will log: ``-``
//	  * %METADATA(ROUTE:com.test.my_filter:unknown_key)% will log: ``-``
//	  * %METADATA(ROUTE:com.test.my_filter):25% will log (truncation at 25 characters): ``{"test_key": "foo", "test``
//
//	.. note::
//
//	  For typed JSON logs, this operator renders a single value with string, numeric, or boolean type
//	  when the referenced key is a simple value. If the referenced key is a struct or list value, a
//	  JSON struct or list is rendered. Structs and lists may be nested. In any event, the maximum
//	  length is ignored.
//
//	.. note::
//
//	  METADATA(DYNAMIC:NAMESPACE:KEY):Z is equivalent to :ref:`DYNAMIC_METADATA(NAMESPACE:KEY):Z<config_access_log_format_dynamic_metadata>`
//	  METADATA(CLUSTER:NAMESPACE:KEY):Z is equivalent to :ref:`CLUSTER_METADATA(NAMESPACE:KEY):Z<config_access_log_format_cluster_metadata>`
//	  METADATA(UPSTREAM_HOST:NAMESPACE:KEY):Z is equivalent to :ref:`UPSTREAM_METADATA(NAMESPACE:KEY):Z<config_access_log_format_upstream_host_metadata>`
//
// .. warning::
//
//	This extension is treated as built-in extension and will be enabled by default now.
//	It is unnecessary to configure this extension.
type Metadata struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Metadata) Reset() {
	*x = Metadata{}
	mi := &file_envoy_extensions_formatter_metadata_v3_metadata_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Metadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Metadata) ProtoMessage() {}

func (x *Metadata) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_formatter_metadata_v3_metadata_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Metadata.ProtoReflect.Descriptor instead.
func (*Metadata) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDescGZIP(), []int{0}
}

var File_envoy_extensions_formatter_metadata_v3_metadata_proto protoreflect.FileDescriptor

const file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDesc = "" +
	"\n" +
	"5envoy/extensions/formatter/metadata/v3/metadata.proto\x12&envoy.extensions.formatter.metadata.v3\x1a\x1dudpa/annotations/status.proto\"\n" +
	"\n" +
	"\bMetadataB\xa9\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"4io.envoyproxy.envoy.extensions.formatter.metadata.v3B\rMetadataProtoP\x01ZXgithub.com/envoyproxy/go-control-plane/envoy/extensions/formatter/metadata/v3;metadatav3b\x06proto3"

var (
	file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDescOnce sync.Once
	file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDescData []byte
)

func file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDescGZIP() []byte {
	file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDesc), len(file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDesc)))
	})
	return file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDescData
}

var file_envoy_extensions_formatter_metadata_v3_metadata_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_formatter_metadata_v3_metadata_proto_goTypes = []any{
	(*Metadata)(nil), // 0: envoy.extensions.formatter.metadata.v3.Metadata
}
var file_envoy_extensions_formatter_metadata_v3_metadata_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_extensions_formatter_metadata_v3_metadata_proto_init() }
func file_envoy_extensions_formatter_metadata_v3_metadata_proto_init() {
	if File_envoy_extensions_formatter_metadata_v3_metadata_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDesc), len(file_envoy_extensions_formatter_metadata_v3_metadata_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_formatter_metadata_v3_metadata_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_formatter_metadata_v3_metadata_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_formatter_metadata_v3_metadata_proto_msgTypes,
	}.Build()
	File_envoy_extensions_formatter_metadata_v3_metadata_proto = out.File
	file_envoy_extensions_formatter_metadata_v3_metadata_proto_goTypes = nil
	file_envoy_extensions_formatter_metadata_v3_metadata_proto_depIdxs = nil
}
