// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/upstreams/http/v3/http_protocol_options.proto

package httpv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v33 "github.com/envoyproxy/go-control-plane/envoy/config/common/matcher/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v32 "github.com/envoyproxy/go-control-plane/envoy/config/route/v3"
	v31 "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/http_connection_manager/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// HttpProtocolOptions specifies Http upstream protocol options. This object
// is used in
// :ref:`typed_extension_protocol_options<envoy_v3_api_field_config.cluster.v3.Cluster.typed_extension_protocol_options>`,
// keyed by the name “envoy.extensions.upstreams.http.v3.HttpProtocolOptions“.
//
// This controls what protocol(s) should be used for upstream and how said protocol(s) are configured.
//
// This replaces the prior pattern of explicit protocol configuration directly
// in the cluster. So a configuration like this, explicitly configuring the use of HTTP/2 upstream:
//
// .. code::
//
//	clusters:
//	  - name: some_service
//	    connect_timeout: 5s
//	    upstream_http_protocol_options:
//	      auto_sni: true
//	    common_http_protocol_options:
//	      idle_timeout: 1s
//	    http2_protocol_options:
//	      max_concurrent_streams: 100
//	     .... [further cluster config]
//
// Would now look like this:
//
// .. code::
//
//	clusters:
//	  - name: some_service
//	    connect_timeout: 5s
//	    typed_extension_protocol_options:
//	      envoy.extensions.upstreams.http.v3.HttpProtocolOptions:
//	        "@type": type.googleapis.com/envoy.extensions.upstreams.http.v3.HttpProtocolOptions
//	        upstream_http_protocol_options:
//	          auto_sni: true
//	        common_http_protocol_options:
//	          idle_timeout: 1s
//	        explicit_http_config:
//	          http2_protocol_options:
//	            max_concurrent_streams: 100
//	     .... [further cluster config]
//
// [#next-free-field: 12]
type HttpProtocolOptions struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// This contains options common across HTTP/1 and HTTP/2
	CommonHttpProtocolOptions *v3.HttpProtocolOptions `protobuf:"bytes,1,opt,name=common_http_protocol_options,json=commonHttpProtocolOptions,proto3" json:"common_http_protocol_options,omitempty"`
	// This contains common protocol options which are only applied upstream.
	UpstreamHttpProtocolOptions *v3.UpstreamHttpProtocolOptions `protobuf:"bytes,2,opt,name=upstream_http_protocol_options,json=upstreamHttpProtocolOptions,proto3" json:"upstream_http_protocol_options,omitempty"`
	// This controls the actual protocol to be used upstream.
	//
	// Types that are valid to be assigned to UpstreamProtocolOptions:
	//
	//	*HttpProtocolOptions_ExplicitHttpConfig_
	//	*HttpProtocolOptions_UseDownstreamProtocolConfig
	//	*HttpProtocolOptions_AutoConfig
	UpstreamProtocolOptions isHttpProtocolOptions_UpstreamProtocolOptions `protobuf_oneof:"upstream_protocol_options"`
	// Optional HTTP filters for the upstream HTTP filter chain.
	//
	// .. note::
	//
	//	Upstream HTTP filters are currently in alpha.
	//
	// These filters will be applied for all HTTP streams which flow through this
	// cluster. Unlike downstream HTTP filters, they will *not* be applied to terminated CONNECT requests.
	//
	// If using upstream HTTP filters, please be aware that local errors sent by
	// upstream HTTP filters will not trigger retries, and local errors sent by
	// upstream HTTP filters will count as a final response if hedging is configured.
	// [#extension-category: envoy.filters.http.upstream]
	HttpFilters []*v31.HttpFilter `protobuf:"bytes,6,rep,name=http_filters,json=httpFilters,proto3" json:"http_filters,omitempty"`
	// Configuration options for Unified Header Validation (UHV).
	// UHV is an extensible mechanism for checking validity of HTTP responses.
	//
	// [#comment:TODO(yanavlasov): Make it a link to the default header validator doc when it becomes visible.]
	// Leaving this field unspecified, selects the default header validator “envoy.http.header_validators.envoy_default“.
	//
	// [#not-implemented-hide:]
	// [#extension-category: envoy.http.header_validators]
	HeaderValidationConfig *v3.TypedExtensionConfig `protobuf:"bytes,7,opt,name=header_validation_config,json=headerValidationConfig,proto3" json:"header_validation_config,omitempty"`
	// Defines http specific outlier detection parameters.
	OutlierDetection *HttpProtocolOptions_OutlierDetection `protobuf:"bytes,8,opt,name=outlier_detection,json=outlierDetection,proto3" json:"outlier_detection,omitempty"`
	// Specifies a list of HTTP-level mirroring policies for requests routed to this cluster.
	// Cluster-level policies override route-level policies when they both are configured.
	//
	// .. note::
	//
	//	Mirroring will not be triggered if the :ref:`primary cluster
	//	<envoy_v3_api_field_config.route.v3.RouteAction.cluster>` does not exist.
	RequestMirrorPolicies []*v32.RouteAction_RequestMirrorPolicy `protobuf:"bytes,9,rep,name=request_mirror_policies,json=requestMirrorPolicies,proto3" json:"request_mirror_policies,omitempty"`
	// Specifies a list of hash policies for consistent hashing load balancing (e.g., Ring Hash or
	// Maglev) for requests routed to this cluster. When configured, cluster-level policies override
	// route-level policies. When not configured, route-level policies (if any) will be used.
	//
	// This enables consistent routing to the same upstream host for all requests to a cluster,
	// which is particularly useful for stateful services like caching, session management, or
	// sticky routing requirements.
	//
	// .. note::
	//
	//	Hash policies are only effective when the cluster is configured with a hash-based load
	//	balancing policy (e.g., :ref:`RING_HASH <envoy_v3_api_enum_value_config.cluster.v3.Cluster.LbPolicy.RING_HASH>`
	//	or :ref:`MAGLEV <envoy_v3_api_enum_value_config.cluster.v3.Cluster.LbPolicy.MAGLEV>`).
	HashPolicy []*v32.RouteAction_HashPolicy `protobuf:"bytes,10,rep,name=hash_policy,json=hashPolicy,proto3" json:"hash_policy,omitempty"`
	// Specifies the retry policy for requests routed to this cluster. When configured,
	// cluster-level retry policy overrides route-level retry policy. When not configured,
	// route-level retry policy (if any) will be used.
	//
	// .. note::
	//
	//	Cluster-level retry policy will override route-level retry policy entirely. Policies are
	//	not merged.
	RetryPolicy   *v32.RetryPolicy `protobuf:"bytes,11,opt,name=retry_policy,json=retryPolicy,proto3" json:"retry_policy,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HttpProtocolOptions) Reset() {
	*x = HttpProtocolOptions{}
	mi := &file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpProtocolOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpProtocolOptions) ProtoMessage() {}

func (x *HttpProtocolOptions) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpProtocolOptions.ProtoReflect.Descriptor instead.
func (*HttpProtocolOptions) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDescGZIP(), []int{0}
}

func (x *HttpProtocolOptions) GetCommonHttpProtocolOptions() *v3.HttpProtocolOptions {
	if x != nil {
		return x.CommonHttpProtocolOptions
	}
	return nil
}

func (x *HttpProtocolOptions) GetUpstreamHttpProtocolOptions() *v3.UpstreamHttpProtocolOptions {
	if x != nil {
		return x.UpstreamHttpProtocolOptions
	}
	return nil
}

func (x *HttpProtocolOptions) GetUpstreamProtocolOptions() isHttpProtocolOptions_UpstreamProtocolOptions {
	if x != nil {
		return x.UpstreamProtocolOptions
	}
	return nil
}

func (x *HttpProtocolOptions) GetExplicitHttpConfig() *HttpProtocolOptions_ExplicitHttpConfig {
	if x != nil {
		if x, ok := x.UpstreamProtocolOptions.(*HttpProtocolOptions_ExplicitHttpConfig_); ok {
			return x.ExplicitHttpConfig
		}
	}
	return nil
}

func (x *HttpProtocolOptions) GetUseDownstreamProtocolConfig() *HttpProtocolOptions_UseDownstreamHttpConfig {
	if x != nil {
		if x, ok := x.UpstreamProtocolOptions.(*HttpProtocolOptions_UseDownstreamProtocolConfig); ok {
			return x.UseDownstreamProtocolConfig
		}
	}
	return nil
}

func (x *HttpProtocolOptions) GetAutoConfig() *HttpProtocolOptions_AutoHttpConfig {
	if x != nil {
		if x, ok := x.UpstreamProtocolOptions.(*HttpProtocolOptions_AutoConfig); ok {
			return x.AutoConfig
		}
	}
	return nil
}

func (x *HttpProtocolOptions) GetHttpFilters() []*v31.HttpFilter {
	if x != nil {
		return x.HttpFilters
	}
	return nil
}

func (x *HttpProtocolOptions) GetHeaderValidationConfig() *v3.TypedExtensionConfig {
	if x != nil {
		return x.HeaderValidationConfig
	}
	return nil
}

func (x *HttpProtocolOptions) GetOutlierDetection() *HttpProtocolOptions_OutlierDetection {
	if x != nil {
		return x.OutlierDetection
	}
	return nil
}

func (x *HttpProtocolOptions) GetRequestMirrorPolicies() []*v32.RouteAction_RequestMirrorPolicy {
	if x != nil {
		return x.RequestMirrorPolicies
	}
	return nil
}

func (x *HttpProtocolOptions) GetHashPolicy() []*v32.RouteAction_HashPolicy {
	if x != nil {
		return x.HashPolicy
	}
	return nil
}

func (x *HttpProtocolOptions) GetRetryPolicy() *v32.RetryPolicy {
	if x != nil {
		return x.RetryPolicy
	}
	return nil
}

type isHttpProtocolOptions_UpstreamProtocolOptions interface {
	isHttpProtocolOptions_UpstreamProtocolOptions()
}

type HttpProtocolOptions_ExplicitHttpConfig_ struct {
	// To explicitly configure either HTTP/1 or HTTP/2 (but not both!) use “explicit_http_config“.
	ExplicitHttpConfig *HttpProtocolOptions_ExplicitHttpConfig `protobuf:"bytes,3,opt,name=explicit_http_config,json=explicitHttpConfig,proto3,oneof"`
}

type HttpProtocolOptions_UseDownstreamProtocolConfig struct {
	// This allows switching on protocol based on what protocol the downstream
	// connection used.
	UseDownstreamProtocolConfig *HttpProtocolOptions_UseDownstreamHttpConfig `protobuf:"bytes,4,opt,name=use_downstream_protocol_config,json=useDownstreamProtocolConfig,proto3,oneof"`
}

type HttpProtocolOptions_AutoConfig struct {
	// This allows switching on protocol based on ALPN
	AutoConfig *HttpProtocolOptions_AutoHttpConfig `protobuf:"bytes,5,opt,name=auto_config,json=autoConfig,proto3,oneof"`
}

func (*HttpProtocolOptions_ExplicitHttpConfig_) isHttpProtocolOptions_UpstreamProtocolOptions() {}

func (*HttpProtocolOptions_UseDownstreamProtocolConfig) isHttpProtocolOptions_UpstreamProtocolOptions() {
}

func (*HttpProtocolOptions_AutoConfig) isHttpProtocolOptions_UpstreamProtocolOptions() {}

// If this is used, the cluster will only operate on one of the possible upstream protocols.
// Note that HTTP/2 or above should generally be used for upstream gRPC clusters.
type HttpProtocolOptions_ExplicitHttpConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to ProtocolConfig:
	//
	//	*HttpProtocolOptions_ExplicitHttpConfig_HttpProtocolOptions
	//	*HttpProtocolOptions_ExplicitHttpConfig_Http2ProtocolOptions
	//	*HttpProtocolOptions_ExplicitHttpConfig_Http3ProtocolOptions
	ProtocolConfig isHttpProtocolOptions_ExplicitHttpConfig_ProtocolConfig `protobuf_oneof:"protocol_config"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *HttpProtocolOptions_ExplicitHttpConfig) Reset() {
	*x = HttpProtocolOptions_ExplicitHttpConfig{}
	mi := &file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpProtocolOptions_ExplicitHttpConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpProtocolOptions_ExplicitHttpConfig) ProtoMessage() {}

func (x *HttpProtocolOptions_ExplicitHttpConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpProtocolOptions_ExplicitHttpConfig.ProtoReflect.Descriptor instead.
func (*HttpProtocolOptions_ExplicitHttpConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDescGZIP(), []int{0, 0}
}

func (x *HttpProtocolOptions_ExplicitHttpConfig) GetProtocolConfig() isHttpProtocolOptions_ExplicitHttpConfig_ProtocolConfig {
	if x != nil {
		return x.ProtocolConfig
	}
	return nil
}

func (x *HttpProtocolOptions_ExplicitHttpConfig) GetHttpProtocolOptions() *v3.Http1ProtocolOptions {
	if x != nil {
		if x, ok := x.ProtocolConfig.(*HttpProtocolOptions_ExplicitHttpConfig_HttpProtocolOptions); ok {
			return x.HttpProtocolOptions
		}
	}
	return nil
}

func (x *HttpProtocolOptions_ExplicitHttpConfig) GetHttp2ProtocolOptions() *v3.Http2ProtocolOptions {
	if x != nil {
		if x, ok := x.ProtocolConfig.(*HttpProtocolOptions_ExplicitHttpConfig_Http2ProtocolOptions); ok {
			return x.Http2ProtocolOptions
		}
	}
	return nil
}

func (x *HttpProtocolOptions_ExplicitHttpConfig) GetHttp3ProtocolOptions() *v3.Http3ProtocolOptions {
	if x != nil {
		if x, ok := x.ProtocolConfig.(*HttpProtocolOptions_ExplicitHttpConfig_Http3ProtocolOptions); ok {
			return x.Http3ProtocolOptions
		}
	}
	return nil
}

type isHttpProtocolOptions_ExplicitHttpConfig_ProtocolConfig interface {
	isHttpProtocolOptions_ExplicitHttpConfig_ProtocolConfig()
}

type HttpProtocolOptions_ExplicitHttpConfig_HttpProtocolOptions struct {
	HttpProtocolOptions *v3.Http1ProtocolOptions `protobuf:"bytes,1,opt,name=http_protocol_options,json=httpProtocolOptions,proto3,oneof"`
}

type HttpProtocolOptions_ExplicitHttpConfig_Http2ProtocolOptions struct {
	Http2ProtocolOptions *v3.Http2ProtocolOptions `protobuf:"bytes,2,opt,name=http2_protocol_options,json=http2ProtocolOptions,proto3,oneof"`
}

type HttpProtocolOptions_ExplicitHttpConfig_Http3ProtocolOptions struct {
	// .. warning::
	//
	//	QUIC upstream support is currently not ready for internet use.
	//	Please see :ref:`here <arch_overview_http3>` for details.
	Http3ProtocolOptions *v3.Http3ProtocolOptions `protobuf:"bytes,3,opt,name=http3_protocol_options,json=http3ProtocolOptions,proto3,oneof"`
}

func (*HttpProtocolOptions_ExplicitHttpConfig_HttpProtocolOptions) isHttpProtocolOptions_ExplicitHttpConfig_ProtocolConfig() {
}

func (*HttpProtocolOptions_ExplicitHttpConfig_Http2ProtocolOptions) isHttpProtocolOptions_ExplicitHttpConfig_ProtocolConfig() {
}

func (*HttpProtocolOptions_ExplicitHttpConfig_Http3ProtocolOptions) isHttpProtocolOptions_ExplicitHttpConfig_ProtocolConfig() {
}

// If this is used, the cluster can use either of the configured protocols, and
// will use whichever protocol was used by the downstream connection.
//
// If HTTP/3 is configured for downstream and not configured for upstream,
// HTTP/3 requests will fail over to HTTP/2.
type HttpProtocolOptions_UseDownstreamHttpConfig struct {
	state                protoimpl.MessageState   `protogen:"open.v1"`
	HttpProtocolOptions  *v3.Http1ProtocolOptions `protobuf:"bytes,1,opt,name=http_protocol_options,json=httpProtocolOptions,proto3" json:"http_protocol_options,omitempty"`
	Http2ProtocolOptions *v3.Http2ProtocolOptions `protobuf:"bytes,2,opt,name=http2_protocol_options,json=http2ProtocolOptions,proto3" json:"http2_protocol_options,omitempty"`
	// .. warning::
	//
	//	QUIC upstream support is currently not ready for internet use.
	//	Please see :ref:`here <arch_overview_http3>` for details.
	Http3ProtocolOptions *v3.Http3ProtocolOptions `protobuf:"bytes,3,opt,name=http3_protocol_options,json=http3ProtocolOptions,proto3" json:"http3_protocol_options,omitempty"`
	unknownFields        protoimpl.UnknownFields
	sizeCache            protoimpl.SizeCache
}

func (x *HttpProtocolOptions_UseDownstreamHttpConfig) Reset() {
	*x = HttpProtocolOptions_UseDownstreamHttpConfig{}
	mi := &file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpProtocolOptions_UseDownstreamHttpConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpProtocolOptions_UseDownstreamHttpConfig) ProtoMessage() {}

func (x *HttpProtocolOptions_UseDownstreamHttpConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpProtocolOptions_UseDownstreamHttpConfig.ProtoReflect.Descriptor instead.
func (*HttpProtocolOptions_UseDownstreamHttpConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDescGZIP(), []int{0, 1}
}

func (x *HttpProtocolOptions_UseDownstreamHttpConfig) GetHttpProtocolOptions() *v3.Http1ProtocolOptions {
	if x != nil {
		return x.HttpProtocolOptions
	}
	return nil
}

func (x *HttpProtocolOptions_UseDownstreamHttpConfig) GetHttp2ProtocolOptions() *v3.Http2ProtocolOptions {
	if x != nil {
		return x.Http2ProtocolOptions
	}
	return nil
}

func (x *HttpProtocolOptions_UseDownstreamHttpConfig) GetHttp3ProtocolOptions() *v3.Http3ProtocolOptions {
	if x != nil {
		return x.Http3ProtocolOptions
	}
	return nil
}

// If this is used, the cluster can use either HTTP/1 or HTTP/2, and will use whichever
// protocol is negotiated by ALPN with the upstream.
// Clusters configured with “AutoHttpConfig“ will use the highest available
// protocol; HTTP/2 if supported, otherwise HTTP/1.
// If the upstream does not support ALPN, “AutoHttpConfig“ will fail over to HTTP/1.
// This can only be used with transport sockets which support ALPN. Using a
// transport socket which does not support ALPN will result in configuration
// failure. The transport layer may be configured with custom ALPN, but the default ALPN
// for the cluster (or if custom ALPN fails) will be "h2,http/1.1".
type HttpProtocolOptions_AutoHttpConfig struct {
	state                protoimpl.MessageState   `protogen:"open.v1"`
	HttpProtocolOptions  *v3.Http1ProtocolOptions `protobuf:"bytes,1,opt,name=http_protocol_options,json=httpProtocolOptions,proto3" json:"http_protocol_options,omitempty"`
	Http2ProtocolOptions *v3.Http2ProtocolOptions `protobuf:"bytes,2,opt,name=http2_protocol_options,json=http2ProtocolOptions,proto3" json:"http2_protocol_options,omitempty"`
	// Unlike HTTP/1 and HTTP/2, HTTP/3 will not be configured unless it is
	// present, and (soon) only if there is an indication of server side
	// support.
	// See :ref:`here <arch_overview_http3_upstream>` for more information on
	// when HTTP/3 will be used, and when Envoy will fail over to TCP.
	//
	// .. warning::
	//
	//	QUIC upstream support is currently not ready for internet use.
	//	Please see :ref:`here <arch_overview_http3>` for details.
	Http3ProtocolOptions *v3.Http3ProtocolOptions `protobuf:"bytes,3,opt,name=http3_protocol_options,json=http3ProtocolOptions,proto3" json:"http3_protocol_options,omitempty"`
	// The presence of alternate protocols cache options causes the use of the
	// alternate protocols cache, which is responsible for parsing and caching
	// HTTP Alt-Svc headers. This enables the use of HTTP/3 for origins that
	// advertise supporting it.
	//
	// .. note::
	//
	//	This is required when HTTP/3 is enabled.
	AlternateProtocolsCacheOptions *v3.AlternateProtocolsCacheOptions `protobuf:"bytes,4,opt,name=alternate_protocols_cache_options,json=alternateProtocolsCacheOptions,proto3" json:"alternate_protocols_cache_options,omitempty"`
	unknownFields                  protoimpl.UnknownFields
	sizeCache                      protoimpl.SizeCache
}

func (x *HttpProtocolOptions_AutoHttpConfig) Reset() {
	*x = HttpProtocolOptions_AutoHttpConfig{}
	mi := &file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpProtocolOptions_AutoHttpConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpProtocolOptions_AutoHttpConfig) ProtoMessage() {}

func (x *HttpProtocolOptions_AutoHttpConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpProtocolOptions_AutoHttpConfig.ProtoReflect.Descriptor instead.
func (*HttpProtocolOptions_AutoHttpConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDescGZIP(), []int{0, 2}
}

func (x *HttpProtocolOptions_AutoHttpConfig) GetHttpProtocolOptions() *v3.Http1ProtocolOptions {
	if x != nil {
		return x.HttpProtocolOptions
	}
	return nil
}

func (x *HttpProtocolOptions_AutoHttpConfig) GetHttp2ProtocolOptions() *v3.Http2ProtocolOptions {
	if x != nil {
		return x.Http2ProtocolOptions
	}
	return nil
}

func (x *HttpProtocolOptions_AutoHttpConfig) GetHttp3ProtocolOptions() *v3.Http3ProtocolOptions {
	if x != nil {
		return x.Http3ProtocolOptions
	}
	return nil
}

func (x *HttpProtocolOptions_AutoHttpConfig) GetAlternateProtocolsCacheOptions() *v3.AlternateProtocolsCacheOptions {
	if x != nil {
		return x.AlternateProtocolsCacheOptions
	}
	return nil
}

type HttpProtocolOptions_OutlierDetection struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// If specified, only responses matching the matcher will be treated by outlier detection as errors.
	// If not specified, only 5xx codes are treated by outlier detection as errors.
	ErrorMatcher  *v33.MatchPredicate `protobuf:"bytes,1,opt,name=error_matcher,json=errorMatcher,proto3" json:"error_matcher,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HttpProtocolOptions_OutlierDetection) Reset() {
	*x = HttpProtocolOptions_OutlierDetection{}
	mi := &file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HttpProtocolOptions_OutlierDetection) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HttpProtocolOptions_OutlierDetection) ProtoMessage() {}

func (x *HttpProtocolOptions_OutlierDetection) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HttpProtocolOptions_OutlierDetection.ProtoReflect.Descriptor instead.
func (*HttpProtocolOptions_OutlierDetection) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDescGZIP(), []int{0, 3}
}

func (x *HttpProtocolOptions_OutlierDetection) GetErrorMatcher() *v33.MatchPredicate {
	if x != nil {
		return x.ErrorMatcher
	}
	return nil
}

var File_envoy_extensions_upstreams_http_v3_http_protocol_options_proto protoreflect.FileDescriptor

const file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDesc = "" +
	"\n" +
	">envoy/extensions/upstreams/http/v3/http_protocol_options.proto\x12\"envoy.extensions.upstreams.http.v3\x1a,envoy/config/common/matcher/v3/matcher.proto\x1a$envoy/config/core/v3/extension.proto\x1a#envoy/config/core/v3/protocol.proto\x1a,envoy/config/route/v3/route_components.proto\x1aYenvoy/extensions/filters/network/http_connection_manager/v3/http_connection_manager.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xb3\x13\n" +
	"\x13HttpProtocolOptions\x12j\n" +
	"\x1ccommon_http_protocol_options\x18\x01 \x01(\v2).envoy.config.core.v3.HttpProtocolOptionsR\x19commonHttpProtocolOptions\x12v\n" +
	"\x1eupstream_http_protocol_options\x18\x02 \x01(\v21.envoy.config.core.v3.UpstreamHttpProtocolOptionsR\x1bupstreamHttpProtocolOptions\x12~\n" +
	"\x14explicit_http_config\x18\x03 \x01(\v2J.envoy.extensions.upstreams.http.v3.HttpProtocolOptions.ExplicitHttpConfigH\x00R\x12explicitHttpConfig\x12\x96\x01\n" +
	"\x1euse_downstream_protocol_config\x18\x04 \x01(\v2O.envoy.extensions.upstreams.http.v3.HttpProtocolOptions.UseDownstreamHttpConfigH\x00R\x1buseDownstreamProtocolConfig\x12i\n" +
	"\vauto_config\x18\x05 \x01(\v2F.envoy.extensions.upstreams.http.v3.HttpProtocolOptions.AutoHttpConfigH\x00R\n" +
	"autoConfig\x12j\n" +
	"\fhttp_filters\x18\x06 \x03(\v2G.envoy.extensions.filters.network.http_connection_manager.v3.HttpFilterR\vhttpFilters\x12d\n" +
	"\x18header_validation_config\x18\a \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigR\x16headerValidationConfig\x12u\n" +
	"\x11outlier_detection\x18\b \x01(\v2H.envoy.extensions.upstreams.http.v3.HttpProtocolOptions.OutlierDetectionR\x10outlierDetection\x12n\n" +
	"\x17request_mirror_policies\x18\t \x03(\v26.envoy.config.route.v3.RouteAction.RequestMirrorPolicyR\x15requestMirrorPolicies\x12N\n" +
	"\vhash_policy\x18\n" +
	" \x03(\v2-.envoy.config.route.v3.RouteAction.HashPolicyR\n" +
	"hashPolicy\x12E\n" +
	"\fretry_policy\x18\v \x01(\v2\".envoy.config.route.v3.RetryPolicyR\vretryPolicy\x1a\xd6\x02\n" +
	"\x12ExplicitHttpConfig\x12`\n" +
	"\x15http_protocol_options\x18\x01 \x01(\v2*.envoy.config.core.v3.Http1ProtocolOptionsH\x00R\x13httpProtocolOptions\x12b\n" +
	"\x16http2_protocol_options\x18\x02 \x01(\v2*.envoy.config.core.v3.Http2ProtocolOptionsH\x00R\x14http2ProtocolOptions\x12b\n" +
	"\x16http3_protocol_options\x18\x03 \x01(\v2*.envoy.config.core.v3.Http3ProtocolOptionsH\x00R\x14http3ProtocolOptionsB\x16\n" +
	"\x0fprotocol_config\x12\x03\xf8B\x01\x1a\xbd\x02\n" +
	"\x17UseDownstreamHttpConfig\x12^\n" +
	"\x15http_protocol_options\x18\x01 \x01(\v2*.envoy.config.core.v3.Http1ProtocolOptionsR\x13httpProtocolOptions\x12`\n" +
	"\x16http2_protocol_options\x18\x02 \x01(\v2*.envoy.config.core.v3.Http2ProtocolOptionsR\x14http2ProtocolOptions\x12`\n" +
	"\x16http3_protocol_options\x18\x03 \x01(\v2*.envoy.config.core.v3.Http3ProtocolOptionsR\x14http3ProtocolOptions\x1a\xb5\x03\n" +
	"\x0eAutoHttpConfig\x12^\n" +
	"\x15http_protocol_options\x18\x01 \x01(\v2*.envoy.config.core.v3.Http1ProtocolOptionsR\x13httpProtocolOptions\x12`\n" +
	"\x16http2_protocol_options\x18\x02 \x01(\v2*.envoy.config.core.v3.Http2ProtocolOptionsR\x14http2ProtocolOptions\x12`\n" +
	"\x16http3_protocol_options\x18\x03 \x01(\v2*.envoy.config.core.v3.Http3ProtocolOptionsR\x14http3ProtocolOptions\x12\x7f\n" +
	"!alternate_protocols_cache_options\x18\x04 \x01(\v24.envoy.config.core.v3.AlternateProtocolsCacheOptionsR\x1ealternateProtocolsCacheOptions\x1aq\n" +
	"\x10OutlierDetection\x12]\n" +
	"\rerror_matcher\x18\x01 \x01(\v2..envoy.config.common.matcher.v3.MatchPredicateB\b\xfaB\x05\x8a\x01\x02\x10\x01R\ferrorMatcherB \n" +
	"\x19upstream_protocol_options\x12\x03\xf8B\x01B\xa8\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"0io.envoyproxy.envoy.extensions.upstreams.http.v3B\x18HttpProtocolOptionsProtoP\x01ZPgithub.com/envoyproxy/go-control-plane/envoy/extensions/upstreams/http/v3;httpv3b\x06proto3"

var (
	file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDescOnce sync.Once
	file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDescData []byte
)

func file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDescGZIP() []byte {
	file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDesc), len(file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDesc)))
	})
	return file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDescData
}

var file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes = make([]protoimpl.MessageInfo, 5)
var file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_goTypes = []any{
	(*HttpProtocolOptions)(nil),                         // 0: envoy.extensions.upstreams.http.v3.HttpProtocolOptions
	(*HttpProtocolOptions_ExplicitHttpConfig)(nil),      // 1: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.ExplicitHttpConfig
	(*HttpProtocolOptions_UseDownstreamHttpConfig)(nil), // 2: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.UseDownstreamHttpConfig
	(*HttpProtocolOptions_AutoHttpConfig)(nil),          // 3: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.AutoHttpConfig
	(*HttpProtocolOptions_OutlierDetection)(nil),        // 4: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.OutlierDetection
	(*v3.HttpProtocolOptions)(nil),                      // 5: envoy.config.core.v3.HttpProtocolOptions
	(*v3.UpstreamHttpProtocolOptions)(nil),              // 6: envoy.config.core.v3.UpstreamHttpProtocolOptions
	(*v31.HttpFilter)(nil),                              // 7: envoy.extensions.filters.network.http_connection_manager.v3.HttpFilter
	(*v3.TypedExtensionConfig)(nil),                     // 8: envoy.config.core.v3.TypedExtensionConfig
	(*v32.RouteAction_RequestMirrorPolicy)(nil),         // 9: envoy.config.route.v3.RouteAction.RequestMirrorPolicy
	(*v32.RouteAction_HashPolicy)(nil),                  // 10: envoy.config.route.v3.RouteAction.HashPolicy
	(*v32.RetryPolicy)(nil),                             // 11: envoy.config.route.v3.RetryPolicy
	(*v3.Http1ProtocolOptions)(nil),                     // 12: envoy.config.core.v3.Http1ProtocolOptions
	(*v3.Http2ProtocolOptions)(nil),                     // 13: envoy.config.core.v3.Http2ProtocolOptions
	(*v3.Http3ProtocolOptions)(nil),                     // 14: envoy.config.core.v3.Http3ProtocolOptions
	(*v3.AlternateProtocolsCacheOptions)(nil),           // 15: envoy.config.core.v3.AlternateProtocolsCacheOptions
	(*v33.MatchPredicate)(nil),                          // 16: envoy.config.common.matcher.v3.MatchPredicate
}
var file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_depIdxs = []int32{
	5,  // 0: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.common_http_protocol_options:type_name -> envoy.config.core.v3.HttpProtocolOptions
	6,  // 1: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.upstream_http_protocol_options:type_name -> envoy.config.core.v3.UpstreamHttpProtocolOptions
	1,  // 2: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.explicit_http_config:type_name -> envoy.extensions.upstreams.http.v3.HttpProtocolOptions.ExplicitHttpConfig
	2,  // 3: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.use_downstream_protocol_config:type_name -> envoy.extensions.upstreams.http.v3.HttpProtocolOptions.UseDownstreamHttpConfig
	3,  // 4: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.auto_config:type_name -> envoy.extensions.upstreams.http.v3.HttpProtocolOptions.AutoHttpConfig
	7,  // 5: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.http_filters:type_name -> envoy.extensions.filters.network.http_connection_manager.v3.HttpFilter
	8,  // 6: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.header_validation_config:type_name -> envoy.config.core.v3.TypedExtensionConfig
	4,  // 7: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.outlier_detection:type_name -> envoy.extensions.upstreams.http.v3.HttpProtocolOptions.OutlierDetection
	9,  // 8: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.request_mirror_policies:type_name -> envoy.config.route.v3.RouteAction.RequestMirrorPolicy
	10, // 9: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.hash_policy:type_name -> envoy.config.route.v3.RouteAction.HashPolicy
	11, // 10: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.retry_policy:type_name -> envoy.config.route.v3.RetryPolicy
	12, // 11: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.ExplicitHttpConfig.http_protocol_options:type_name -> envoy.config.core.v3.Http1ProtocolOptions
	13, // 12: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.ExplicitHttpConfig.http2_protocol_options:type_name -> envoy.config.core.v3.Http2ProtocolOptions
	14, // 13: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.ExplicitHttpConfig.http3_protocol_options:type_name -> envoy.config.core.v3.Http3ProtocolOptions
	12, // 14: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.UseDownstreamHttpConfig.http_protocol_options:type_name -> envoy.config.core.v3.Http1ProtocolOptions
	13, // 15: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.UseDownstreamHttpConfig.http2_protocol_options:type_name -> envoy.config.core.v3.Http2ProtocolOptions
	14, // 16: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.UseDownstreamHttpConfig.http3_protocol_options:type_name -> envoy.config.core.v3.Http3ProtocolOptions
	12, // 17: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.AutoHttpConfig.http_protocol_options:type_name -> envoy.config.core.v3.Http1ProtocolOptions
	13, // 18: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.AutoHttpConfig.http2_protocol_options:type_name -> envoy.config.core.v3.Http2ProtocolOptions
	14, // 19: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.AutoHttpConfig.http3_protocol_options:type_name -> envoy.config.core.v3.Http3ProtocolOptions
	15, // 20: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.AutoHttpConfig.alternate_protocols_cache_options:type_name -> envoy.config.core.v3.AlternateProtocolsCacheOptions
	16, // 21: envoy.extensions.upstreams.http.v3.HttpProtocolOptions.OutlierDetection.error_matcher:type_name -> envoy.config.common.matcher.v3.MatchPredicate
	22, // [22:22] is the sub-list for method output_type
	22, // [22:22] is the sub-list for method input_type
	22, // [22:22] is the sub-list for extension type_name
	22, // [22:22] is the sub-list for extension extendee
	0,  // [0:22] is the sub-list for field type_name
}

func init() { file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_init() }
func file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_init() {
	if File_envoy_extensions_upstreams_http_v3_http_protocol_options_proto != nil {
		return
	}
	file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[0].OneofWrappers = []any{
		(*HttpProtocolOptions_ExplicitHttpConfig_)(nil),
		(*HttpProtocolOptions_UseDownstreamProtocolConfig)(nil),
		(*HttpProtocolOptions_AutoConfig)(nil),
	}
	file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes[1].OneofWrappers = []any{
		(*HttpProtocolOptions_ExplicitHttpConfig_HttpProtocolOptions)(nil),
		(*HttpProtocolOptions_ExplicitHttpConfig_Http2ProtocolOptions)(nil),
		(*HttpProtocolOptions_ExplicitHttpConfig_Http3ProtocolOptions)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDesc), len(file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   5,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_msgTypes,
	}.Build()
	File_envoy_extensions_upstreams_http_v3_http_protocol_options_proto = out.File
	file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_goTypes = nil
	file_envoy_extensions_upstreams_http_v3_http_protocol_options_proto_depIdxs = nil
}
