// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/listener/original_src/v3/original_src.proto

package original_srcv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The Original Src filter binds upstream connections to the original source address determined
// for the connection. This address could come from something like the Proxy Protocol filter, or it
// could come from trusted http headers.
type OriginalSrc struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Whether to bind the port to the one used in the original downstream connection.
	// [#not-implemented-hide:]
	BindPort bool `protobuf:"varint,1,opt,name=bind_port,json=bindPort,proto3" json:"bind_port,omitempty"`
	// Sets the SO_MARK option on the upstream connection's socket to the provided value. Used to
	// ensure that non-local addresses may be routed back through envoy when binding to the original
	// source address. The option will not be applied if the mark is 0.
	Mark          uint32 `protobuf:"varint,2,opt,name=mark,proto3" json:"mark,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *OriginalSrc) Reset() {
	*x = OriginalSrc{}
	mi := &file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OriginalSrc) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OriginalSrc) ProtoMessage() {}

func (x *OriginalSrc) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OriginalSrc.ProtoReflect.Descriptor instead.
func (*OriginalSrc) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDescGZIP(), []int{0}
}

func (x *OriginalSrc) GetBindPort() bool {
	if x != nil {
		return x.BindPort
	}
	return false
}

func (x *OriginalSrc) GetMark() uint32 {
	if x != nil {
		return x.Mark
	}
	return 0
}

var File_envoy_extensions_filters_listener_original_src_v3_original_src_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDesc = "" +
	"\n" +
	"Denvoy/extensions/filters/listener/original_src/v3/original_src.proto\x121envoy.extensions.filters.listener.original_src.v3\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\x85\x01\n" +
	"\vOriginalSrc\x12\x1b\n" +
	"\tbind_port\x18\x01 \x01(\bR\bbindPort\x12\x12\n" +
	"\x04mark\x18\x02 \x01(\rR\x04mark:E\x9aň\x1e@\n" +
	">envoy.config.filter.listener.original_src.v2alpha1.OriginalSrcB\xc6\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"?io.envoyproxy.envoy.extensions.filters.listener.original_src.v3B\x10OriginalSrcProtoP\x01Zggithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/listener/original_src/v3;original_srcv3b\x06proto3"

var (
	file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDescData []byte
)

func file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDesc), len(file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDescData
}

var file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_goTypes = []any{
	(*OriginalSrc)(nil), // 0: envoy.extensions.filters.listener.original_src.v3.OriginalSrc
}
var file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_init() }
func file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_init() {
	if File_envoy_extensions_filters_listener_original_src_v3_original_src_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDesc), len(file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_listener_original_src_v3_original_src_proto = out.File
	file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_goTypes = nil
	file_envoy_extensions_filters_listener_original_src_v3_original_src_proto_depIdxs = nil
}
