// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/service/endpoint/v3/leds.proto

package endpointv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/service/discovery/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#not-implemented-hide:] Not configuration. Workaround c++ protobuf issue with importing
// services: https://github.com/google/protobuf/issues/4221 and protoxform to upgrade the file.
type LedsDummy struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *LedsDummy) Reset() {
	*x = LedsDummy{}
	mi := &file_envoy_service_endpoint_v3_leds_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *LedsDummy) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*LedsDummy) ProtoMessage() {}

func (x *LedsDummy) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_endpoint_v3_leds_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use LedsDummy.ProtoReflect.Descriptor instead.
func (*LedsDummy) Descriptor() ([]byte, []int) {
	return file_envoy_service_endpoint_v3_leds_proto_rawDescGZIP(), []int{0}
}

var File_envoy_service_endpoint_v3_leds_proto protoreflect.FileDescriptor

const file_envoy_service_endpoint_v3_leds_proto_rawDesc = "" +
	"\n" +
	"$envoy/service/endpoint/v3/leds.proto\x12\x19envoy.service.endpoint.v3\x1a*envoy/service/discovery/v3/discovery.proto\x1a envoy/annotations/resource.proto\x1a\x1dudpa/annotations/status.proto\"\v\n" +
	"\tLedsDummy2\xd7\x01\n" +
	" LocalityEndpointDiscoveryService\x12\x85\x01\n" +
	"\x16DeltaLocalityEndpoints\x121.envoy.service.discovery.v3.DeltaDiscoveryRequest\x1a2.envoy.service.discovery.v3.DeltaDiscoveryResponse\"\x00(\x010\x01\x1a+\x8a\xa4\x96\xf3\a%\n" +
	"#envoy.config.endpoint.v3.LbEndpointB\x8b\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"'io.envoyproxy.envoy.service.endpoint.v3B\tLedsProtoP\x01ZKgithub.com/envoyproxy/go-control-plane/envoy/service/endpoint/v3;endpointv3b\x06proto3"

var (
	file_envoy_service_endpoint_v3_leds_proto_rawDescOnce sync.Once
	file_envoy_service_endpoint_v3_leds_proto_rawDescData []byte
)

func file_envoy_service_endpoint_v3_leds_proto_rawDescGZIP() []byte {
	file_envoy_service_endpoint_v3_leds_proto_rawDescOnce.Do(func() {
		file_envoy_service_endpoint_v3_leds_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_service_endpoint_v3_leds_proto_rawDesc), len(file_envoy_service_endpoint_v3_leds_proto_rawDesc)))
	})
	return file_envoy_service_endpoint_v3_leds_proto_rawDescData
}

var file_envoy_service_endpoint_v3_leds_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_service_endpoint_v3_leds_proto_goTypes = []any{
	(*LedsDummy)(nil),                 // 0: envoy.service.endpoint.v3.LedsDummy
	(*v3.DeltaDiscoveryRequest)(nil),  // 1: envoy.service.discovery.v3.DeltaDiscoveryRequest
	(*v3.DeltaDiscoveryResponse)(nil), // 2: envoy.service.discovery.v3.DeltaDiscoveryResponse
}
var file_envoy_service_endpoint_v3_leds_proto_depIdxs = []int32{
	1, // 0: envoy.service.endpoint.v3.LocalityEndpointDiscoveryService.DeltaLocalityEndpoints:input_type -> envoy.service.discovery.v3.DeltaDiscoveryRequest
	2, // 1: envoy.service.endpoint.v3.LocalityEndpointDiscoveryService.DeltaLocalityEndpoints:output_type -> envoy.service.discovery.v3.DeltaDiscoveryResponse
	1, // [1:2] is the sub-list for method output_type
	0, // [0:1] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_service_endpoint_v3_leds_proto_init() }
func file_envoy_service_endpoint_v3_leds_proto_init() {
	if File_envoy_service_endpoint_v3_leds_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_service_endpoint_v3_leds_proto_rawDesc), len(file_envoy_service_endpoint_v3_leds_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_envoy_service_endpoint_v3_leds_proto_goTypes,
		DependencyIndexes: file_envoy_service_endpoint_v3_leds_proto_depIdxs,
		MessageInfos:      file_envoy_service_endpoint_v3_leds_proto_msgTypes,
	}.Build()
	File_envoy_service_endpoint_v3_leds_proto = out.File
	file_envoy_service_endpoint_v3_leds_proto_goTypes = nil
	file_envoy_service_endpoint_v3_leds_proto_depIdxs = nil
}
