// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/rate_limit_descriptors/expr/v3/expr.proto

package exprv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	v1alpha1 "google.golang.org/genproto/googleapis/api/expr/v1alpha1"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The following descriptor entry is appended with a value computed
// from a symbolic Common Expression Language expression.
// See :ref:`attributes <arch_overview_attributes>` for the set of
// available attributes.
//
// .. code-block:: cpp
//
//	("<descriptor_key>", "<expression_value>")
type Descriptor struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The key to use in the descriptor entry.
	DescriptorKey string `protobuf:"bytes,1,opt,name=descriptor_key,json=descriptorKey,proto3" json:"descriptor_key,omitempty"`
	// If set to true, Envoy skips the descriptor if the expression evaluates to an error.
	// By default, the rate limit is not applied when an expression produces an error.
	SkipIfError bool `protobuf:"varint,2,opt,name=skip_if_error,json=skipIfError,proto3" json:"skip_if_error,omitempty"`
	// Types that are valid to be assigned to ExprSpecifier:
	//
	//	*Descriptor_Text
	//	*Descriptor_Parsed
	ExprSpecifier isDescriptor_ExprSpecifier `protobuf_oneof:"expr_specifier"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Descriptor) Reset() {
	*x = Descriptor{}
	mi := &file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Descriptor) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Descriptor) ProtoMessage() {}

func (x *Descriptor) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Descriptor.ProtoReflect.Descriptor instead.
func (*Descriptor) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDescGZIP(), []int{0}
}

func (x *Descriptor) GetDescriptorKey() string {
	if x != nil {
		return x.DescriptorKey
	}
	return ""
}

func (x *Descriptor) GetSkipIfError() bool {
	if x != nil {
		return x.SkipIfError
	}
	return false
}

func (x *Descriptor) GetExprSpecifier() isDescriptor_ExprSpecifier {
	if x != nil {
		return x.ExprSpecifier
	}
	return nil
}

func (x *Descriptor) GetText() string {
	if x != nil {
		if x, ok := x.ExprSpecifier.(*Descriptor_Text); ok {
			return x.Text
		}
	}
	return ""
}

func (x *Descriptor) GetParsed() *v1alpha1.Expr {
	if x != nil {
		if x, ok := x.ExprSpecifier.(*Descriptor_Parsed); ok {
			return x.Parsed
		}
	}
	return nil
}

type isDescriptor_ExprSpecifier interface {
	isDescriptor_ExprSpecifier()
}

type Descriptor_Text struct {
	// Expression in a text form, e.g. "connection.requested_server_name".
	Text string `protobuf:"bytes,3,opt,name=text,proto3,oneof"`
}

type Descriptor_Parsed struct {
	// Parsed expression in AST form.
	Parsed *v1alpha1.Expr `protobuf:"bytes,4,opt,name=parsed,proto3,oneof"`
}

func (*Descriptor_Text) isDescriptor_ExprSpecifier() {}

func (*Descriptor_Parsed) isDescriptor_ExprSpecifier() {}

var File_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto protoreflect.FileDescriptor

const file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDesc = "" +
	"\n" +
	":envoy/extensions/rate_limit_descriptors/expr/v3/expr.proto\x12/envoy.extensions.rate_limit_descriptors.expr.v3\x1a%google/api/expr/v1alpha1/syntax.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xcb\x01\n" +
	"\n" +
	"Descriptor\x12.\n" +
	"\x0edescriptor_key\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\rdescriptorKey\x12\"\n" +
	"\rskip_if_error\x18\x02 \x01(\bR\vskipIfError\x12\x1d\n" +
	"\x04text\x18\x03 \x01(\tB\a\xfaB\x04r\x02\x10\x01H\x00R\x04text\x128\n" +
	"\x06parsed\x18\x04 \x01(\v2\x1e.google.api.expr.v1alpha1.ExprH\x00R\x06parsedB\x10\n" +
	"\x0eexpr_specifierB\xb3\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"=io.envoyproxy.envoy.extensions.rate_limit_descriptors.expr.v3B\tExprProtoP\x01Z]github.com/envoyproxy/go-control-plane/envoy/extensions/rate_limit_descriptors/expr/v3;exprv3b\x06proto3"

var (
	file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDescOnce sync.Once
	file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDescData []byte
)

func file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDescGZIP() []byte {
	file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDesc), len(file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDesc)))
	})
	return file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDescData
}

var file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_goTypes = []any{
	(*Descriptor)(nil),    // 0: envoy.extensions.rate_limit_descriptors.expr.v3.Descriptor
	(*v1alpha1.Expr)(nil), // 1: google.api.expr.v1alpha1.Expr
}
var file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.rate_limit_descriptors.expr.v3.Descriptor.parsed:type_name -> google.api.expr.v1alpha1.Expr
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_init() }
func file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_init() {
	if File_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto != nil {
		return
	}
	file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_msgTypes[0].OneofWrappers = []any{
		(*Descriptor_Text)(nil),
		(*Descriptor_Parsed)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDesc), len(file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_msgTypes,
	}.Build()
	File_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto = out.File
	file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_goTypes = nil
	file_envoy_extensions_rate_limit_descriptors_expr_v3_expr_proto_depIdxs = nil
}
