// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/service/auth/v3/external_auth.proto

package authv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v31 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/type/v3"
	status "google.golang.org/genproto/googleapis/rpc/status"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	structpb "google.golang.org/protobuf/types/known/structpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type CheckRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The request attributes.
	Attributes    *AttributeContext `protobuf:"bytes,1,opt,name=attributes,proto3" json:"attributes,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CheckRequest) Reset() {
	*x = CheckRequest{}
	mi := &file_envoy_service_auth_v3_external_auth_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CheckRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CheckRequest) ProtoMessage() {}

func (x *CheckRequest) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_auth_v3_external_auth_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CheckRequest.ProtoReflect.Descriptor instead.
func (*CheckRequest) Descriptor() ([]byte, []int) {
	return file_envoy_service_auth_v3_external_auth_proto_rawDescGZIP(), []int{0}
}

func (x *CheckRequest) GetAttributes() *AttributeContext {
	if x != nil {
		return x.Attributes
	}
	return nil
}

// HTTP attributes for a denied response.
type DeniedHttpResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// This field allows the authorization service to send an HTTP response status code to the
	// downstream client. If not set, Envoy sends “403 Forbidden“ HTTP status code by default.
	Status *v3.HttpStatus `protobuf:"bytes,1,opt,name=status,proto3" json:"status,omitempty"`
	// This field allows the authorization service to send HTTP response headers
	// to the downstream client. Note that the :ref:`append field in HeaderValueOption <envoy_v3_api_field_config.core.v3.HeaderValueOption.append>` defaults to
	// false when used in this message.
	Headers []*v31.HeaderValueOption `protobuf:"bytes,2,rep,name=headers,proto3" json:"headers,omitempty"`
	// This field allows the authorization service to send a response body data
	// to the downstream client.
	Body          string `protobuf:"bytes,3,opt,name=body,proto3" json:"body,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DeniedHttpResponse) Reset() {
	*x = DeniedHttpResponse{}
	mi := &file_envoy_service_auth_v3_external_auth_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DeniedHttpResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DeniedHttpResponse) ProtoMessage() {}

func (x *DeniedHttpResponse) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_auth_v3_external_auth_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DeniedHttpResponse.ProtoReflect.Descriptor instead.
func (*DeniedHttpResponse) Descriptor() ([]byte, []int) {
	return file_envoy_service_auth_v3_external_auth_proto_rawDescGZIP(), []int{1}
}

func (x *DeniedHttpResponse) GetStatus() *v3.HttpStatus {
	if x != nil {
		return x.Status
	}
	return nil
}

func (x *DeniedHttpResponse) GetHeaders() []*v31.HeaderValueOption {
	if x != nil {
		return x.Headers
	}
	return nil
}

func (x *DeniedHttpResponse) GetBody() string {
	if x != nil {
		return x.Body
	}
	return ""
}

// HTTP attributes for an OK response.
// [#next-free-field: 9]
type OkHttpResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// HTTP entity headers in addition to the original request headers. This allows the authorization
	// service to append, to add or to override headers from the original request before
	// dispatching it to the upstream. Note that the :ref:`append field in HeaderValueOption <envoy_v3_api_field_config.core.v3.HeaderValueOption.append>` defaults to
	// false when used in this message. By setting the “append“ field to “true“,
	// the filter will append the correspondent header value to the matched request header.
	// By leaving “append“ as false, the filter will either add a new header, or override an existing
	// one if there is a match.
	Headers []*v31.HeaderValueOption `protobuf:"bytes,2,rep,name=headers,proto3" json:"headers,omitempty"`
	// HTTP entity headers to remove from the original request before dispatching
	// it to the upstream. This allows the authorization service to act on auth
	// related headers (like “Authorization“), process them, and consume them.
	// Under this model, the upstream will either receive the request (if it's
	// authorized) or not receive it (if it's not), but will not see headers
	// containing authorization credentials.
	//
	// Pseudo headers (such as “:authority“, “:method“, “:path“ etc), as well as
	// the header “Host“, may not be removed as that would make the request
	// malformed. If mentioned in “headers_to_remove“ these special headers will
	// be ignored.
	//
	// When using the HTTP service this must instead be set by the HTTP
	// authorization service as a comma separated list like so:
	// “x-envoy-auth-headers-to-remove: one-auth-header, another-auth-header“.
	HeadersToRemove []string `protobuf:"bytes,5,rep,name=headers_to_remove,json=headersToRemove,proto3" json:"headers_to_remove,omitempty"`
	// This field has been deprecated in favor of :ref:`CheckResponse.dynamic_metadata
	// <envoy_v3_api_field_service.auth.v3.CheckResponse.dynamic_metadata>`. Until it is removed,
	// setting this field overrides :ref:`CheckResponse.dynamic_metadata
	// <envoy_v3_api_field_service.auth.v3.CheckResponse.dynamic_metadata>`.
	//
	// Deprecated: Marked as deprecated in envoy/service/auth/v3/external_auth.proto.
	DynamicMetadata *structpb.Struct `protobuf:"bytes,3,opt,name=dynamic_metadata,json=dynamicMetadata,proto3" json:"dynamic_metadata,omitempty"`
	// This field allows the authorization service to send HTTP response headers
	// to the downstream client on success. Note that the :ref:`append field in HeaderValueOption <envoy_v3_api_field_config.core.v3.HeaderValueOption.append>`
	// defaults to false when used in this message.
	ResponseHeadersToAdd []*v31.HeaderValueOption `protobuf:"bytes,6,rep,name=response_headers_to_add,json=responseHeadersToAdd,proto3" json:"response_headers_to_add,omitempty"`
	// This field allows the authorization service to set (and overwrite) query
	// string parameters on the original request before it is sent upstream.
	QueryParametersToSet []*v31.QueryParameter `protobuf:"bytes,7,rep,name=query_parameters_to_set,json=queryParametersToSet,proto3" json:"query_parameters_to_set,omitempty"`
	// This field allows the authorization service to specify which query parameters
	// should be removed from the original request before it is sent upstream. Each
	// element in this list is a case-sensitive query parameter name to be removed.
	QueryParametersToRemove []string `protobuf:"bytes,8,rep,name=query_parameters_to_remove,json=queryParametersToRemove,proto3" json:"query_parameters_to_remove,omitempty"`
	unknownFields           protoimpl.UnknownFields
	sizeCache               protoimpl.SizeCache
}

func (x *OkHttpResponse) Reset() {
	*x = OkHttpResponse{}
	mi := &file_envoy_service_auth_v3_external_auth_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *OkHttpResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*OkHttpResponse) ProtoMessage() {}

func (x *OkHttpResponse) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_auth_v3_external_auth_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use OkHttpResponse.ProtoReflect.Descriptor instead.
func (*OkHttpResponse) Descriptor() ([]byte, []int) {
	return file_envoy_service_auth_v3_external_auth_proto_rawDescGZIP(), []int{2}
}

func (x *OkHttpResponse) GetHeaders() []*v31.HeaderValueOption {
	if x != nil {
		return x.Headers
	}
	return nil
}

func (x *OkHttpResponse) GetHeadersToRemove() []string {
	if x != nil {
		return x.HeadersToRemove
	}
	return nil
}

// Deprecated: Marked as deprecated in envoy/service/auth/v3/external_auth.proto.
func (x *OkHttpResponse) GetDynamicMetadata() *structpb.Struct {
	if x != nil {
		return x.DynamicMetadata
	}
	return nil
}

func (x *OkHttpResponse) GetResponseHeadersToAdd() []*v31.HeaderValueOption {
	if x != nil {
		return x.ResponseHeadersToAdd
	}
	return nil
}

func (x *OkHttpResponse) GetQueryParametersToSet() []*v31.QueryParameter {
	if x != nil {
		return x.QueryParametersToSet
	}
	return nil
}

func (x *OkHttpResponse) GetQueryParametersToRemove() []string {
	if x != nil {
		return x.QueryParametersToRemove
	}
	return nil
}

// Intended for gRPC and Network Authorization servers “only“.
// [#next-free-field: 6]
type CheckResponse struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Status “OK“ allows the request. Any other status indicates the request should be denied, and
	// for HTTP filter, if not overridden by :ref:`denied HTTP response status <envoy_v3_api_field_service.auth.v3.DeniedHttpResponse.status>`
	// Envoy sends “403 Forbidden“ HTTP status code by default.
	Status *status.Status `protobuf:"bytes,1,opt,name=status,proto3" json:"status,omitempty"`
	// An message that contains HTTP response attributes. This message is
	// used when the authorization service needs to send custom responses to the
	// downstream client or, to modify/add request headers being dispatched to the upstream.
	//
	// Types that are valid to be assigned to HttpResponse:
	//
	//	*CheckResponse_DeniedResponse
	//	*CheckResponse_OkResponse
	//	*CheckResponse_ErrorResponse
	HttpResponse isCheckResponse_HttpResponse `protobuf_oneof:"http_response"`
	// Optional response metadata that will be emitted as dynamic metadata to be consumed by the next
	// filter. This metadata lives in a namespace specified by the canonical name of extension filter
	// that requires it:
	//
	// - :ref:`envoy.filters.http.ext_authz <config_http_filters_ext_authz_dynamic_metadata>` for HTTP filter.
	// - :ref:`envoy.filters.network.ext_authz <config_network_filters_ext_authz_dynamic_metadata>` for network filter.
	DynamicMetadata *structpb.Struct `protobuf:"bytes,4,opt,name=dynamic_metadata,json=dynamicMetadata,proto3" json:"dynamic_metadata,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *CheckResponse) Reset() {
	*x = CheckResponse{}
	mi := &file_envoy_service_auth_v3_external_auth_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CheckResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CheckResponse) ProtoMessage() {}

func (x *CheckResponse) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_auth_v3_external_auth_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CheckResponse.ProtoReflect.Descriptor instead.
func (*CheckResponse) Descriptor() ([]byte, []int) {
	return file_envoy_service_auth_v3_external_auth_proto_rawDescGZIP(), []int{3}
}

func (x *CheckResponse) GetStatus() *status.Status {
	if x != nil {
		return x.Status
	}
	return nil
}

func (x *CheckResponse) GetHttpResponse() isCheckResponse_HttpResponse {
	if x != nil {
		return x.HttpResponse
	}
	return nil
}

func (x *CheckResponse) GetDeniedResponse() *DeniedHttpResponse {
	if x != nil {
		if x, ok := x.HttpResponse.(*CheckResponse_DeniedResponse); ok {
			return x.DeniedResponse
		}
	}
	return nil
}

func (x *CheckResponse) GetOkResponse() *OkHttpResponse {
	if x != nil {
		if x, ok := x.HttpResponse.(*CheckResponse_OkResponse); ok {
			return x.OkResponse
		}
	}
	return nil
}

func (x *CheckResponse) GetErrorResponse() *DeniedHttpResponse {
	if x != nil {
		if x, ok := x.HttpResponse.(*CheckResponse_ErrorResponse); ok {
			return x.ErrorResponse
		}
	}
	return nil
}

func (x *CheckResponse) GetDynamicMetadata() *structpb.Struct {
	if x != nil {
		return x.DynamicMetadata
	}
	return nil
}

type isCheckResponse_HttpResponse interface {
	isCheckResponse_HttpResponse()
}

type CheckResponse_DeniedResponse struct {
	// Supplies http attributes for a denied response.
	DeniedResponse *DeniedHttpResponse `protobuf:"bytes,2,opt,name=denied_response,json=deniedResponse,proto3,oneof"`
}

type CheckResponse_OkResponse struct {
	// Supplies http attributes for an ok response.
	OkResponse *OkHttpResponse `protobuf:"bytes,3,opt,name=ok_response,json=okResponse,proto3,oneof"`
}

type CheckResponse_ErrorResponse struct {
	// Supplies http attributes for an error response. This is used when the authorization
	// service encounters an internal error and wants to return custom headers and body to the
	// downstream client. When “error_response“ is set, the ext_authz filter increments the
	// “ext_authz_error“ stat and respects the :ref:`failure_mode_allow
	// <envoy_v3_api_field_extensions.filters.http.ext_authz.v3.ExtAuthz.failure_mode_allow>`
	// configuration. The HTTP status code, headers, and body are taken from the
	// :ref:`DeniedHttpResponse <envoy_v3_api_msg_service.auth.v3.DeniedHttpResponse>` message.
	// If the status field is not set, Envoy sends the status code configured via
	// :ref:`status_on_error <envoy_v3_api_field_extensions.filters.http.ext_authz.v3.ExtAuthz.status_on_error>`,
	// which defaults to “403 Forbidden“.
	ErrorResponse *DeniedHttpResponse `protobuf:"bytes,5,opt,name=error_response,json=errorResponse,proto3,oneof"`
}

func (*CheckResponse_DeniedResponse) isCheckResponse_HttpResponse() {}

func (*CheckResponse_OkResponse) isCheckResponse_HttpResponse() {}

func (*CheckResponse_ErrorResponse) isCheckResponse_HttpResponse() {}

var File_envoy_service_auth_v3_external_auth_proto protoreflect.FileDescriptor

const file_envoy_service_auth_v3_external_auth_proto_rawDesc = "" +
	"\n" +
	")envoy/service/auth/v3/external_auth.proto\x12\x15envoy.service.auth.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a-envoy/service/auth/v3/attribute_context.proto\x1a\x1fenvoy/type/v3/http_status.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x17google/rpc/status.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\x82\x01\n" +
	"\fCheckRequest\x12G\n" +
	"\n" +
	"attributes\x18\x01 \x01(\v2'.envoy.service.auth.v3.AttributeContextR\n" +
	"attributes:)\x9aň\x1e$\n" +
	"\"envoy.service.auth.v2.CheckRequest\"\xcf\x01\n" +
	"\x12DeniedHttpResponse\x121\n" +
	"\x06status\x18\x01 \x01(\v2\x19.envoy.type.v3.HttpStatusR\x06status\x12A\n" +
	"\aheaders\x18\x02 \x03(\v2'.envoy.config.core.v3.HeaderValueOptionR\aheaders\x12\x12\n" +
	"\x04body\x18\x03 \x01(\tR\x04body:/\x9aň\x1e*\n" +
	"(envoy.service.auth.v2.DeniedHttpResponse\"\xf7\x03\n" +
	"\x0eOkHttpResponse\x12A\n" +
	"\aheaders\x18\x02 \x03(\v2'.envoy.config.core.v3.HeaderValueOptionR\aheaders\x12*\n" +
	"\x11headers_to_remove\x18\x05 \x03(\tR\x0fheadersToRemove\x12O\n" +
	"\x10dynamic_metadata\x18\x03 \x01(\v2\x17.google.protobuf.StructB\v\x92ǆ\xd8\x04\x033.0\x18\x01R\x0fdynamicMetadata\x12^\n" +
	"\x17response_headers_to_add\x18\x06 \x03(\v2'.envoy.config.core.v3.HeaderValueOptionR\x14responseHeadersToAdd\x12[\n" +
	"\x17query_parameters_to_set\x18\a \x03(\v2$.envoy.config.core.v3.QueryParameterR\x14queryParametersToSet\x12;\n" +
	"\x1aquery_parameters_to_remove\x18\b \x03(\tR\x17queryParametersToRemove:+\x9aň\x1e&\n" +
	"$envoy.service.auth.v2.OkHttpResponse\"\xb0\x03\n" +
	"\rCheckResponse\x12*\n" +
	"\x06status\x18\x01 \x01(\v2\x12.google.rpc.StatusR\x06status\x12T\n" +
	"\x0fdenied_response\x18\x02 \x01(\v2).envoy.service.auth.v3.DeniedHttpResponseH\x00R\x0edeniedResponse\x12H\n" +
	"\vok_response\x18\x03 \x01(\v2%.envoy.service.auth.v3.OkHttpResponseH\x00R\n" +
	"okResponse\x12R\n" +
	"\x0eerror_response\x18\x05 \x01(\v2).envoy.service.auth.v3.DeniedHttpResponseH\x00R\rerrorResponse\x12B\n" +
	"\x10dynamic_metadata\x18\x04 \x01(\v2\x17.google.protobuf.StructR\x0fdynamicMetadata:*\x9aň\x1e%\n" +
	"#envoy.service.auth.v2.CheckResponseB\x0f\n" +
	"\rhttp_response2e\n" +
	"\rAuthorization\x12T\n" +
	"\x05Check\x12#.envoy.service.auth.v3.CheckRequest\x1a$.envoy.service.auth.v3.CheckResponse\"\x00B\x87\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"#io.envoyproxy.envoy.service.auth.v3B\x11ExternalAuthProtoP\x01ZCgithub.com/envoyproxy/go-control-plane/envoy/service/auth/v3;authv3b\x06proto3"

var (
	file_envoy_service_auth_v3_external_auth_proto_rawDescOnce sync.Once
	file_envoy_service_auth_v3_external_auth_proto_rawDescData []byte
)

func file_envoy_service_auth_v3_external_auth_proto_rawDescGZIP() []byte {
	file_envoy_service_auth_v3_external_auth_proto_rawDescOnce.Do(func() {
		file_envoy_service_auth_v3_external_auth_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_service_auth_v3_external_auth_proto_rawDesc), len(file_envoy_service_auth_v3_external_auth_proto_rawDesc)))
	})
	return file_envoy_service_auth_v3_external_auth_proto_rawDescData
}

var file_envoy_service_auth_v3_external_auth_proto_msgTypes = make([]protoimpl.MessageInfo, 4)
var file_envoy_service_auth_v3_external_auth_proto_goTypes = []any{
	(*CheckRequest)(nil),          // 0: envoy.service.auth.v3.CheckRequest
	(*DeniedHttpResponse)(nil),    // 1: envoy.service.auth.v3.DeniedHttpResponse
	(*OkHttpResponse)(nil),        // 2: envoy.service.auth.v3.OkHttpResponse
	(*CheckResponse)(nil),         // 3: envoy.service.auth.v3.CheckResponse
	(*AttributeContext)(nil),      // 4: envoy.service.auth.v3.AttributeContext
	(*v3.HttpStatus)(nil),         // 5: envoy.type.v3.HttpStatus
	(*v31.HeaderValueOption)(nil), // 6: envoy.config.core.v3.HeaderValueOption
	(*structpb.Struct)(nil),       // 7: google.protobuf.Struct
	(*v31.QueryParameter)(nil),    // 8: envoy.config.core.v3.QueryParameter
	(*status.Status)(nil),         // 9: google.rpc.Status
}
var file_envoy_service_auth_v3_external_auth_proto_depIdxs = []int32{
	4,  // 0: envoy.service.auth.v3.CheckRequest.attributes:type_name -> envoy.service.auth.v3.AttributeContext
	5,  // 1: envoy.service.auth.v3.DeniedHttpResponse.status:type_name -> envoy.type.v3.HttpStatus
	6,  // 2: envoy.service.auth.v3.DeniedHttpResponse.headers:type_name -> envoy.config.core.v3.HeaderValueOption
	6,  // 3: envoy.service.auth.v3.OkHttpResponse.headers:type_name -> envoy.config.core.v3.HeaderValueOption
	7,  // 4: envoy.service.auth.v3.OkHttpResponse.dynamic_metadata:type_name -> google.protobuf.Struct
	6,  // 5: envoy.service.auth.v3.OkHttpResponse.response_headers_to_add:type_name -> envoy.config.core.v3.HeaderValueOption
	8,  // 6: envoy.service.auth.v3.OkHttpResponse.query_parameters_to_set:type_name -> envoy.config.core.v3.QueryParameter
	9,  // 7: envoy.service.auth.v3.CheckResponse.status:type_name -> google.rpc.Status
	1,  // 8: envoy.service.auth.v3.CheckResponse.denied_response:type_name -> envoy.service.auth.v3.DeniedHttpResponse
	2,  // 9: envoy.service.auth.v3.CheckResponse.ok_response:type_name -> envoy.service.auth.v3.OkHttpResponse
	1,  // 10: envoy.service.auth.v3.CheckResponse.error_response:type_name -> envoy.service.auth.v3.DeniedHttpResponse
	7,  // 11: envoy.service.auth.v3.CheckResponse.dynamic_metadata:type_name -> google.protobuf.Struct
	0,  // 12: envoy.service.auth.v3.Authorization.Check:input_type -> envoy.service.auth.v3.CheckRequest
	3,  // 13: envoy.service.auth.v3.Authorization.Check:output_type -> envoy.service.auth.v3.CheckResponse
	13, // [13:14] is the sub-list for method output_type
	12, // [12:13] is the sub-list for method input_type
	12, // [12:12] is the sub-list for extension type_name
	12, // [12:12] is the sub-list for extension extendee
	0,  // [0:12] is the sub-list for field type_name
}

func init() { file_envoy_service_auth_v3_external_auth_proto_init() }
func file_envoy_service_auth_v3_external_auth_proto_init() {
	if File_envoy_service_auth_v3_external_auth_proto != nil {
		return
	}
	file_envoy_service_auth_v3_attribute_context_proto_init()
	file_envoy_service_auth_v3_external_auth_proto_msgTypes[3].OneofWrappers = []any{
		(*CheckResponse_DeniedResponse)(nil),
		(*CheckResponse_OkResponse)(nil),
		(*CheckResponse_ErrorResponse)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_service_auth_v3_external_auth_proto_rawDesc), len(file_envoy_service_auth_v3_external_auth_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   4,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_envoy_service_auth_v3_external_auth_proto_goTypes,
		DependencyIndexes: file_envoy_service_auth_v3_external_auth_proto_depIdxs,
		MessageInfos:      file_envoy_service_auth_v3_external_auth_proto_msgTypes,
	}.Build()
	File_envoy_service_auth_v3_external_auth_proto = out.File
	file_envoy_service_auth_v3_external_auth_proto_goTypes = nil
	file_envoy_service_auth_v3_external_auth_proto_depIdxs = nil
}
