// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/key_value/file_based/v3/config.proto

package file_basedv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#extension: envoy.key_value.file_based]
// This is configuration to flush a key value store out to disk.
type FileBasedKeyValueStoreConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The filename to read the keys and values from, and write the keys and
	// values to.
	Filename string `protobuf:"bytes,1,opt,name=filename,proto3" json:"filename,omitempty"`
	// The interval at which the key value store should be flushed to the file.
	FlushInterval *durationpb.Duration `protobuf:"bytes,2,opt,name=flush_interval,json=flushInterval,proto3" json:"flush_interval,omitempty"`
	// The maximum number of entries to cache, or 0 to allow for unlimited entries.
	// Defaults to 1000 if not present.
	MaxEntries    *wrapperspb.UInt32Value `protobuf:"bytes,3,opt,name=max_entries,json=maxEntries,proto3" json:"max_entries,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *FileBasedKeyValueStoreConfig) Reset() {
	*x = FileBasedKeyValueStoreConfig{}
	mi := &file_envoy_extensions_key_value_file_based_v3_config_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FileBasedKeyValueStoreConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FileBasedKeyValueStoreConfig) ProtoMessage() {}

func (x *FileBasedKeyValueStoreConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_key_value_file_based_v3_config_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FileBasedKeyValueStoreConfig.ProtoReflect.Descriptor instead.
func (*FileBasedKeyValueStoreConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_key_value_file_based_v3_config_proto_rawDescGZIP(), []int{0}
}

func (x *FileBasedKeyValueStoreConfig) GetFilename() string {
	if x != nil {
		return x.Filename
	}
	return ""
}

func (x *FileBasedKeyValueStoreConfig) GetFlushInterval() *durationpb.Duration {
	if x != nil {
		return x.FlushInterval
	}
	return nil
}

func (x *FileBasedKeyValueStoreConfig) GetMaxEntries() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MaxEntries
	}
	return nil
}

var File_envoy_extensions_key_value_file_based_v3_config_proto protoreflect.FileDescriptor

const file_envoy_extensions_key_value_file_based_v3_config_proto_rawDesc = "" +
	"\n" +
	"5envoy/extensions/key_value/file_based/v3/config.proto\x12(envoy.extensions.key_value.file_based.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xce\x01\n" +
	"\x1cFileBasedKeyValueStoreConfig\x12#\n" +
	"\bfilename\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\bfilename\x12@\n" +
	"\x0eflush_interval\x18\x02 \x01(\v2\x19.google.protobuf.DurationR\rflushInterval\x12=\n" +
	"\vmax_entries\x18\x03 \x01(\v2\x1c.google.protobuf.UInt32ValueR\n" +
	"maxEntries:\b\xd2Ƥ\xe1\x06\x02\b\x01B\xad\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"6io.envoyproxy.envoy.extensions.key_value.file_based.v3B\vConfigProtoP\x01Z\\github.com/envoyproxy/go-control-plane/envoy/extensions/key_value/file_based/v3;file_basedv3b\x06proto3"

var (
	file_envoy_extensions_key_value_file_based_v3_config_proto_rawDescOnce sync.Once
	file_envoy_extensions_key_value_file_based_v3_config_proto_rawDescData []byte
)

func file_envoy_extensions_key_value_file_based_v3_config_proto_rawDescGZIP() []byte {
	file_envoy_extensions_key_value_file_based_v3_config_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_key_value_file_based_v3_config_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_key_value_file_based_v3_config_proto_rawDesc), len(file_envoy_extensions_key_value_file_based_v3_config_proto_rawDesc)))
	})
	return file_envoy_extensions_key_value_file_based_v3_config_proto_rawDescData
}

var file_envoy_extensions_key_value_file_based_v3_config_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_key_value_file_based_v3_config_proto_goTypes = []any{
	(*FileBasedKeyValueStoreConfig)(nil), // 0: envoy.extensions.key_value.file_based.v3.FileBasedKeyValueStoreConfig
	(*durationpb.Duration)(nil),          // 1: google.protobuf.Duration
	(*wrapperspb.UInt32Value)(nil),       // 2: google.protobuf.UInt32Value
}
var file_envoy_extensions_key_value_file_based_v3_config_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.key_value.file_based.v3.FileBasedKeyValueStoreConfig.flush_interval:type_name -> google.protobuf.Duration
	2, // 1: envoy.extensions.key_value.file_based.v3.FileBasedKeyValueStoreConfig.max_entries:type_name -> google.protobuf.UInt32Value
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_extensions_key_value_file_based_v3_config_proto_init() }
func file_envoy_extensions_key_value_file_based_v3_config_proto_init() {
	if File_envoy_extensions_key_value_file_based_v3_config_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_key_value_file_based_v3_config_proto_rawDesc), len(file_envoy_extensions_key_value_file_based_v3_config_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_key_value_file_based_v3_config_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_key_value_file_based_v3_config_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_key_value_file_based_v3_config_proto_msgTypes,
	}.Build()
	File_envoy_extensions_key_value_file_based_v3_config_proto = out.File
	file_envoy_extensions_key_value_file_based_v3_config_proto_goTypes = nil
	file_envoy_extensions_key_value_file_based_v3_config_proto_depIdxs = nil
}
