// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/common/matching/v3/extension_matcher.proto

package matchingv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v31 "github.com/cncf/xds/go/xds/type/matcher/v3"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/common/matcher/v3"
	v32 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Wrapper around an existing extension that provides an associated matcher. This allows
// decorating an existing extension with a matcher, which can be used to match against
// relevant protocol data.
type ExtensionWithMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The associated matcher. This is deprecated in favor of xds_matcher.
	//
	// Deprecated: Marked as deprecated in envoy/extensions/common/matching/v3/extension_matcher.proto.
	Matcher *v3.Matcher `protobuf:"bytes,1,opt,name=matcher,proto3" json:"matcher,omitempty"`
	// The associated matcher.
	XdsMatcher *v31.Matcher `protobuf:"bytes,3,opt,name=xds_matcher,json=xdsMatcher,proto3" json:"xds_matcher,omitempty"`
	// The underlying extension config.
	ExtensionConfig *v32.TypedExtensionConfig `protobuf:"bytes,2,opt,name=extension_config,json=extensionConfig,proto3" json:"extension_config,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *ExtensionWithMatcher) Reset() {
	*x = ExtensionWithMatcher{}
	mi := &file_envoy_extensions_common_matching_v3_extension_matcher_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ExtensionWithMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ExtensionWithMatcher) ProtoMessage() {}

func (x *ExtensionWithMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_common_matching_v3_extension_matcher_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ExtensionWithMatcher.ProtoReflect.Descriptor instead.
func (*ExtensionWithMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDescGZIP(), []int{0}
}

// Deprecated: Marked as deprecated in envoy/extensions/common/matching/v3/extension_matcher.proto.
func (x *ExtensionWithMatcher) GetMatcher() *v3.Matcher {
	if x != nil {
		return x.Matcher
	}
	return nil
}

func (x *ExtensionWithMatcher) GetXdsMatcher() *v31.Matcher {
	if x != nil {
		return x.XdsMatcher
	}
	return nil
}

func (x *ExtensionWithMatcher) GetExtensionConfig() *v32.TypedExtensionConfig {
	if x != nil {
		return x.ExtensionConfig
	}
	return nil
}

// Extra settings on a per virtualhost/route/weighted-cluster level.
type ExtensionWithMatcherPerRoute struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Matcher override.
	XdsMatcher    *v31.Matcher `protobuf:"bytes,1,opt,name=xds_matcher,json=xdsMatcher,proto3" json:"xds_matcher,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ExtensionWithMatcherPerRoute) Reset() {
	*x = ExtensionWithMatcherPerRoute{}
	mi := &file_envoy_extensions_common_matching_v3_extension_matcher_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ExtensionWithMatcherPerRoute) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ExtensionWithMatcherPerRoute) ProtoMessage() {}

func (x *ExtensionWithMatcherPerRoute) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_common_matching_v3_extension_matcher_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ExtensionWithMatcherPerRoute.ProtoReflect.Descriptor instead.
func (*ExtensionWithMatcherPerRoute) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDescGZIP(), []int{1}
}

func (x *ExtensionWithMatcherPerRoute) GetXdsMatcher() *v31.Matcher {
	if x != nil {
		return x.XdsMatcher
	}
	return nil
}

var File_envoy_extensions_common_matching_v3_extension_matcher_proto protoreflect.FileDescriptor

const file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDesc = "" +
	"\n" +
	";envoy/extensions/common/matching/v3/extension_matcher.proto\x12#envoy.extensions.common.matching.v3\x1a,envoy/config/common/matcher/v3/matcher.proto\x1a$envoy/config/core/v3/extension.proto\x1a!xds/type/matcher/v3/matcher.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x86\x02\n" +
	"\x14ExtensionWithMatcher\x12N\n" +
	"\amatcher\x18\x01 \x01(\v2'.envoy.config.common.matcher.v3.MatcherB\v\x92ǆ\xd8\x04\x033.0\x18\x01R\amatcher\x12=\n" +
	"\vxds_matcher\x18\x03 \x01(\v2\x1c.xds.type.matcher.v3.MatcherR\n" +
	"xdsMatcher\x12_\n" +
	"\x10extension_config\x18\x02 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x0fextensionConfig\"]\n" +
	"\x1cExtensionWithMatcherPerRoute\x12=\n" +
	"\vxds_matcher\x18\x01 \x01(\v2\x1c.xds.type.matcher.v3.MatcherR\n" +
	"xdsMatcherB\xab\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"1io.envoyproxy.envoy.extensions.common.matching.v3B\x15ExtensionMatcherProtoP\x01ZUgithub.com/envoyproxy/go-control-plane/envoy/extensions/common/matching/v3;matchingv3b\x06proto3"

var (
	file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDescOnce sync.Once
	file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDescData []byte
)

func file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDescGZIP() []byte {
	file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDesc), len(file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDesc)))
	})
	return file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDescData
}

var file_envoy_extensions_common_matching_v3_extension_matcher_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_common_matching_v3_extension_matcher_proto_goTypes = []any{
	(*ExtensionWithMatcher)(nil),         // 0: envoy.extensions.common.matching.v3.ExtensionWithMatcher
	(*ExtensionWithMatcherPerRoute)(nil), // 1: envoy.extensions.common.matching.v3.ExtensionWithMatcherPerRoute
	(*v3.Matcher)(nil),                   // 2: envoy.config.common.matcher.v3.Matcher
	(*v31.Matcher)(nil),                  // 3: xds.type.matcher.v3.Matcher
	(*v32.TypedExtensionConfig)(nil),     // 4: envoy.config.core.v3.TypedExtensionConfig
}
var file_envoy_extensions_common_matching_v3_extension_matcher_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.common.matching.v3.ExtensionWithMatcher.matcher:type_name -> envoy.config.common.matcher.v3.Matcher
	3, // 1: envoy.extensions.common.matching.v3.ExtensionWithMatcher.xds_matcher:type_name -> xds.type.matcher.v3.Matcher
	4, // 2: envoy.extensions.common.matching.v3.ExtensionWithMatcher.extension_config:type_name -> envoy.config.core.v3.TypedExtensionConfig
	3, // 3: envoy.extensions.common.matching.v3.ExtensionWithMatcherPerRoute.xds_matcher:type_name -> xds.type.matcher.v3.Matcher
	4, // [4:4] is the sub-list for method output_type
	4, // [4:4] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_envoy_extensions_common_matching_v3_extension_matcher_proto_init() }
func file_envoy_extensions_common_matching_v3_extension_matcher_proto_init() {
	if File_envoy_extensions_common_matching_v3_extension_matcher_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDesc), len(file_envoy_extensions_common_matching_v3_extension_matcher_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_common_matching_v3_extension_matcher_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_common_matching_v3_extension_matcher_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_common_matching_v3_extension_matcher_proto_msgTypes,
	}.Build()
	File_envoy_extensions_common_matching_v3_extension_matcher_proto = out.File
	file_envoy_extensions_common_matching_v3_extension_matcher_proto_goTypes = nil
	file_envoy_extensions_common_matching_v3_extension_matcher_proto_depIdxs = nil
}
