// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/network/socket_interface/v3/default_socket_interface.proto

package socket_interfacev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for default socket interface that relies on OS dependent syscall to create
// sockets.
type DefaultSocketInterface struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// io_uring options. io_uring is only valid in Linux with at least kernel version 5.11. Otherwise,
	// Envoy will fall back to use the default socket API. If not set then io_uring will not be
	// enabled.
	IoUringOptions *IoUringOptions `protobuf:"bytes,1,opt,name=io_uring_options,json=ioUringOptions,proto3" json:"io_uring_options,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *DefaultSocketInterface) Reset() {
	*x = DefaultSocketInterface{}
	mi := &file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DefaultSocketInterface) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DefaultSocketInterface) ProtoMessage() {}

func (x *DefaultSocketInterface) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DefaultSocketInterface.ProtoReflect.Descriptor instead.
func (*DefaultSocketInterface) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDescGZIP(), []int{0}
}

func (x *DefaultSocketInterface) GetIoUringOptions() *IoUringOptions {
	if x != nil {
		return x.IoUringOptions
	}
	return nil
}

type IoUringOptions struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The size for io_uring submission queues (SQ). io_uring is built with a fixed size in each
	// thread during configuration, and each io_uring operation creates a submission queue
	// entry (SQE). The default is 1000.
	IoUringSize *wrapperspb.UInt32Value `protobuf:"bytes,1,opt,name=io_uring_size,json=ioUringSize,proto3" json:"io_uring_size,omitempty"`
	// Enable io_uring submission queue polling (SQPOLL). io_uring SQPOLL mode polls all SQEs in the
	// SQ in the kernel thread. io_uring SQPOLL mode may reduce latency and increase CPU usage as a
	// cost. The default is false.
	EnableSubmissionQueuePolling bool `protobuf:"varint,2,opt,name=enable_submission_queue_polling,json=enableSubmissionQueuePolling,proto3" json:"enable_submission_queue_polling,omitempty"`
	// The size of an io_uring socket's read buffer. Each io_uring read operation will allocate a
	// buffer of the given size. If the given buffer is too small, the socket will have read multiple
	// times for all the data. The default is 8192.
	ReadBufferSize *wrapperspb.UInt32Value `protobuf:"bytes,3,opt,name=read_buffer_size,json=readBufferSize,proto3" json:"read_buffer_size,omitempty"`
	// The write timeout of an io_uring socket on closing in ms. io_uring writes and closes
	// asynchronously. If the remote stops reading, the io_uring write operation may never complete.
	// The operation is canceled and the socket is closed after the timeout. The default is 1000.
	WriteTimeoutMs *wrapperspb.UInt32Value `protobuf:"bytes,4,opt,name=write_timeout_ms,json=writeTimeoutMs,proto3" json:"write_timeout_ms,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *IoUringOptions) Reset() {
	*x = IoUringOptions{}
	mi := &file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *IoUringOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*IoUringOptions) ProtoMessage() {}

func (x *IoUringOptions) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use IoUringOptions.ProtoReflect.Descriptor instead.
func (*IoUringOptions) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDescGZIP(), []int{1}
}

func (x *IoUringOptions) GetIoUringSize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.IoUringSize
	}
	return nil
}

func (x *IoUringOptions) GetEnableSubmissionQueuePolling() bool {
	if x != nil {
		return x.EnableSubmissionQueuePolling
	}
	return false
}

func (x *IoUringOptions) GetReadBufferSize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.ReadBufferSize
	}
	return nil
}

func (x *IoUringOptions) GetWriteTimeoutMs() *wrapperspb.UInt32Value {
	if x != nil {
		return x.WriteTimeoutMs
	}
	return nil
}

var File_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto protoreflect.FileDescriptor

const file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDesc = "" +
	"\n" +
	"Kenvoy/extensions/network/socket_interface/v3/default_socket_interface.proto\x12,envoy.extensions.network.socket_interface.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\"\x80\x01\n" +
	"\x16DefaultSocketInterface\x12f\n" +
	"\x10io_uring_options\x18\x01 \x01(\v2<.envoy.extensions.network.socket_interface.v3.IoUringOptionsR\x0eioUringOptions\"\xa9\x02\n" +
	"\x0eIoUringOptions\x12@\n" +
	"\rio_uring_size\x18\x01 \x01(\v2\x1c.google.protobuf.UInt32ValueR\vioUringSize\x12E\n" +
	"\x1fenable_submission_queue_polling\x18\x02 \x01(\bR\x1cenableSubmissionQueuePolling\x12F\n" +
	"\x10read_buffer_size\x18\x03 \x01(\v2\x1c.google.protobuf.UInt32ValueR\x0ereadBufferSize\x12F\n" +
	"\x10write_timeout_ms\x18\x04 \x01(\v2\x1c.google.protobuf.UInt32ValueR\x0ewriteTimeoutMsB\xcb\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	":io.envoyproxy.envoy.extensions.network.socket_interface.v3B\x1bDefaultSocketInterfaceProtoP\x01Zfgithub.com/envoyproxy/go-control-plane/envoy/extensions/network/socket_interface/v3;socket_interfacev3b\x06proto3"

var (
	file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDescOnce sync.Once
	file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDescData []byte
)

func file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDescGZIP() []byte {
	file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDesc), len(file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDesc)))
	})
	return file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDescData
}

var file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_goTypes = []any{
	(*DefaultSocketInterface)(nil), // 0: envoy.extensions.network.socket_interface.v3.DefaultSocketInterface
	(*IoUringOptions)(nil),         // 1: envoy.extensions.network.socket_interface.v3.IoUringOptions
	(*wrapperspb.UInt32Value)(nil), // 2: google.protobuf.UInt32Value
}
var file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.network.socket_interface.v3.DefaultSocketInterface.io_uring_options:type_name -> envoy.extensions.network.socket_interface.v3.IoUringOptions
	2, // 1: envoy.extensions.network.socket_interface.v3.IoUringOptions.io_uring_size:type_name -> google.protobuf.UInt32Value
	2, // 2: envoy.extensions.network.socket_interface.v3.IoUringOptions.read_buffer_size:type_name -> google.protobuf.UInt32Value
	2, // 3: envoy.extensions.network.socket_interface.v3.IoUringOptions.write_timeout_ms:type_name -> google.protobuf.UInt32Value
	4, // [4:4] is the sub-list for method output_type
	4, // [4:4] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_init() }
func file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_init() {
	if File_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDesc), len(file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_msgTypes,
	}.Build()
	File_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto = out.File
	file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_goTypes = nil
	file_envoy_extensions_network_socket_interface_v3_default_socket_interface_proto_depIdxs = nil
}
