// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/service/metrics/v3/metrics_service.proto

package metricsv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	_go "github.com/prometheus/client_model/go"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type StreamMetricsResponse struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StreamMetricsResponse) Reset() {
	*x = StreamMetricsResponse{}
	mi := &file_envoy_service_metrics_v3_metrics_service_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StreamMetricsResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StreamMetricsResponse) ProtoMessage() {}

func (x *StreamMetricsResponse) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_metrics_v3_metrics_service_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StreamMetricsResponse.ProtoReflect.Descriptor instead.
func (*StreamMetricsResponse) Descriptor() ([]byte, []int) {
	return file_envoy_service_metrics_v3_metrics_service_proto_rawDescGZIP(), []int{0}
}

type StreamMetricsMessage struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Identifier data effectively is a structured metadata. As a performance optimization this will
	// only be sent in the first message on the stream.
	Identifier *StreamMetricsMessage_Identifier `protobuf:"bytes,1,opt,name=identifier,proto3" json:"identifier,omitempty"`
	// A list of metric entries
	EnvoyMetrics  []*_go.MetricFamily `protobuf:"bytes,2,rep,name=envoy_metrics,json=envoyMetrics,proto3" json:"envoy_metrics,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StreamMetricsMessage) Reset() {
	*x = StreamMetricsMessage{}
	mi := &file_envoy_service_metrics_v3_metrics_service_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StreamMetricsMessage) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StreamMetricsMessage) ProtoMessage() {}

func (x *StreamMetricsMessage) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_metrics_v3_metrics_service_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StreamMetricsMessage.ProtoReflect.Descriptor instead.
func (*StreamMetricsMessage) Descriptor() ([]byte, []int) {
	return file_envoy_service_metrics_v3_metrics_service_proto_rawDescGZIP(), []int{1}
}

func (x *StreamMetricsMessage) GetIdentifier() *StreamMetricsMessage_Identifier {
	if x != nil {
		return x.Identifier
	}
	return nil
}

func (x *StreamMetricsMessage) GetEnvoyMetrics() []*_go.MetricFamily {
	if x != nil {
		return x.EnvoyMetrics
	}
	return nil
}

type StreamMetricsMessage_Identifier struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The node sending metrics over the stream.
	Node          *v3.Node `protobuf:"bytes,1,opt,name=node,proto3" json:"node,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *StreamMetricsMessage_Identifier) Reset() {
	*x = StreamMetricsMessage_Identifier{}
	mi := &file_envoy_service_metrics_v3_metrics_service_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *StreamMetricsMessage_Identifier) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*StreamMetricsMessage_Identifier) ProtoMessage() {}

func (x *StreamMetricsMessage_Identifier) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_metrics_v3_metrics_service_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use StreamMetricsMessage_Identifier.ProtoReflect.Descriptor instead.
func (*StreamMetricsMessage_Identifier) Descriptor() ([]byte, []int) {
	return file_envoy_service_metrics_v3_metrics_service_proto_rawDescGZIP(), []int{1, 0}
}

func (x *StreamMetricsMessage_Identifier) GetNode() *v3.Node {
	if x != nil {
		return x.Node
	}
	return nil
}

var File_envoy_service_metrics_v3_metrics_service_proto protoreflect.FileDescriptor

const file_envoy_service_metrics_v3_metrics_service_proto_rawDesc = "" +
	"\n" +
	".envoy/service/metrics/v3/metrics_service.proto\x12\x18envoy.service.metrics.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a\"io/prometheus/client/metrics.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"N\n" +
	"\x15StreamMetricsResponse:5\x9aň\x1e0\n" +
	".envoy.service.metrics.v2.StreamMetricsResponse\"\xfa\x02\n" +
	"\x14StreamMetricsMessage\x12Y\n" +
	"\n" +
	"identifier\x18\x01 \x01(\v29.envoy.service.metrics.v3.StreamMetricsMessage.IdentifierR\n" +
	"identifier\x12G\n" +
	"\renvoy_metrics\x18\x02 \x03(\v2\".io.prometheus.client.MetricFamilyR\fenvoyMetrics\x1a\x87\x01\n" +
	"\n" +
	"Identifier\x128\n" +
	"\x04node\x18\x01 \x01(\v2\x1a.envoy.config.core.v3.NodeB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x04node:?\x9aň\x1e:\n" +
	"8envoy.service.metrics.v2.StreamMetricsMessage.Identifier:4\x9aň\x1e/\n" +
	"-envoy.service.metrics.v2.StreamMetricsMessage2\x86\x01\n" +
	"\x0eMetricsService\x12t\n" +
	"\rStreamMetrics\x12..envoy.service.metrics.v3.StreamMetricsMessage\x1a/.envoy.service.metrics.v3.StreamMetricsResponse\"\x00(\x01B\x92\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"&io.envoyproxy.envoy.service.metrics.v3B\x13MetricsServiceProtoP\x01ZIgithub.com/envoyproxy/go-control-plane/envoy/service/metrics/v3;metricsv3b\x06proto3"

var (
	file_envoy_service_metrics_v3_metrics_service_proto_rawDescOnce sync.Once
	file_envoy_service_metrics_v3_metrics_service_proto_rawDescData []byte
)

func file_envoy_service_metrics_v3_metrics_service_proto_rawDescGZIP() []byte {
	file_envoy_service_metrics_v3_metrics_service_proto_rawDescOnce.Do(func() {
		file_envoy_service_metrics_v3_metrics_service_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_service_metrics_v3_metrics_service_proto_rawDesc), len(file_envoy_service_metrics_v3_metrics_service_proto_rawDesc)))
	})
	return file_envoy_service_metrics_v3_metrics_service_proto_rawDescData
}

var file_envoy_service_metrics_v3_metrics_service_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_envoy_service_metrics_v3_metrics_service_proto_goTypes = []any{
	(*StreamMetricsResponse)(nil),           // 0: envoy.service.metrics.v3.StreamMetricsResponse
	(*StreamMetricsMessage)(nil),            // 1: envoy.service.metrics.v3.StreamMetricsMessage
	(*StreamMetricsMessage_Identifier)(nil), // 2: envoy.service.metrics.v3.StreamMetricsMessage.Identifier
	(*_go.MetricFamily)(nil),                // 3: io.prometheus.client.MetricFamily
	(*v3.Node)(nil),                         // 4: envoy.config.core.v3.Node
}
var file_envoy_service_metrics_v3_metrics_service_proto_depIdxs = []int32{
	2, // 0: envoy.service.metrics.v3.StreamMetricsMessage.identifier:type_name -> envoy.service.metrics.v3.StreamMetricsMessage.Identifier
	3, // 1: envoy.service.metrics.v3.StreamMetricsMessage.envoy_metrics:type_name -> io.prometheus.client.MetricFamily
	4, // 2: envoy.service.metrics.v3.StreamMetricsMessage.Identifier.node:type_name -> envoy.config.core.v3.Node
	1, // 3: envoy.service.metrics.v3.MetricsService.StreamMetrics:input_type -> envoy.service.metrics.v3.StreamMetricsMessage
	0, // 4: envoy.service.metrics.v3.MetricsService.StreamMetrics:output_type -> envoy.service.metrics.v3.StreamMetricsResponse
	4, // [4:5] is the sub-list for method output_type
	3, // [3:4] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_envoy_service_metrics_v3_metrics_service_proto_init() }
func file_envoy_service_metrics_v3_metrics_service_proto_init() {
	if File_envoy_service_metrics_v3_metrics_service_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_service_metrics_v3_metrics_service_proto_rawDesc), len(file_envoy_service_metrics_v3_metrics_service_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_envoy_service_metrics_v3_metrics_service_proto_goTypes,
		DependencyIndexes: file_envoy_service_metrics_v3_metrics_service_proto_depIdxs,
		MessageInfos:      file_envoy_service_metrics_v3_metrics_service_proto_msgTypes,
	}.Build()
	File_envoy_service_metrics_v3_metrics_service_proto = out.File
	file_envoy_service_metrics_v3_metrics_service_proto_goTypes = nil
	file_envoy_service_metrics_v3_metrics_service_proto_depIdxs = nil
}
