// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/compression/brotli/compressor/v3/brotli.proto

package compressorv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Brotli_EncoderMode int32

const (
	Brotli_DEFAULT Brotli_EncoderMode = 0
	Brotli_GENERIC Brotli_EncoderMode = 1
	Brotli_TEXT    Brotli_EncoderMode = 2
	Brotli_FONT    Brotli_EncoderMode = 3
)

// Enum value maps for Brotli_EncoderMode.
var (
	Brotli_EncoderMode_name = map[int32]string{
		0: "DEFAULT",
		1: "GENERIC",
		2: "TEXT",
		3: "FONT",
	}
	Brotli_EncoderMode_value = map[string]int32{
		"DEFAULT": 0,
		"GENERIC": 1,
		"TEXT":    2,
		"FONT":    3,
	}
)

func (x Brotli_EncoderMode) Enum() *Brotli_EncoderMode {
	p := new(Brotli_EncoderMode)
	*p = x
	return p
}

func (x Brotli_EncoderMode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (Brotli_EncoderMode) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_enumTypes[0].Descriptor()
}

func (Brotli_EncoderMode) Type() protoreflect.EnumType {
	return &file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_enumTypes[0]
}

func (x Brotli_EncoderMode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use Brotli_EncoderMode.Descriptor instead.
func (Brotli_EncoderMode) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDescGZIP(), []int{0, 0}
}

// [#next-free-field: 7]
type Brotli struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Value from 0 to 11 that controls the main compression speed-density lever.
	// The higher quality, the slower compression. The default value is 3.
	Quality *wrapperspb.UInt32Value `protobuf:"bytes,1,opt,name=quality,proto3" json:"quality,omitempty"`
	// A value used to tune encoder for specific input. For more information about modes,
	// please refer to brotli manual: https://brotli.org/encode.html#aa6f
	// This field will be set to "DEFAULT" if not specified.
	EncoderMode Brotli_EncoderMode `protobuf:"varint,2,opt,name=encoder_mode,json=encoderMode,proto3,enum=envoy.extensions.compression.brotli.compressor.v3.Brotli_EncoderMode" json:"encoder_mode,omitempty"`
	// Value from 10 to 24 that represents the base two logarithmic of the compressor's window size.
	// Larger window results in better compression at the expense of memory usage. The default is 18.
	// For more details about this parameter, please refer to brotli manual:
	// https://brotli.org/encode.html#a9a8
	WindowBits *wrapperspb.UInt32Value `protobuf:"bytes,3,opt,name=window_bits,json=windowBits,proto3" json:"window_bits,omitempty"`
	// Value from 16 to 24 that represents the base two logarithmic of the compressor's input block
	// size. Larger input block results in better compression at the expense of memory usage. The
	// default is 24. For more details about this parameter, please refer to brotli manual:
	// https://brotli.org/encode.html#a9a8
	InputBlockBits *wrapperspb.UInt32Value `protobuf:"bytes,4,opt,name=input_block_bits,json=inputBlockBits,proto3" json:"input_block_bits,omitempty"`
	// Value for compressor's next output buffer. If not set, defaults to 4096.
	ChunkSize *wrapperspb.UInt32Value `protobuf:"bytes,5,opt,name=chunk_size,json=chunkSize,proto3" json:"chunk_size,omitempty"`
	// If true, disables "literal context modeling" format feature.
	// This flag is a "decoding-speed vs compression ratio" trade-off.
	DisableLiteralContextModeling bool `protobuf:"varint,6,opt,name=disable_literal_context_modeling,json=disableLiteralContextModeling,proto3" json:"disable_literal_context_modeling,omitempty"`
	unknownFields                 protoimpl.UnknownFields
	sizeCache                     protoimpl.SizeCache
}

func (x *Brotli) Reset() {
	*x = Brotli{}
	mi := &file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Brotli) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Brotli) ProtoMessage() {}

func (x *Brotli) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Brotli.ProtoReflect.Descriptor instead.
func (*Brotli) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDescGZIP(), []int{0}
}

func (x *Brotli) GetQuality() *wrapperspb.UInt32Value {
	if x != nil {
		return x.Quality
	}
	return nil
}

func (x *Brotli) GetEncoderMode() Brotli_EncoderMode {
	if x != nil {
		return x.EncoderMode
	}
	return Brotli_DEFAULT
}

func (x *Brotli) GetWindowBits() *wrapperspb.UInt32Value {
	if x != nil {
		return x.WindowBits
	}
	return nil
}

func (x *Brotli) GetInputBlockBits() *wrapperspb.UInt32Value {
	if x != nil {
		return x.InputBlockBits
	}
	return nil
}

func (x *Brotli) GetChunkSize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.ChunkSize
	}
	return nil
}

func (x *Brotli) GetDisableLiteralContextModeling() bool {
	if x != nil {
		return x.DisableLiteralContextModeling
	}
	return false
}

var File_envoy_extensions_compression_brotli_compressor_v3_brotli_proto protoreflect.FileDescriptor

const file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDesc = "" +
	"\n" +
	">envoy/extensions/compression/brotli/compressor/v3/brotli.proto\x121envoy.extensions.compression.brotli.compressor.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xab\x04\n" +
	"\x06Brotli\x12?\n" +
	"\aquality\x18\x01 \x01(\v2\x1c.google.protobuf.UInt32ValueB\a\xfaB\x04*\x02\x18\vR\aquality\x12r\n" +
	"\fencoder_mode\x18\x02 \x01(\x0e2E.envoy.extensions.compression.brotli.compressor.v3.Brotli.EncoderModeB\b\xfaB\x05\x82\x01\x02\x10\x01R\vencoderMode\x12H\n" +
	"\vwindow_bits\x18\x03 \x01(\v2\x1c.google.protobuf.UInt32ValueB\t\xfaB\x06*\x04\x18\x18(\n" +
	"R\n" +
	"windowBits\x12Q\n" +
	"\x10input_block_bits\x18\x04 \x01(\v2\x1c.google.protobuf.UInt32ValueB\t\xfaB\x06*\x04\x18\x18(\x10R\x0einputBlockBits\x12I\n" +
	"\n" +
	"chunk_size\x18\x05 \x01(\v2\x1c.google.protobuf.UInt32ValueB\f\xfaB\t*\a\x18\x80\x80\x04(\x80 R\tchunkSize\x12G\n" +
	" disable_literal_context_modeling\x18\x06 \x01(\bR\x1ddisableLiteralContextModeling\";\n" +
	"\vEncoderMode\x12\v\n" +
	"\aDEFAULT\x10\x00\x12\v\n" +
	"\aGENERIC\x10\x01\x12\b\n" +
	"\x04TEXT\x10\x02\x12\b\n" +
	"\x04FONT\x10\x03B\xbf\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"?io.envoyproxy.envoy.extensions.compression.brotli.compressor.v3B\vBrotliProtoP\x01Zegithub.com/envoyproxy/go-control-plane/envoy/extensions/compression/brotli/compressor/v3;compressorv3b\x06proto3"

var (
	file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDescOnce sync.Once
	file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDescData []byte
)

func file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDescGZIP() []byte {
	file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDesc), len(file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDesc)))
	})
	return file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDescData
}

var file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_goTypes = []any{
	(Brotli_EncoderMode)(0),        // 0: envoy.extensions.compression.brotli.compressor.v3.Brotli.EncoderMode
	(*Brotli)(nil),                 // 1: envoy.extensions.compression.brotli.compressor.v3.Brotli
	(*wrapperspb.UInt32Value)(nil), // 2: google.protobuf.UInt32Value
}
var file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.compression.brotli.compressor.v3.Brotli.quality:type_name -> google.protobuf.UInt32Value
	0, // 1: envoy.extensions.compression.brotli.compressor.v3.Brotli.encoder_mode:type_name -> envoy.extensions.compression.brotli.compressor.v3.Brotli.EncoderMode
	2, // 2: envoy.extensions.compression.brotli.compressor.v3.Brotli.window_bits:type_name -> google.protobuf.UInt32Value
	2, // 3: envoy.extensions.compression.brotli.compressor.v3.Brotli.input_block_bits:type_name -> google.protobuf.UInt32Value
	2, // 4: envoy.extensions.compression.brotli.compressor.v3.Brotli.chunk_size:type_name -> google.protobuf.UInt32Value
	5, // [5:5] is the sub-list for method output_type
	5, // [5:5] is the sub-list for method input_type
	5, // [5:5] is the sub-list for extension type_name
	5, // [5:5] is the sub-list for extension extendee
	0, // [0:5] is the sub-list for field type_name
}

func init() { file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_init() }
func file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_init() {
	if File_envoy_extensions_compression_brotli_compressor_v3_brotli_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDesc), len(file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_depIdxs,
		EnumInfos:         file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_enumTypes,
		MessageInfos:      file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_msgTypes,
	}.Build()
	File_envoy_extensions_compression_brotli_compressor_v3_brotli_proto = out.File
	file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_goTypes = nil
	file_envoy_extensions_compression_brotli_compressor_v3_brotli_proto_depIdxs = nil
}
