// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/http/grpc_http1_bridge/v3/config.proto

package grpc_http1_bridgev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// gRPC HTTP/1.1 Bridge filter config.
type Config struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// If true then requests with content type set to “application/x-protobuf“ will be automatically converted to gRPC.
	// This works by prepending the payload data with the gRPC header frame, as defined by the wiring format, and
	// Content-Type will be updated accordingly before sending the request.
	// For the requests that went through this upgrade the filter will also strip the frame before forwarding the
	// response to the client.
	UpgradeProtobufToGrpc bool `protobuf:"varint,1,opt,name=upgrade_protobuf_to_grpc,json=upgradeProtobufToGrpc,proto3" json:"upgrade_protobuf_to_grpc,omitempty"`
	// If true then query parameters in request's URL path will be removed.
	IgnoreQueryParameters bool `protobuf:"varint,2,opt,name=ignore_query_parameters,json=ignoreQueryParameters,proto3" json:"ignore_query_parameters,omitempty"`
	unknownFields         protoimpl.UnknownFields
	sizeCache             protoimpl.SizeCache
}

func (x *Config) Reset() {
	*x = Config{}
	mi := &file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config) ProtoMessage() {}

func (x *Config) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config.ProtoReflect.Descriptor instead.
func (*Config) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDescGZIP(), []int{0}
}

func (x *Config) GetUpgradeProtobufToGrpc() bool {
	if x != nil {
		return x.UpgradeProtobufToGrpc
	}
	return false
}

func (x *Config) GetIgnoreQueryParameters() bool {
	if x != nil {
		return x.IgnoreQueryParameters
	}
	return false
}

var File_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDesc = "" +
	"\n" +
	"?envoy/extensions/filters/http/grpc_http1_bridge/v3/config.proto\x122envoy.extensions.filters.http.grpc_http1_bridge.v3\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\xb6\x01\n" +
	"\x06Config\x127\n" +
	"\x18upgrade_protobuf_to_grpc\x18\x01 \x01(\bR\x15upgradeProtobufToGrpc\x126\n" +
	"\x17ignore_query_parameters\x18\x02 \x01(\bR\x15ignoreQueryParameters:;\x9aň\x1e6\n" +
	"4envoy.config.filter.http.grpc_http1_bridge.v2.ConfigB\xc8\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"@io.envoyproxy.envoy.extensions.filters.http.grpc_http1_bridge.v3B\vConfigProtoP\x01Zmgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/grpc_http1_bridge/v3;grpc_http1_bridgev3b\x06proto3"

var (
	file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDescData []byte
)

func file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDesc), len(file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDescData
}

var file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_goTypes = []any{
	(*Config)(nil), // 0: envoy.extensions.filters.http.grpc_http1_bridge.v3.Config
}
var file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_init() }
func file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_init() {
	if File_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDesc), len(file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto = out.File
	file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_goTypes = nil
	file_envoy_extensions_filters_http_grpc_http1_bridge_v3_config_proto_depIdxs = nil
}
