// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/access_loggers/file/v3/file.proto

package filev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	structpb "google.golang.org/protobuf/types/known/structpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Custom configuration for an :ref:`AccessLog <envoy_v3_api_msg_config.accesslog.v3.AccessLog>`
// that writes log entries directly to a file. Configures the built-in “envoy.access_loggers.file“
// AccessLog.
// [#next-free-field: 6]
type FileAccessLog struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// A path to a local file to which to write the access log entries.
	Path string `protobuf:"bytes,1,opt,name=path,proto3" json:"path,omitempty"`
	// Types that are valid to be assigned to AccessLogFormat:
	//
	//	*FileAccessLog_Format
	//	*FileAccessLog_JsonFormat
	//	*FileAccessLog_TypedJsonFormat
	//	*FileAccessLog_LogFormat
	AccessLogFormat isFileAccessLog_AccessLogFormat `protobuf_oneof:"access_log_format"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *FileAccessLog) Reset() {
	*x = FileAccessLog{}
	mi := &file_envoy_extensions_access_loggers_file_v3_file_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *FileAccessLog) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*FileAccessLog) ProtoMessage() {}

func (x *FileAccessLog) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_access_loggers_file_v3_file_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use FileAccessLog.ProtoReflect.Descriptor instead.
func (*FileAccessLog) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_access_loggers_file_v3_file_proto_rawDescGZIP(), []int{0}
}

func (x *FileAccessLog) GetPath() string {
	if x != nil {
		return x.Path
	}
	return ""
}

func (x *FileAccessLog) GetAccessLogFormat() isFileAccessLog_AccessLogFormat {
	if x != nil {
		return x.AccessLogFormat
	}
	return nil
}

// Deprecated: Marked as deprecated in envoy/extensions/access_loggers/file/v3/file.proto.
func (x *FileAccessLog) GetFormat() string {
	if x != nil {
		if x, ok := x.AccessLogFormat.(*FileAccessLog_Format); ok {
			return x.Format
		}
	}
	return ""
}

// Deprecated: Marked as deprecated in envoy/extensions/access_loggers/file/v3/file.proto.
func (x *FileAccessLog) GetJsonFormat() *structpb.Struct {
	if x != nil {
		if x, ok := x.AccessLogFormat.(*FileAccessLog_JsonFormat); ok {
			return x.JsonFormat
		}
	}
	return nil
}

// Deprecated: Marked as deprecated in envoy/extensions/access_loggers/file/v3/file.proto.
func (x *FileAccessLog) GetTypedJsonFormat() *structpb.Struct {
	if x != nil {
		if x, ok := x.AccessLogFormat.(*FileAccessLog_TypedJsonFormat); ok {
			return x.TypedJsonFormat
		}
	}
	return nil
}

func (x *FileAccessLog) GetLogFormat() *v3.SubstitutionFormatString {
	if x != nil {
		if x, ok := x.AccessLogFormat.(*FileAccessLog_LogFormat); ok {
			return x.LogFormat
		}
	}
	return nil
}

type isFileAccessLog_AccessLogFormat interface {
	isFileAccessLog_AccessLogFormat()
}

type FileAccessLog_Format struct {
	// Access log :ref:`format string<config_access_log_format_strings>`.
	// Envoy supports :ref:`custom access log formats <config_access_log_format>` as well as a
	// :ref:`default format <config_access_log_default_format>`.
	// This field is deprecated.
	// Please use :ref:`log_format <envoy_v3_api_field_extensions.access_loggers.file.v3.FileAccessLog.log_format>`.
	//
	// Deprecated: Marked as deprecated in envoy/extensions/access_loggers/file/v3/file.proto.
	Format string `protobuf:"bytes,2,opt,name=format,proto3,oneof"`
}

type FileAccessLog_JsonFormat struct {
	// Access log :ref:`format dictionary<config_access_log_format_dictionaries>`. All values
	// are rendered as strings.
	// This field is deprecated.
	// Please use :ref:`log_format <envoy_v3_api_field_extensions.access_loggers.file.v3.FileAccessLog.log_format>`.
	//
	// Deprecated: Marked as deprecated in envoy/extensions/access_loggers/file/v3/file.proto.
	JsonFormat *structpb.Struct `protobuf:"bytes,3,opt,name=json_format,json=jsonFormat,proto3,oneof"`
}

type FileAccessLog_TypedJsonFormat struct {
	// Access log :ref:`format dictionary<config_access_log_format_dictionaries>`. Values are
	// rendered as strings, numbers, or boolean values as appropriate. Nested JSON objects may
	// be produced by some command operators (e.g.FILTER_STATE or DYNAMIC_METADATA). See the
	// documentation for a specific command operator for details.
	// This field is deprecated.
	// Please use :ref:`log_format <envoy_v3_api_field_extensions.access_loggers.file.v3.FileAccessLog.log_format>`.
	//
	// Deprecated: Marked as deprecated in envoy/extensions/access_loggers/file/v3/file.proto.
	TypedJsonFormat *structpb.Struct `protobuf:"bytes,4,opt,name=typed_json_format,json=typedJsonFormat,proto3,oneof"`
}

type FileAccessLog_LogFormat struct {
	// Configuration to form access log data and format.
	// If not specified, use :ref:`default format <config_access_log_default_format>`.
	LogFormat *v3.SubstitutionFormatString `protobuf:"bytes,5,opt,name=log_format,json=logFormat,proto3,oneof"`
}

func (*FileAccessLog_Format) isFileAccessLog_AccessLogFormat() {}

func (*FileAccessLog_JsonFormat) isFileAccessLog_AccessLogFormat() {}

func (*FileAccessLog_TypedJsonFormat) isFileAccessLog_AccessLogFormat() {}

func (*FileAccessLog_LogFormat) isFileAccessLog_AccessLogFormat() {}

var File_envoy_extensions_access_loggers_file_v3_file_proto protoreflect.FileDescriptor

const file_envoy_extensions_access_loggers_file_v3_file_proto_rawDesc = "" +
	"\n" +
	"2envoy/extensions/access_loggers/file/v3/file.proto\x12'envoy.extensions.access_loggers.file.v3\x1a5envoy/config/core/v3/substitution_format_string.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x90\x03\n" +
	"\rFileAccessLog\x12\x1b\n" +
	"\x04path\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04path\x12%\n" +
	"\x06format\x18\x02 \x01(\tB\v\x92ǆ\xd8\x04\x033.0\x18\x01H\x00R\x06format\x12G\n" +
	"\vjson_format\x18\x03 \x01(\v2\x17.google.protobuf.StructB\v\x92ǆ\xd8\x04\x033.0\x18\x01H\x00R\n" +
	"jsonFormat\x12R\n" +
	"\x11typed_json_format\x18\x04 \x01(\v2\x17.google.protobuf.StructB\v\x92ǆ\xd8\x04\x033.0\x18\x01H\x00R\x0ftypedJsonFormat\x12Y\n" +
	"\n" +
	"log_format\x18\x05 \x01(\v2..envoy.config.core.v3.SubstitutionFormatStringB\b\xfaB\x05\x8a\x01\x02\x10\x01H\x00R\tlogFormat:.\x9aň\x1e)\n" +
	"'envoy.config.accesslog.v2.FileAccessLogB\x13\n" +
	"\x11access_log_formatB\xa3\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"5io.envoyproxy.envoy.extensions.access_loggers.file.v3B\tFileProtoP\x01ZUgithub.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/file/v3;filev3b\x06proto3"

var (
	file_envoy_extensions_access_loggers_file_v3_file_proto_rawDescOnce sync.Once
	file_envoy_extensions_access_loggers_file_v3_file_proto_rawDescData []byte
)

func file_envoy_extensions_access_loggers_file_v3_file_proto_rawDescGZIP() []byte {
	file_envoy_extensions_access_loggers_file_v3_file_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_access_loggers_file_v3_file_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_access_loggers_file_v3_file_proto_rawDesc), len(file_envoy_extensions_access_loggers_file_v3_file_proto_rawDesc)))
	})
	return file_envoy_extensions_access_loggers_file_v3_file_proto_rawDescData
}

var file_envoy_extensions_access_loggers_file_v3_file_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_access_loggers_file_v3_file_proto_goTypes = []any{
	(*FileAccessLog)(nil),               // 0: envoy.extensions.access_loggers.file.v3.FileAccessLog
	(*structpb.Struct)(nil),             // 1: google.protobuf.Struct
	(*v3.SubstitutionFormatString)(nil), // 2: envoy.config.core.v3.SubstitutionFormatString
}
var file_envoy_extensions_access_loggers_file_v3_file_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.access_loggers.file.v3.FileAccessLog.json_format:type_name -> google.protobuf.Struct
	1, // 1: envoy.extensions.access_loggers.file.v3.FileAccessLog.typed_json_format:type_name -> google.protobuf.Struct
	2, // 2: envoy.extensions.access_loggers.file.v3.FileAccessLog.log_format:type_name -> envoy.config.core.v3.SubstitutionFormatString
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_envoy_extensions_access_loggers_file_v3_file_proto_init() }
func file_envoy_extensions_access_loggers_file_v3_file_proto_init() {
	if File_envoy_extensions_access_loggers_file_v3_file_proto != nil {
		return
	}
	file_envoy_extensions_access_loggers_file_v3_file_proto_msgTypes[0].OneofWrappers = []any{
		(*FileAccessLog_Format)(nil),
		(*FileAccessLog_JsonFormat)(nil),
		(*FileAccessLog_TypedJsonFormat)(nil),
		(*FileAccessLog_LogFormat)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_access_loggers_file_v3_file_proto_rawDesc), len(file_envoy_extensions_access_loggers_file_v3_file_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_access_loggers_file_v3_file_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_access_loggers_file_v3_file_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_access_loggers_file_v3_file_proto_msgTypes,
	}.Build()
	File_envoy_extensions_access_loggers_file_v3_file_proto = out.File
	file_envoy_extensions_access_loggers_file_v3_file_proto_goTypes = nil
	file_envoy_extensions_access_loggers_file_v3_file_proto_depIdxs = nil
}
