// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/http/original_ip_detection/custom_header/v3/custom_header.proto

package custom_headerv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/type/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// This extension allows for the original downstream remote IP to be detected
// by reading the value from a configured header name. If the value is successfully parsed
// as an IP, it'll be treated as the effective downstream remote address and seen as such
// by all filters. See :ref:`original_ip_detection_extensions
// <envoy_v3_api_field_extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.original_ip_detection_extensions>`
// for an overview of how extensions operate and what happens when an extension fails
// to detect the remote IP.
//
// [#extension: envoy.http.original_ip_detection.custom_header]
type CustomHeaderConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The header name containing the original downstream remote address, if present.
	//
	// Note: in the case of a multi-valued header, only the first value is tried and the rest are ignored.
	HeaderName string `protobuf:"bytes,1,opt,name=header_name,json=headerName,proto3" json:"header_name,omitempty"`
	// If set to true, the extension could decide that the detected address should be treated as
	// trusted by the HCM. If the address is considered :ref:`trusted<config_http_conn_man_headers_x-forwarded-for_trusted_client_address>`,
	// it might be used as input to determine if the request is internal (among other things).
	AllowExtensionToSetAddressAsTrusted bool `protobuf:"varint,2,opt,name=allow_extension_to_set_address_as_trusted,json=allowExtensionToSetAddressAsTrusted,proto3" json:"allow_extension_to_set_address_as_trusted,omitempty"`
	// If this is set, the request will be rejected when detection fails using it as the HTTP response status.
	//
	// .. note::
	//
	//	If this is set to < 400 or > 511, the default status 403 will be used instead.
	RejectWithStatus *v3.HttpStatus `protobuf:"bytes,3,opt,name=reject_with_status,json=rejectWithStatus,proto3" json:"reject_with_status,omitempty"`
	unknownFields    protoimpl.UnknownFields
	sizeCache        protoimpl.SizeCache
}

func (x *CustomHeaderConfig) Reset() {
	*x = CustomHeaderConfig{}
	mi := &file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CustomHeaderConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CustomHeaderConfig) ProtoMessage() {}

func (x *CustomHeaderConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CustomHeaderConfig.ProtoReflect.Descriptor instead.
func (*CustomHeaderConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDescGZIP(), []int{0}
}

func (x *CustomHeaderConfig) GetHeaderName() string {
	if x != nil {
		return x.HeaderName
	}
	return ""
}

func (x *CustomHeaderConfig) GetAllowExtensionToSetAddressAsTrusted() bool {
	if x != nil {
		return x.AllowExtensionToSetAddressAsTrusted
	}
	return false
}

func (x *CustomHeaderConfig) GetRejectWithStatus() *v3.HttpStatus {
	if x != nil {
		return x.RejectWithStatus
	}
	return nil
}

var File_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto protoreflect.FileDescriptor

const file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDesc = "" +
	"\n" +
	"Penvoy/extensions/http/original_ip_detection/custom_header/v3/custom_header.proto\x12<envoy.extensions.http.original_ip_detection.custom_header.v3\x1a\x1fenvoy/type/v3/http_status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xe5\x01\n" +
	"\x12CustomHeaderConfig\x12.\n" +
	"\vheader_name\x18\x01 \x01(\tB\r\xfaB\n" +
	"r\b\x10\x01\xc8\x01\x01\xc0\x01\x01R\n" +
	"headerName\x12V\n" +
	")allow_extension_to_set_address_as_trusted\x18\x02 \x01(\bR#allowExtensionToSetAddressAsTrusted\x12G\n" +
	"\x12reject_with_status\x18\x03 \x01(\v2\x19.envoy.type.v3.HttpStatusR\x10rejectWithStatusB\xde\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Jio.envoyproxy.envoy.extensions.http.original_ip_detection.custom_header.v3B\x11CustomHeaderProtoP\x01Zsgithub.com/envoyproxy/go-control-plane/envoy/extensions/http/original_ip_detection/custom_header/v3;custom_headerv3b\x06proto3"

var (
	file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDescOnce sync.Once
	file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDescData []byte
)

func file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDescGZIP() []byte {
	file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDesc), len(file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDesc)))
	})
	return file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDescData
}

var file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_goTypes = []any{
	(*CustomHeaderConfig)(nil), // 0: envoy.extensions.http.original_ip_detection.custom_header.v3.CustomHeaderConfig
	(*v3.HttpStatus)(nil),      // 1: envoy.type.v3.HttpStatus
}
var file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.http.original_ip_detection.custom_header.v3.CustomHeaderConfig.reject_with_status:type_name -> envoy.type.v3.HttpStatus
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_init()
}
func file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_init() {
	if File_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDesc), len(file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_msgTypes,
	}.Build()
	File_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto = out.File
	file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_goTypes = nil
	file_envoy_extensions_http_original_ip_detection_custom_header_v3_custom_header_proto_depIdxs = nil
}
