// Code generated by go-swagger; DO NOT EDIT.

// Copyright Authors of Cilium
// SPDX-License-Identifier: Apache-2.0

package daemon

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the generate command

import (
	"net/http"

	"github.com/go-openapi/runtime/middleware"
)

// GetMapHandlerFunc turns a function with the right signature into a get map handler
type GetMapHandlerFunc func(GetMapParams) middleware.Responder

// Handle executing the request and returning a response
func (fn GetMapHandlerFunc) Handle(params GetMapParams) middleware.Responder {
	return fn(params)
}

// GetMapHandler interface for that can handle valid get map params
type GetMapHandler interface {
	Handle(GetMapParams) middleware.Responder
}

// NewGetMap creates a new http.Handler for the get map operation
func NewGetMap(ctx *middleware.Context, handler GetMapHandler) *GetMap {
	return &GetMap{Context: ctx, Handler: handler}
}

/*
	GetMap swagger:route GET /map daemon getMap

List all open maps
*/
type GetMap struct {
	Context *middleware.Context
	Handler GetMapHandler
}

func (o *GetMap) ServeHTTP(rw http.ResponseWriter, r *http.Request) {
	route, rCtx, _ := o.Context.RouteInfo(r)
	if rCtx != nil {
		*r = *rCtx
	}
	var Params = NewGetMapParams()
	if err := o.Context.BindValidRequest(r, route, &Params); err != nil { // bind params
		o.Context.Respond(rw, r, route.Produces, route, err)
		return
	}

	res := o.Handler.Handle(Params) // actually handle the request

	o.Context.Respond(rw, r, route.Produces, route, res)

}
