// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/thrift_proxy/filters/payload_to_metadata/v3/payload_to_metadata.proto

package payload_to_metadatav3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/type/matcher/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type PayloadToMetadata_ValueType int32

const (
	PayloadToMetadata_STRING PayloadToMetadata_ValueType = 0
	PayloadToMetadata_NUMBER PayloadToMetadata_ValueType = 1
)

// Enum value maps for PayloadToMetadata_ValueType.
var (
	PayloadToMetadata_ValueType_name = map[int32]string{
		0: "STRING",
		1: "NUMBER",
	}
	PayloadToMetadata_ValueType_value = map[string]int32{
		"STRING": 0,
		"NUMBER": 1,
	}
)

func (x PayloadToMetadata_ValueType) Enum() *PayloadToMetadata_ValueType {
	p := new(PayloadToMetadata_ValueType)
	*p = x
	return p
}

func (x PayloadToMetadata_ValueType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (PayloadToMetadata_ValueType) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_enumTypes[0].Descriptor()
}

func (PayloadToMetadata_ValueType) Type() protoreflect.EnumType {
	return &file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_enumTypes[0]
}

func (x PayloadToMetadata_ValueType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use PayloadToMetadata_ValueType.Descriptor instead.
func (PayloadToMetadata_ValueType) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDescGZIP(), []int{0, 0}
}

type PayloadToMetadata struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The list of rules to apply to requests.
	RequestRules  []*PayloadToMetadata_Rule `protobuf:"bytes,1,rep,name=request_rules,json=requestRules,proto3" json:"request_rules,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PayloadToMetadata) Reset() {
	*x = PayloadToMetadata{}
	mi := &file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PayloadToMetadata) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PayloadToMetadata) ProtoMessage() {}

func (x *PayloadToMetadata) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PayloadToMetadata.ProtoReflect.Descriptor instead.
func (*PayloadToMetadata) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDescGZIP(), []int{0}
}

func (x *PayloadToMetadata) GetRequestRules() []*PayloadToMetadata_Rule {
	if x != nil {
		return x.RequestRules
	}
	return nil
}

// [#next-free-field: 6]
type PayloadToMetadata_KeyValuePair struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The namespace — if this is empty, the filter's namespace will be used.
	MetadataNamespace string `protobuf:"bytes,1,opt,name=metadata_namespace,json=metadataNamespace,proto3" json:"metadata_namespace,omitempty"`
	// The key to use within the namespace.
	Key string `protobuf:"bytes,2,opt,name=key,proto3" json:"key,omitempty"`
	// Types that are valid to be assigned to ValueType:
	//
	//	*PayloadToMetadata_KeyValuePair_Value
	//	*PayloadToMetadata_KeyValuePair_RegexValueRewrite
	ValueType isPayloadToMetadata_KeyValuePair_ValueType `protobuf_oneof:"value_type"`
	// The value's type — defaults to string.
	Type          PayloadToMetadata_ValueType `protobuf:"varint,5,opt,name=type,proto3,enum=envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata_ValueType" json:"type,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PayloadToMetadata_KeyValuePair) Reset() {
	*x = PayloadToMetadata_KeyValuePair{}
	mi := &file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PayloadToMetadata_KeyValuePair) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PayloadToMetadata_KeyValuePair) ProtoMessage() {}

func (x *PayloadToMetadata_KeyValuePair) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PayloadToMetadata_KeyValuePair.ProtoReflect.Descriptor instead.
func (*PayloadToMetadata_KeyValuePair) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDescGZIP(), []int{0, 0}
}

func (x *PayloadToMetadata_KeyValuePair) GetMetadataNamespace() string {
	if x != nil {
		return x.MetadataNamespace
	}
	return ""
}

func (x *PayloadToMetadata_KeyValuePair) GetKey() string {
	if x != nil {
		return x.Key
	}
	return ""
}

func (x *PayloadToMetadata_KeyValuePair) GetValueType() isPayloadToMetadata_KeyValuePair_ValueType {
	if x != nil {
		return x.ValueType
	}
	return nil
}

func (x *PayloadToMetadata_KeyValuePair) GetValue() string {
	if x != nil {
		if x, ok := x.ValueType.(*PayloadToMetadata_KeyValuePair_Value); ok {
			return x.Value
		}
	}
	return ""
}

func (x *PayloadToMetadata_KeyValuePair) GetRegexValueRewrite() *v3.RegexMatchAndSubstitute {
	if x != nil {
		if x, ok := x.ValueType.(*PayloadToMetadata_KeyValuePair_RegexValueRewrite); ok {
			return x.RegexValueRewrite
		}
	}
	return nil
}

func (x *PayloadToMetadata_KeyValuePair) GetType() PayloadToMetadata_ValueType {
	if x != nil {
		return x.Type
	}
	return PayloadToMetadata_STRING
}

type isPayloadToMetadata_KeyValuePair_ValueType interface {
	isPayloadToMetadata_KeyValuePair_ValueType()
}

type PayloadToMetadata_KeyValuePair_Value struct {
	// The value to pair with the given key.
	//
	// When used for on_present case, if value is non-empty it'll be used instead
	// of the field value. If both are empty, the field value is used as-is.
	//
	// When used for on_missing case, a non-empty value must be provided.
	Value string `protobuf:"bytes,3,opt,name=value,proto3,oneof"`
}

type PayloadToMetadata_KeyValuePair_RegexValueRewrite struct {
	// If present, the header's value will be matched and substituted with this.
	// If there is no match or substitution, the field value is used as-is.
	//
	// This is only used for on_present.
	RegexValueRewrite *v3.RegexMatchAndSubstitute `protobuf:"bytes,4,opt,name=regex_value_rewrite,json=regexValueRewrite,proto3,oneof"`
}

func (*PayloadToMetadata_KeyValuePair_Value) isPayloadToMetadata_KeyValuePair_ValueType() {}

func (*PayloadToMetadata_KeyValuePair_RegexValueRewrite) isPayloadToMetadata_KeyValuePair_ValueType() {
}

// A Rule defines what metadata to apply when a field is present or missing.
// [#next-free-field: 6]
type PayloadToMetadata_Rule struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to MatchSpecifier:
	//
	//	*PayloadToMetadata_Rule_MethodName
	//	*PayloadToMetadata_Rule_ServiceName
	MatchSpecifier isPayloadToMetadata_Rule_MatchSpecifier `protobuf_oneof:"match_specifier"`
	// Specifies that a match will be performed on the value of a field.
	FieldSelector *PayloadToMetadata_FieldSelector `protobuf:"bytes,3,opt,name=field_selector,json=fieldSelector,proto3" json:"field_selector,omitempty"`
	// If the field is present, apply this metadata KeyValuePair.
	OnPresent *PayloadToMetadata_KeyValuePair `protobuf:"bytes,4,opt,name=on_present,json=onPresent,proto3" json:"on_present,omitempty"`
	// If the field is missing, apply this metadata KeyValuePair.
	//
	// The value in the KeyValuePair must be set, since it'll be used in lieu
	// of the missing field value.
	OnMissing     *PayloadToMetadata_KeyValuePair `protobuf:"bytes,5,opt,name=on_missing,json=onMissing,proto3" json:"on_missing,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PayloadToMetadata_Rule) Reset() {
	*x = PayloadToMetadata_Rule{}
	mi := &file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PayloadToMetadata_Rule) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PayloadToMetadata_Rule) ProtoMessage() {}

func (x *PayloadToMetadata_Rule) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PayloadToMetadata_Rule.ProtoReflect.Descriptor instead.
func (*PayloadToMetadata_Rule) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDescGZIP(), []int{0, 1}
}

func (x *PayloadToMetadata_Rule) GetMatchSpecifier() isPayloadToMetadata_Rule_MatchSpecifier {
	if x != nil {
		return x.MatchSpecifier
	}
	return nil
}

func (x *PayloadToMetadata_Rule) GetMethodName() string {
	if x != nil {
		if x, ok := x.MatchSpecifier.(*PayloadToMetadata_Rule_MethodName); ok {
			return x.MethodName
		}
	}
	return ""
}

func (x *PayloadToMetadata_Rule) GetServiceName() string {
	if x != nil {
		if x, ok := x.MatchSpecifier.(*PayloadToMetadata_Rule_ServiceName); ok {
			return x.ServiceName
		}
	}
	return ""
}

func (x *PayloadToMetadata_Rule) GetFieldSelector() *PayloadToMetadata_FieldSelector {
	if x != nil {
		return x.FieldSelector
	}
	return nil
}

func (x *PayloadToMetadata_Rule) GetOnPresent() *PayloadToMetadata_KeyValuePair {
	if x != nil {
		return x.OnPresent
	}
	return nil
}

func (x *PayloadToMetadata_Rule) GetOnMissing() *PayloadToMetadata_KeyValuePair {
	if x != nil {
		return x.OnMissing
	}
	return nil
}

type isPayloadToMetadata_Rule_MatchSpecifier interface {
	isPayloadToMetadata_Rule_MatchSpecifier()
}

type PayloadToMetadata_Rule_MethodName struct {
	// If specified, the route must exactly match the request method name. As a special case,
	// an empty string matches any request method name.
	MethodName string `protobuf:"bytes,1,opt,name=method_name,json=methodName,proto3,oneof"`
}

type PayloadToMetadata_Rule_ServiceName struct {
	// If specified, the route must have the service name as the request method name prefix.
	// As a special case, an empty string matches any service name. Only relevant when service
	// multiplexing.
	ServiceName string `protobuf:"bytes,2,opt,name=service_name,json=serviceName,proto3,oneof"`
}

func (*PayloadToMetadata_Rule_MethodName) isPayloadToMetadata_Rule_MatchSpecifier() {}

func (*PayloadToMetadata_Rule_ServiceName) isPayloadToMetadata_Rule_MatchSpecifier() {}

type PayloadToMetadata_FieldSelector struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// field name to log
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// field id to match
	Id int32 `protobuf:"varint,2,opt,name=id,proto3" json:"id,omitempty"`
	// next node of the field selector
	Child         *PayloadToMetadata_FieldSelector `protobuf:"bytes,3,opt,name=child,proto3" json:"child,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PayloadToMetadata_FieldSelector) Reset() {
	*x = PayloadToMetadata_FieldSelector{}
	mi := &file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PayloadToMetadata_FieldSelector) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PayloadToMetadata_FieldSelector) ProtoMessage() {}

func (x *PayloadToMetadata_FieldSelector) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PayloadToMetadata_FieldSelector.ProtoReflect.Descriptor instead.
func (*PayloadToMetadata_FieldSelector) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDescGZIP(), []int{0, 2}
}

func (x *PayloadToMetadata_FieldSelector) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *PayloadToMetadata_FieldSelector) GetId() int32 {
	if x != nil {
		return x.Id
	}
	return 0
}

func (x *PayloadToMetadata_FieldSelector) GetChild() *PayloadToMetadata_FieldSelector {
	if x != nil {
		return x.Child
	}
	return nil
}

var File_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDesc = "" +
	"\n" +
	"fenvoy/extensions/filters/network/thrift_proxy/filters/payload_to_metadata/v3/payload_to_metadata.proto\x12Lenvoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3\x1a!envoy/type/matcher/v3/regex.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xbc\n" +
	"\n" +
	"\x11PayloadToMetadata\x12\x93\x01\n" +
	"\rrequest_rules\x18\x01 \x03(\v2d.envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.RuleB\b\xfaB\x05\x92\x01\x02\b\x01R\frequestRules\x1a\xea\x02\n" +
	"\fKeyValuePair\x12-\n" +
	"\x12metadata_namespace\x18\x01 \x01(\tR\x11metadataNamespace\x12\x19\n" +
	"\x03key\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x03key\x12\x16\n" +
	"\x05value\x18\x03 \x01(\tH\x00R\x05value\x12`\n" +
	"\x13regex_value_rewrite\x18\x04 \x01(\v2..envoy.type.matcher.v3.RegexMatchAndSubstituteH\x00R\x11regexValueRewrite\x12\x87\x01\n" +
	"\x04type\x18\x05 \x01(\x0e2i.envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.ValueTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\x04typeB\f\n" +
	"\n" +
	"value_type\x1a\xa3\x04\n" +
	"\x04Rule\x12!\n" +
	"\vmethod_name\x18\x01 \x01(\tH\x00R\n" +
	"methodName\x12#\n" +
	"\fservice_name\x18\x02 \x01(\tH\x00R\vserviceName\x12\x9e\x01\n" +
	"\x0efield_selector\x18\x03 \x01(\v2m.envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.FieldSelectorB\b\xfaB\x05\x8a\x01\x02\x10\x01R\rfieldSelector\x12\x8b\x01\n" +
	"\n" +
	"on_present\x18\x04 \x01(\v2l.envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.KeyValuePairR\tonPresent\x12\x8b\x01\n" +
	"\n" +
	"on_missing\x18\x05 \x01(\v2l.envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.KeyValuePairR\tonMissingB\x16\n" +
	"\x0fmatch_specifier\x12\x03\xf8B\x01\x1a\xd8\x01\n" +
	"\rFieldSelector\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04name\x12$\n" +
	"\x02id\x18\x02 \x01(\x05B\x14\xfaB\x11\x1a\x0f\x18\xff\xff\x01(\x80\x80\xfe\xff\xff\xff\xff\xff\xff\x01R\x02id\x12\x83\x01\n" +
	"\x05child\x18\x03 \x01(\v2m.envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.FieldSelectorR\x05child\"#\n" +
	"\tValueType\x12\n" +
	"\n" +
	"\x06STRING\x10\x00\x12\n" +
	"\n" +
	"\x06NUMBER\x10\x01B\x8a\x02\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Zio.envoyproxy.envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3B\x16PayloadToMetadataProtoP\x01Z\x89\x01github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/thrift_proxy/filters/payload_to_metadata/v3;payload_to_metadatav3b\x06proto3"

var (
	file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDesc), len(file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDescData
}

var file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes = make([]protoimpl.MessageInfo, 4)
var file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_goTypes = []any{
	(PayloadToMetadata_ValueType)(0),        // 0: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.ValueType
	(*PayloadToMetadata)(nil),               // 1: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata
	(*PayloadToMetadata_KeyValuePair)(nil),  // 2: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.KeyValuePair
	(*PayloadToMetadata_Rule)(nil),          // 3: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.Rule
	(*PayloadToMetadata_FieldSelector)(nil), // 4: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.FieldSelector
	(*v3.RegexMatchAndSubstitute)(nil),      // 5: envoy.type.matcher.v3.RegexMatchAndSubstitute
}
var file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_depIdxs = []int32{
	3, // 0: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.request_rules:type_name -> envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.Rule
	5, // 1: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.KeyValuePair.regex_value_rewrite:type_name -> envoy.type.matcher.v3.RegexMatchAndSubstitute
	0, // 2: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.KeyValuePair.type:type_name -> envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.ValueType
	4, // 3: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.Rule.field_selector:type_name -> envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.FieldSelector
	2, // 4: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.Rule.on_present:type_name -> envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.KeyValuePair
	2, // 5: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.Rule.on_missing:type_name -> envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.KeyValuePair
	4, // 6: envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.FieldSelector.child:type_name -> envoy.extensions.filters.network.thrift_proxy.filters.payload_to_metadata.v3.PayloadToMetadata.FieldSelector
	7, // [7:7] is the sub-list for method output_type
	7, // [7:7] is the sub-list for method input_type
	7, // [7:7] is the sub-list for extension type_name
	7, // [7:7] is the sub-list for extension extendee
	0, // [0:7] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_init()
}
func file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_init() {
	if File_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto != nil {
		return
	}
	file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes[1].OneofWrappers = []any{
		(*PayloadToMetadata_KeyValuePair_Value)(nil),
		(*PayloadToMetadata_KeyValuePair_RegexValueRewrite)(nil),
	}
	file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes[2].OneofWrappers = []any{
		(*PayloadToMetadata_Rule_MethodName)(nil),
		(*PayloadToMetadata_Rule_ServiceName)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDesc), len(file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   4,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_depIdxs,
		EnumInfos:         file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_enumTypes,
		MessageInfos:      file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto = out.File
	file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_goTypes = nil
	file_envoy_extensions_filters_network_thrift_proxy_filters_payload_to_metadata_v3_payload_to_metadata_proto_depIdxs = nil
}
