// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/common/tap/v3/common.proto

package tapv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/tap/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Common configuration for all tap extensions.
type CommonExtensionConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to ConfigType:
	//
	//	*CommonExtensionConfig_AdminConfig
	//	*CommonExtensionConfig_StaticConfig
	ConfigType    isCommonExtensionConfig_ConfigType `protobuf_oneof:"config_type"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CommonExtensionConfig) Reset() {
	*x = CommonExtensionConfig{}
	mi := &file_envoy_extensions_common_tap_v3_common_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CommonExtensionConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CommonExtensionConfig) ProtoMessage() {}

func (x *CommonExtensionConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_common_tap_v3_common_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CommonExtensionConfig.ProtoReflect.Descriptor instead.
func (*CommonExtensionConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_common_tap_v3_common_proto_rawDescGZIP(), []int{0}
}

func (x *CommonExtensionConfig) GetConfigType() isCommonExtensionConfig_ConfigType {
	if x != nil {
		return x.ConfigType
	}
	return nil
}

func (x *CommonExtensionConfig) GetAdminConfig() *AdminConfig {
	if x != nil {
		if x, ok := x.ConfigType.(*CommonExtensionConfig_AdminConfig); ok {
			return x.AdminConfig
		}
	}
	return nil
}

func (x *CommonExtensionConfig) GetStaticConfig() *v3.TapConfig {
	if x != nil {
		if x, ok := x.ConfigType.(*CommonExtensionConfig_StaticConfig); ok {
			return x.StaticConfig
		}
	}
	return nil
}

type isCommonExtensionConfig_ConfigType interface {
	isCommonExtensionConfig_ConfigType()
}

type CommonExtensionConfig_AdminConfig struct {
	// If specified, the tap filter will be configured via an admin handler.
	AdminConfig *AdminConfig `protobuf:"bytes,1,opt,name=admin_config,json=adminConfig,proto3,oneof"`
}

type CommonExtensionConfig_StaticConfig struct {
	// If specified, the tap filter will be configured via a static configuration that cannot be
	// changed.
	StaticConfig *v3.TapConfig `protobuf:"bytes,2,opt,name=static_config,json=staticConfig,proto3,oneof"`
}

func (*CommonExtensionConfig_AdminConfig) isCommonExtensionConfig_ConfigType() {}

func (*CommonExtensionConfig_StaticConfig) isCommonExtensionConfig_ConfigType() {}

// Configuration for the admin handler. See :ref:`here <config_http_filters_tap_admin_handler>` for
// more information.
type AdminConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Opaque configuration ID. When requests are made to the admin handler, the passed opaque ID is
	// matched to the configured filter opaque ID to determine which filter to configure.
	ConfigId      string `protobuf:"bytes,1,opt,name=config_id,json=configId,proto3" json:"config_id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *AdminConfig) Reset() {
	*x = AdminConfig{}
	mi := &file_envoy_extensions_common_tap_v3_common_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AdminConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AdminConfig) ProtoMessage() {}

func (x *AdminConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_common_tap_v3_common_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AdminConfig.ProtoReflect.Descriptor instead.
func (*AdminConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_common_tap_v3_common_proto_rawDescGZIP(), []int{1}
}

func (x *AdminConfig) GetConfigId() string {
	if x != nil {
		return x.ConfigId
	}
	return ""
}

var File_envoy_extensions_common_tap_v3_common_proto protoreflect.FileDescriptor

const file_envoy_extensions_common_tap_v3_common_proto_rawDesc = "" +
	"\n" +
	"+envoy/extensions/common/tap/v3/common.proto\x12\x1eenvoy.extensions.common.tap.v3\x1a envoy/config/tap/v3/common.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x82\x02\n" +
	"\x15CommonExtensionConfig\x12P\n" +
	"\fadmin_config\x18\x01 \x01(\v2+.envoy.extensions.common.tap.v3.AdminConfigH\x00R\vadminConfig\x12E\n" +
	"\rstatic_config\x18\x02 \x01(\v2\x1e.envoy.config.tap.v3.TapConfigH\x00R\fstaticConfig:<\x9aň\x1e7\n" +
	"5envoy.config.common.tap.v2alpha.CommonExtensionConfigB\x12\n" +
	"\vconfig_type\x12\x03\xf8B\x01\"g\n" +
	"\vAdminConfig\x12$\n" +
	"\tconfig_id\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\bconfigId:2\x9aň\x1e-\n" +
	"+envoy.config.common.tap.v2alpha.AdminConfigB\x92\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	",io.envoyproxy.envoy.extensions.common.tap.v3B\vCommonProtoP\x01ZKgithub.com/envoyproxy/go-control-plane/envoy/extensions/common/tap/v3;tapv3b\x06proto3"

var (
	file_envoy_extensions_common_tap_v3_common_proto_rawDescOnce sync.Once
	file_envoy_extensions_common_tap_v3_common_proto_rawDescData []byte
)

func file_envoy_extensions_common_tap_v3_common_proto_rawDescGZIP() []byte {
	file_envoy_extensions_common_tap_v3_common_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_common_tap_v3_common_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_common_tap_v3_common_proto_rawDesc), len(file_envoy_extensions_common_tap_v3_common_proto_rawDesc)))
	})
	return file_envoy_extensions_common_tap_v3_common_proto_rawDescData
}

var file_envoy_extensions_common_tap_v3_common_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_common_tap_v3_common_proto_goTypes = []any{
	(*CommonExtensionConfig)(nil), // 0: envoy.extensions.common.tap.v3.CommonExtensionConfig
	(*AdminConfig)(nil),           // 1: envoy.extensions.common.tap.v3.AdminConfig
	(*v3.TapConfig)(nil),          // 2: envoy.config.tap.v3.TapConfig
}
var file_envoy_extensions_common_tap_v3_common_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.common.tap.v3.CommonExtensionConfig.admin_config:type_name -> envoy.extensions.common.tap.v3.AdminConfig
	2, // 1: envoy.extensions.common.tap.v3.CommonExtensionConfig.static_config:type_name -> envoy.config.tap.v3.TapConfig
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_extensions_common_tap_v3_common_proto_init() }
func file_envoy_extensions_common_tap_v3_common_proto_init() {
	if File_envoy_extensions_common_tap_v3_common_proto != nil {
		return
	}
	file_envoy_extensions_common_tap_v3_common_proto_msgTypes[0].OneofWrappers = []any{
		(*CommonExtensionConfig_AdminConfig)(nil),
		(*CommonExtensionConfig_StaticConfig)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_common_tap_v3_common_proto_rawDesc), len(file_envoy_extensions_common_tap_v3_common_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_common_tap_v3_common_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_common_tap_v3_common_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_common_tap_v3_common_proto_msgTypes,
	}.Build()
	File_envoy_extensions_common_tap_v3_common_proto = out.File
	file_envoy_extensions_common_tap_v3_common_proto_goTypes = nil
	file_envoy_extensions_common_tap_v3_common_proto_depIdxs = nil
}
