// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/generic_proxy/matcher/v3/matcher.proto

package matcherv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/type/matcher/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Used to match request service of the downstream request. Only applicable if a service provided
// by the application protocol.
// This is deprecated and should be replaced by HostMatchInput. This is kept for backward compatibility.
type ServiceMatchInput struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ServiceMatchInput) Reset() {
	*x = ServiceMatchInput{}
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ServiceMatchInput) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ServiceMatchInput) ProtoMessage() {}

func (x *ServiceMatchInput) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ServiceMatchInput.ProtoReflect.Descriptor instead.
func (*ServiceMatchInput) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescGZIP(), []int{0}
}

// Used to match request host of the generic downstream request. Only applicable if a host provided
// by the application protocol.
// This is same with the ServiceMatchInput and this should be preferred over ServiceMatchInput.
type HostMatchInput struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HostMatchInput) Reset() {
	*x = HostMatchInput{}
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HostMatchInput) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HostMatchInput) ProtoMessage() {}

func (x *HostMatchInput) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HostMatchInput.ProtoReflect.Descriptor instead.
func (*HostMatchInput) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescGZIP(), []int{1}
}

// Used to match request path of the generic downstream request. Only applicable if a path provided
// by the application protocol.
type PathMatchInput struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PathMatchInput) Reset() {
	*x = PathMatchInput{}
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PathMatchInput) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PathMatchInput) ProtoMessage() {}

func (x *PathMatchInput) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PathMatchInput.ProtoReflect.Descriptor instead.
func (*PathMatchInput) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescGZIP(), []int{2}
}

// Used to match request method of the generic downstream request. Only applicable if a method provided
// by the application protocol.
type MethodMatchInput struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *MethodMatchInput) Reset() {
	*x = MethodMatchInput{}
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *MethodMatchInput) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*MethodMatchInput) ProtoMessage() {}

func (x *MethodMatchInput) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use MethodMatchInput.ProtoReflect.Descriptor instead.
func (*MethodMatchInput) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescGZIP(), []int{3}
}

// Used to match an arbitrary property of the generic downstream request.
// These properties are populated by the codecs of application protocols.
type PropertyMatchInput struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The property name to match on.
	PropertyName  string `protobuf:"bytes,1,opt,name=property_name,json=propertyName,proto3" json:"property_name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *PropertyMatchInput) Reset() {
	*x = PropertyMatchInput{}
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *PropertyMatchInput) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*PropertyMatchInput) ProtoMessage() {}

func (x *PropertyMatchInput) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use PropertyMatchInput.ProtoReflect.Descriptor instead.
func (*PropertyMatchInput) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescGZIP(), []int{4}
}

func (x *PropertyMatchInput) GetPropertyName() string {
	if x != nil {
		return x.PropertyName
	}
	return ""
}

// Used to match an whole generic downstream request.
type RequestMatchInput struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RequestMatchInput) Reset() {
	*x = RequestMatchInput{}
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RequestMatchInput) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RequestMatchInput) ProtoMessage() {}

func (x *RequestMatchInput) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RequestMatchInput.ProtoReflect.Descriptor instead.
func (*RequestMatchInput) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescGZIP(), []int{5}
}

// Used to match an arbitrary key-value pair for headers, trailers or properties.
type KeyValueMatchEntry struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The key name to match on.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// The key value pattern.
	StringMatch   *v3.StringMatcher `protobuf:"bytes,2,opt,name=string_match,json=stringMatch,proto3" json:"string_match,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *KeyValueMatchEntry) Reset() {
	*x = KeyValueMatchEntry{}
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *KeyValueMatchEntry) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*KeyValueMatchEntry) ProtoMessage() {}

func (x *KeyValueMatchEntry) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use KeyValueMatchEntry.ProtoReflect.Descriptor instead.
func (*KeyValueMatchEntry) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescGZIP(), []int{6}
}

func (x *KeyValueMatchEntry) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *KeyValueMatchEntry) GetStringMatch() *v3.StringMatcher {
	if x != nil {
		return x.StringMatch
	}
	return nil
}

// Custom matcher to match on the generic downstream request. This is used to match
// multiple fields of the downstream request and avoid complex combinations of
// HostMatchInput, PathMatchInput, MethodMatchInput and PropertyMatchInput.
type RequestMatcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Optional host pattern to match on. If not specified, any host will match.
	Host *v3.StringMatcher `protobuf:"bytes,1,opt,name=host,proto3" json:"host,omitempty"`
	// Optional path pattern to match on. If not specified, any path will match.
	Path *v3.StringMatcher `protobuf:"bytes,2,opt,name=path,proto3" json:"path,omitempty"`
	// Optional method pattern to match on. If not specified, any method will match.
	Method *v3.StringMatcher `protobuf:"bytes,3,opt,name=method,proto3" json:"method,omitempty"`
	// Optional arbitrary properties to match on. If not specified, any properties
	// will match. The key is the property name and the value is the property value
	// to match on.
	Properties    []*KeyValueMatchEntry `protobuf:"bytes,4,rep,name=properties,proto3" json:"properties,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RequestMatcher) Reset() {
	*x = RequestMatcher{}
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RequestMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RequestMatcher) ProtoMessage() {}

func (x *RequestMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RequestMatcher.ProtoReflect.Descriptor instead.
func (*RequestMatcher) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescGZIP(), []int{7}
}

func (x *RequestMatcher) GetHost() *v3.StringMatcher {
	if x != nil {
		return x.Host
	}
	return nil
}

func (x *RequestMatcher) GetPath() *v3.StringMatcher {
	if x != nil {
		return x.Path
	}
	return nil
}

func (x *RequestMatcher) GetMethod() *v3.StringMatcher {
	if x != nil {
		return x.Method
	}
	return nil
}

func (x *RequestMatcher) GetProperties() []*KeyValueMatchEntry {
	if x != nil {
		return x.Properties
	}
	return nil
}

var File_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDesc = "" +
	"\n" +
	"Genvoy/extensions/filters/network/generic_proxy/matcher/v3/matcher.proto\x129envoy.extensions.filters.network.generic_proxy.matcher.v3\x1a\"envoy/type/matcher/v3/string.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x13\n" +
	"\x11ServiceMatchInput\"\x10\n" +
	"\x0eHostMatchInput\"\x10\n" +
	"\x0ePathMatchInput\"\x12\n" +
	"\x10MethodMatchInput\"B\n" +
	"\x12PropertyMatchInput\x12,\n" +
	"\rproperty_name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\fpropertyName\"\x13\n" +
	"\x11RequestMatchInput\"\x84\x01\n" +
	"\x12KeyValueMatchEntry\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04name\x12Q\n" +
	"\fstring_match\x18\x02 \x01(\v2$.envoy.type.matcher.v3.StringMatcherB\b\xfaB\x05\x8a\x01\x02\x10\x01R\vstringMatch\"\xb1\x02\n" +
	"\x0eRequestMatcher\x128\n" +
	"\x04host\x18\x01 \x01(\v2$.envoy.type.matcher.v3.StringMatcherR\x04host\x128\n" +
	"\x04path\x18\x02 \x01(\v2$.envoy.type.matcher.v3.StringMatcherR\x04path\x12<\n" +
	"\x06method\x18\x03 \x01(\v2$.envoy.type.matcher.v3.StringMatcherR\x06method\x12m\n" +
	"\n" +
	"properties\x18\x04 \x03(\v2M.envoy.extensions.filters.network.generic_proxy.matcher.v3.KeyValueMatchEntryR\n" +
	"propertiesB\xd5\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"Gio.envoyproxy.envoy.extensions.filters.network.generic_proxy.matcher.v3B\fMatcherProtoP\x01Zjgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/generic_proxy/matcher/v3;matcherv3b\x06proto3"

var (
	file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDesc), len(file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDescData
}

var file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes = make([]protoimpl.MessageInfo, 8)
var file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_goTypes = []any{
	(*ServiceMatchInput)(nil),  // 0: envoy.extensions.filters.network.generic_proxy.matcher.v3.ServiceMatchInput
	(*HostMatchInput)(nil),     // 1: envoy.extensions.filters.network.generic_proxy.matcher.v3.HostMatchInput
	(*PathMatchInput)(nil),     // 2: envoy.extensions.filters.network.generic_proxy.matcher.v3.PathMatchInput
	(*MethodMatchInput)(nil),   // 3: envoy.extensions.filters.network.generic_proxy.matcher.v3.MethodMatchInput
	(*PropertyMatchInput)(nil), // 4: envoy.extensions.filters.network.generic_proxy.matcher.v3.PropertyMatchInput
	(*RequestMatchInput)(nil),  // 5: envoy.extensions.filters.network.generic_proxy.matcher.v3.RequestMatchInput
	(*KeyValueMatchEntry)(nil), // 6: envoy.extensions.filters.network.generic_proxy.matcher.v3.KeyValueMatchEntry
	(*RequestMatcher)(nil),     // 7: envoy.extensions.filters.network.generic_proxy.matcher.v3.RequestMatcher
	(*v3.StringMatcher)(nil),   // 8: envoy.type.matcher.v3.StringMatcher
}
var file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_depIdxs = []int32{
	8, // 0: envoy.extensions.filters.network.generic_proxy.matcher.v3.KeyValueMatchEntry.string_match:type_name -> envoy.type.matcher.v3.StringMatcher
	8, // 1: envoy.extensions.filters.network.generic_proxy.matcher.v3.RequestMatcher.host:type_name -> envoy.type.matcher.v3.StringMatcher
	8, // 2: envoy.extensions.filters.network.generic_proxy.matcher.v3.RequestMatcher.path:type_name -> envoy.type.matcher.v3.StringMatcher
	8, // 3: envoy.extensions.filters.network.generic_proxy.matcher.v3.RequestMatcher.method:type_name -> envoy.type.matcher.v3.StringMatcher
	6, // 4: envoy.extensions.filters.network.generic_proxy.matcher.v3.RequestMatcher.properties:type_name -> envoy.extensions.filters.network.generic_proxy.matcher.v3.KeyValueMatchEntry
	5, // [5:5] is the sub-list for method output_type
	5, // [5:5] is the sub-list for method input_type
	5, // [5:5] is the sub-list for extension type_name
	5, // [5:5] is the sub-list for extension extendee
	0, // [0:5] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_init() }
func file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_init() {
	if File_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDesc), len(file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   8,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto = out.File
	file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_goTypes = nil
	file_envoy_extensions_filters_network_generic_proxy_matcher_v3_matcher_proto_depIdxs = nil
}
