// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/upstreams/tcp/v3/tcp_protocol_options.proto

package tcpv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type TcpProtocolOptions struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The idle timeout for the connection. The idle timeout is defined as the period in which
	// the connection is not associated with a downstream connection. When the idle timeout is
	// reached, the connection will be closed.
	//
	// If not set, the default idle timeout is 10 minutes. To disable idle timeouts, explicitly set this to 0.
	//
	// .. warning::
	//
	//	Disabling this timeout has a highly likelihood of yielding connection leaks due to lost TCP
	//	FIN packets, etc.
	IdleTimeout   *durationpb.Duration `protobuf:"bytes,1,opt,name=idle_timeout,json=idleTimeout,proto3" json:"idle_timeout,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TcpProtocolOptions) Reset() {
	*x = TcpProtocolOptions{}
	mi := &file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TcpProtocolOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TcpProtocolOptions) ProtoMessage() {}

func (x *TcpProtocolOptions) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TcpProtocolOptions.ProtoReflect.Descriptor instead.
func (*TcpProtocolOptions) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDescGZIP(), []int{0}
}

func (x *TcpProtocolOptions) GetIdleTimeout() *durationpb.Duration {
	if x != nil {
		return x.IdleTimeout
	}
	return nil
}

var File_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto protoreflect.FileDescriptor

const file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDesc = "" +
	"\n" +
	"<envoy/extensions/upstreams/tcp/v3/tcp_protocol_options.proto\x12!envoy.extensions.upstreams.tcp.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\"R\n" +
	"\x12TcpProtocolOptions\x12<\n" +
	"\fidle_timeout\x18\x01 \x01(\v2\x19.google.protobuf.DurationR\vidleTimeoutB\xa4\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"/io.envoyproxy.envoy.extensions.upstreams.tcp.v3B\x17TcpProtocolOptionsProtoP\x01ZNgithub.com/envoyproxy/go-control-plane/envoy/extensions/upstreams/tcp/v3;tcpv3b\x06proto3"

var (
	file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDescOnce sync.Once
	file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDescData []byte
)

func file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDescGZIP() []byte {
	file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDesc), len(file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDesc)))
	})
	return file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDescData
}

var file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_goTypes = []any{
	(*TcpProtocolOptions)(nil),  // 0: envoy.extensions.upstreams.tcp.v3.TcpProtocolOptions
	(*durationpb.Duration)(nil), // 1: google.protobuf.Duration
}
var file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.upstreams.tcp.v3.TcpProtocolOptions.idle_timeout:type_name -> google.protobuf.Duration
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_init() }
func file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_init() {
	if File_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDesc), len(file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_msgTypes,
	}.Build()
	File_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto = out.File
	file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_goTypes = nil
	file_envoy_extensions_upstreams_tcp_v3_tcp_protocol_options_proto_depIdxs = nil
}
