// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/health_checkers/thrift/v3/thrift.proto

package thriftv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/thrift_proxy/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Thrift struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Specifies the method name that will be set on each health check request dispatched to an upstream host.
	// Note that method name is case sensitive.
	MethodName string `protobuf:"bytes,1,opt,name=method_name,json=methodName,proto3" json:"method_name,omitempty"`
	// Configures the transport type to be used with the health checks. Note that
	// :ref:`AUTO_TRANSPORT<envoy_v3_api_enum_value_extensions.filters.network.thrift_proxy.v3.TransportType.AUTO_TRANSPORT>`
	// is not supported, and we don't honor :ref:`ThriftProtocolOptions<envoy_v3_api_msg_extensions.filters.network.thrift_proxy.v3.ThriftProtocolOptions>`
	// since it's possible to set to :ref:`AUTO_TRANSPORT<envoy_v3_api_enum_value_extensions.filters.network.thrift_proxy.v3.TransportType.AUTO_TRANSPORT>`.
	// [#extension-category: envoy.filters.network]
	Transport v3.TransportType `protobuf:"varint,2,opt,name=transport,proto3,enum=envoy.extensions.filters.network.thrift_proxy.v3.TransportType" json:"transport,omitempty"`
	// Configures the protocol type to be used with the health checks. Note that
	// :ref:`AUTO_PROTOCOL<envoy_v3_api_enum_value_extensions.filters.network.thrift_proxy.v3.ProtocolType.AUTO_PROTOCOL>`
	// and :ref:`TWITTER<envoy_v3_api_enum_value_extensions.filters.network.thrift_proxy.v3.ProtocolType.TWITTER>`
	// are not supported, and we don't honor :ref:`ThriftProtocolOptions<envoy_v3_api_msg_extensions.filters.network.thrift_proxy.v3.ThriftProtocolOptions>`
	// since it's possible to set to :ref:`AUTO_PROTOCOL<envoy_v3_api_enum_value_extensions.filters.network.thrift_proxy.v3.ProtocolType.AUTO_PROTOCOL>`
	// or :ref:`TWITTER<envoy_v3_api_enum_value_extensions.filters.network.thrift_proxy.v3.ProtocolType.TWITTER>`.
	Protocol      v3.ProtocolType `protobuf:"varint,3,opt,name=protocol,proto3,enum=envoy.extensions.filters.network.thrift_proxy.v3.ProtocolType" json:"protocol,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Thrift) Reset() {
	*x = Thrift{}
	mi := &file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Thrift) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Thrift) ProtoMessage() {}

func (x *Thrift) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Thrift.ProtoReflect.Descriptor instead.
func (*Thrift) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDescGZIP(), []int{0}
}

func (x *Thrift) GetMethodName() string {
	if x != nil {
		return x.MethodName
	}
	return ""
}

func (x *Thrift) GetTransport() v3.TransportType {
	if x != nil {
		return x.Transport
	}
	return v3.TransportType(0)
}

func (x *Thrift) GetProtocol() v3.ProtocolType {
	if x != nil {
		return x.Protocol
	}
	return v3.ProtocolType(0)
}

var File_envoy_extensions_health_checkers_thrift_v3_thrift_proto protoreflect.FileDescriptor

const file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDesc = "" +
	"\n" +
	"7envoy/extensions/health_checkers/thrift/v3/thrift.proto\x12*envoy.extensions.health_checkers.thrift.v3\x1aCenvoy/extensions/filters/network/thrift_proxy/v3/thrift_proxy.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x81\x02\n" +
	"\x06Thrift\x12(\n" +
	"\vmethod_name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"methodName\x12g\n" +
	"\ttransport\x18\x02 \x01(\x0e2?.envoy.extensions.filters.network.thrift_proxy.v3.TransportTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\ttransport\x12d\n" +
	"\bprotocol\x18\x03 \x01(\x0e2>.envoy.extensions.filters.network.thrift_proxy.v3.ProtocolTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\bprotocolB\xad\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"8io.envoyproxy.envoy.extensions.health_checkers.thrift.v3B\vThriftProtoP\x01ZZgithub.com/envoyproxy/go-control-plane/envoy/extensions/health_checkers/thrift/v3;thriftv3b\x06proto3"

var (
	file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDescOnce sync.Once
	file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDescData []byte
)

func file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDescGZIP() []byte {
	file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDesc), len(file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDesc)))
	})
	return file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDescData
}

var file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_goTypes = []any{
	(*Thrift)(nil),        // 0: envoy.extensions.health_checkers.thrift.v3.Thrift
	(v3.TransportType)(0), // 1: envoy.extensions.filters.network.thrift_proxy.v3.TransportType
	(v3.ProtocolType)(0),  // 2: envoy.extensions.filters.network.thrift_proxy.v3.ProtocolType
}
var file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.health_checkers.thrift.v3.Thrift.transport:type_name -> envoy.extensions.filters.network.thrift_proxy.v3.TransportType
	2, // 1: envoy.extensions.health_checkers.thrift.v3.Thrift.protocol:type_name -> envoy.extensions.filters.network.thrift_proxy.v3.ProtocolType
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_init() }
func file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_init() {
	if File_envoy_extensions_health_checkers_thrift_v3_thrift_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDesc), len(file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_msgTypes,
	}.Build()
	File_envoy_extensions_health_checkers_thrift_v3_thrift_proto = out.File
	file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_goTypes = nil
	file_envoy_extensions_health_checkers_thrift_v3_thrift_proto_depIdxs = nil
}
