// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/rbac/principals/mtls_authenticated/v3/mtls_authenticated.proto

package mtls_authenticatedv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Authentication attributes for a downstream mTLS connection. All modes require that a peer certificate
// was presented and validated using the ValidationContext in the DownstreamTlsContext configuration.
//
// If neither field is set, a configuration loading error will be generated. This is so that
// not validating SANs requires an affirmative configuration to disable, to prevent accidentally
// not configuring SAN validation.
//
// If “any_validated_client_certificate“ is set in addition to “san_matcher“ or a future field
// which specifies additional validation, the other field always takes precedence over
// “any_validated_client_certificate“ and all specified validation is performed.
type Config struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Specifies a SAN that must be present in the validated peer certificate.
	SanMatcher *v3.SubjectAltNameMatcher `protobuf:"bytes,1,opt,name=san_matcher,json=sanMatcher,proto3" json:"san_matcher,omitempty"`
	// Only require that the peer certificate is present and valid.
	AnyValidatedClientCertificate bool `protobuf:"varint,2,opt,name=any_validated_client_certificate,json=anyValidatedClientCertificate,proto3" json:"any_validated_client_certificate,omitempty"`
	unknownFields                 protoimpl.UnknownFields
	sizeCache                     protoimpl.SizeCache
}

func (x *Config) Reset() {
	*x = Config{}
	mi := &file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Config) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Config) ProtoMessage() {}

func (x *Config) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Config.ProtoReflect.Descriptor instead.
func (*Config) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDescGZIP(), []int{0}
}

func (x *Config) GetSanMatcher() *v3.SubjectAltNameMatcher {
	if x != nil {
		return x.SanMatcher
	}
	return nil
}

func (x *Config) GetAnyValidatedClientCertificate() bool {
	if x != nil {
		return x.AnyValidatedClientCertificate
	}
	return false
}

var File_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto protoreflect.FileDescriptor

const file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDesc = "" +
	"\n" +
	"Oenvoy/extensions/rbac/principals/mtls_authenticated/v3/mtls_authenticated.proto\x126envoy.extensions.rbac.principals.mtls_authenticated.v3\x1a6envoy/extensions/transport_sockets/tls/v3/common.proto\x1a\x1dudpa/annotations/status.proto\"\xb4\x01\n" +
	"\x06Config\x12a\n" +
	"\vsan_matcher\x18\x01 \x01(\v2@.envoy.extensions.transport_sockets.tls.v3.SubjectAltNameMatcherR\n" +
	"sanMatcher\x12G\n" +
	" any_validated_client_certificate\x18\x02 \x01(\bR\x1danyValidatedClientCertificateB\xdc\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Dio.envoyproxy.envoy.extensions.rbac.principals.mtls_authenticated.v3B\x16MtlsAuthenticatedProtoP\x01Zrgithub.com/envoyproxy/go-control-plane/envoy/extensions/rbac/principals/mtls_authenticated/v3;mtls_authenticatedv3b\x06proto3"

var (
	file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDescOnce sync.Once
	file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDescData []byte
)

func file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDescGZIP() []byte {
	file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDesc), len(file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDesc)))
	})
	return file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDescData
}

var file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_goTypes = []any{
	(*Config)(nil),                   // 0: envoy.extensions.rbac.principals.mtls_authenticated.v3.Config
	(*v3.SubjectAltNameMatcher)(nil), // 1: envoy.extensions.transport_sockets.tls.v3.SubjectAltNameMatcher
}
var file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.rbac.principals.mtls_authenticated.v3.Config.san_matcher:type_name -> envoy.extensions.transport_sockets.tls.v3.SubjectAltNameMatcher
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_init()
}
func file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_init() {
	if File_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDesc), len(file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_msgTypes,
	}.Build()
	File_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto = out.File
	file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_goTypes = nil
	file_envoy_extensions_rbac_principals_mtls_authenticated_v3_mtls_authenticated_proto_depIdxs = nil
}
