// SPDX-License-Identifier: Apache-2.0
// Copyright Authors of Cilium

// Code generated by client-gen. DO NOT EDIT.

package v1

type EndpointsExpansion interface{}

type NamespaceExpansion interface{}

type NodeExpansion interface{}

type PodExpansion interface{}

type SecretExpansion interface{}

type ServiceExpansion interface{}
