// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/generic_proxy/codecs/http1/v3/http1.proto

package http1v3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for HTTP codec. This HTTP1 codec is used to parse and serialize HTTP1 messages
// for the generic proxy filter.
// Any decoding error will result in the generic proxy closing the connection.
//
// .. note::
//
//	This codec only supports HTTP1.1 messages and does not support HTTP1.0 messages. And it limits
//	part of the HTTP1.1 features, such as upgrade, connect, etc.
//	This codec is mainly designed for the features evaluation of the generic proxy filter. Please
//	be cautious when using it in production.
type Http1CodecConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// If true, the codec will parse and serialize HTTP1 messages in a single frame per message.
	//
	// A frame is a minimal unit of data that can be processed by the generic proxy. If false, the
	// codec will parse and serialize HTTP1 messages in a streaming way. In this case, the codec
	// will output multiple frames for a single HTTP1 message to the generic proxy.
	// If true, the codec will buffer the entire HTTP1 message body before sending it to the generic
	// proxy. This may have better performance in small message scenarios and is more friendly to
	// handle the HTTP1 message body. This also may result in higher memory usage and latency if
	// the message body is large.
	//
	// Default is true.
	SingleFrameMode *wrapperspb.BoolValue `protobuf:"bytes,1,opt,name=single_frame_mode,json=singleFrameMode,proto3" json:"single_frame_mode,omitempty"`
	// The maximum size of the HTTP1 message body in bytes. If not set, 8*1024*1024 (8MB) is used.
	// This only makes sense when single_frame_mode is true.
	// If the HTTP1 message body size exceeds this value, this will result in a decoding error
	// and the generic proxy will close the connection.
	MaxBufferSize *wrapperspb.UInt32Value `protobuf:"bytes,2,opt,name=max_buffer_size,json=maxBufferSize,proto3" json:"max_buffer_size,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Http1CodecConfig) Reset() {
	*x = Http1CodecConfig{}
	mi := &file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Http1CodecConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Http1CodecConfig) ProtoMessage() {}

func (x *Http1CodecConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Http1CodecConfig.ProtoReflect.Descriptor instead.
func (*Http1CodecConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDescGZIP(), []int{0}
}

func (x *Http1CodecConfig) GetSingleFrameMode() *wrapperspb.BoolValue {
	if x != nil {
		return x.SingleFrameMode
	}
	return nil
}

func (x *Http1CodecConfig) GetMaxBufferSize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MaxBufferSize
	}
	return nil
}

var File_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDesc = "" +
	"\n" +
	"Jenvoy/extensions/filters/network/generic_proxy/codecs/http1/v3/http1.proto\x12>envoy.extensions.filters.network.generic_proxy.codecs.http1.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"\xa0\x01\n" +
	"\x10Http1CodecConfig\x12F\n" +
	"\x11single_frame_mode\x18\x01 \x01(\v2\x1a.google.protobuf.BoolValueR\x0fsingleFrameMode\x12D\n" +
	"\x0fmax_buffer_size\x18\x02 \x01(\v2\x1c.google.protobuf.UInt32ValueR\rmaxBufferSizeB\xdb\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"Lio.envoyproxy.envoy.extensions.filters.network.generic_proxy.codecs.http1.v3B\n" +
	"Http1ProtoP\x01Zmgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/generic_proxy/codecs/http1/v3;http1v3b\x06proto3"

var (
	file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDesc), len(file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDescData
}

var file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_goTypes = []any{
	(*Http1CodecConfig)(nil),       // 0: envoy.extensions.filters.network.generic_proxy.codecs.http1.v3.Http1CodecConfig
	(*wrapperspb.BoolValue)(nil),   // 1: google.protobuf.BoolValue
	(*wrapperspb.UInt32Value)(nil), // 2: google.protobuf.UInt32Value
}
var file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.network.generic_proxy.codecs.http1.v3.Http1CodecConfig.single_frame_mode:type_name -> google.protobuf.BoolValue
	2, // 1: envoy.extensions.filters.network.generic_proxy.codecs.http1.v3.Http1CodecConfig.max_buffer_size:type_name -> google.protobuf.UInt32Value
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_init() }
func file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_init() {
	if File_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDesc), len(file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto = out.File
	file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_goTypes = nil
	file_envoy_extensions_filters_network_generic_proxy_codecs_http1_v3_http1_proto_depIdxs = nil
}
