// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/http/buffer/v3/buffer.proto

package bufferv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Buffer struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The maximum request size that the filter will buffer before the connection
	// manager will stop buffering and return a 413 response.
	MaxRequestBytes *wrapperspb.UInt32Value `protobuf:"bytes,1,opt,name=max_request_bytes,json=maxRequestBytes,proto3" json:"max_request_bytes,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *Buffer) Reset() {
	*x = Buffer{}
	mi := &file_envoy_extensions_filters_http_buffer_v3_buffer_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Buffer) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Buffer) ProtoMessage() {}

func (x *Buffer) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_buffer_v3_buffer_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Buffer.ProtoReflect.Descriptor instead.
func (*Buffer) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDescGZIP(), []int{0}
}

func (x *Buffer) GetMaxRequestBytes() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MaxRequestBytes
	}
	return nil
}

type BufferPerRoute struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to Override:
	//
	//	*BufferPerRoute_Disabled
	//	*BufferPerRoute_Buffer
	Override      isBufferPerRoute_Override `protobuf_oneof:"override"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *BufferPerRoute) Reset() {
	*x = BufferPerRoute{}
	mi := &file_envoy_extensions_filters_http_buffer_v3_buffer_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *BufferPerRoute) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*BufferPerRoute) ProtoMessage() {}

func (x *BufferPerRoute) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_buffer_v3_buffer_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use BufferPerRoute.ProtoReflect.Descriptor instead.
func (*BufferPerRoute) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDescGZIP(), []int{1}
}

func (x *BufferPerRoute) GetOverride() isBufferPerRoute_Override {
	if x != nil {
		return x.Override
	}
	return nil
}

func (x *BufferPerRoute) GetDisabled() bool {
	if x != nil {
		if x, ok := x.Override.(*BufferPerRoute_Disabled); ok {
			return x.Disabled
		}
	}
	return false
}

func (x *BufferPerRoute) GetBuffer() *Buffer {
	if x != nil {
		if x, ok := x.Override.(*BufferPerRoute_Buffer); ok {
			return x.Buffer
		}
	}
	return nil
}

type isBufferPerRoute_Override interface {
	isBufferPerRoute_Override()
}

type BufferPerRoute_Disabled struct {
	// Disable the buffer filter for this particular vhost or route.
	Disabled bool `protobuf:"varint,1,opt,name=disabled,proto3,oneof"`
}

type BufferPerRoute_Buffer struct {
	// Override the global configuration of the filter with this new config.
	Buffer *Buffer `protobuf:"bytes,2,opt,name=buffer,proto3,oneof"`
}

func (*BufferPerRoute_Disabled) isBufferPerRoute_Override() {}

func (*BufferPerRoute_Buffer) isBufferPerRoute_Override() {}

var File_envoy_extensions_filters_http_buffer_v3_buffer_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDesc = "" +
	"\n" +
	"4envoy/extensions/filters/http/buffer/v3/buffer.proto\x12'envoy.extensions.filters.http.buffer.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x98\x01\n" +
	"\x06Buffer\x12V\n" +
	"\x11max_request_bytes\x18\x01 \x01(\v2\x1c.google.protobuf.UInt32ValueB\f\xfaB\t\x8a\x01\x02\x10\x01*\x02 \x00R\x0fmaxRequestBytes:0\x9aň\x1e+\n" +
	")envoy.config.filter.http.buffer.v2.BufferJ\x04\b\x02\x10\x03\"\xd7\x01\n" +
	"\x0eBufferPerRoute\x12%\n" +
	"\bdisabled\x18\x01 \x01(\bB\a\xfaB\x04j\x02\b\x01H\x00R\bdisabled\x12S\n" +
	"\x06buffer\x18\x02 \x01(\v2/.envoy.extensions.filters.http.buffer.v3.BufferB\b\xfaB\x05\x8a\x01\x02\x10\x01H\x00R\x06buffer:8\x9aň\x1e3\n" +
	"1envoy.config.filter.http.buffer.v2.BufferPerRouteB\x0f\n" +
	"\boverride\x12\x03\xf8B\x01B\xa7\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"5io.envoyproxy.envoy.extensions.filters.http.buffer.v3B\vBufferProtoP\x01ZWgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/buffer/v3;bufferv3b\x06proto3"

var (
	file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDescData []byte
)

func file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDesc), len(file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDescData
}

var file_envoy_extensions_filters_http_buffer_v3_buffer_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_filters_http_buffer_v3_buffer_proto_goTypes = []any{
	(*Buffer)(nil),                 // 0: envoy.extensions.filters.http.buffer.v3.Buffer
	(*BufferPerRoute)(nil),         // 1: envoy.extensions.filters.http.buffer.v3.BufferPerRoute
	(*wrapperspb.UInt32Value)(nil), // 2: google.protobuf.UInt32Value
}
var file_envoy_extensions_filters_http_buffer_v3_buffer_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.filters.http.buffer.v3.Buffer.max_request_bytes:type_name -> google.protobuf.UInt32Value
	0, // 1: envoy.extensions.filters.http.buffer.v3.BufferPerRoute.buffer:type_name -> envoy.extensions.filters.http.buffer.v3.Buffer
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_http_buffer_v3_buffer_proto_init() }
func file_envoy_extensions_filters_http_buffer_v3_buffer_proto_init() {
	if File_envoy_extensions_filters_http_buffer_v3_buffer_proto != nil {
		return
	}
	file_envoy_extensions_filters_http_buffer_v3_buffer_proto_msgTypes[1].OneofWrappers = []any{
		(*BufferPerRoute_Disabled)(nil),
		(*BufferPerRoute_Buffer)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDesc), len(file_envoy_extensions_filters_http_buffer_v3_buffer_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_http_buffer_v3_buffer_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_http_buffer_v3_buffer_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_http_buffer_v3_buffer_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_http_buffer_v3_buffer_proto = out.File
	file_envoy_extensions_filters_http_buffer_v3_buffer_proto_goTypes = nil
	file_envoy_extensions_filters_http_buffer_v3_buffer_proto_depIdxs = nil
}
