// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/listener/tls_inspector/v3/tls_inspector.proto

package tls_inspectorv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#next-free-field: 6]
type TlsInspector struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Populate “JA3“ fingerprint hash using data from the TLS Client Hello packet. Default is false.
	EnableJa3Fingerprinting *wrapperspb.BoolValue `protobuf:"bytes,1,opt,name=enable_ja3_fingerprinting,json=enableJa3Fingerprinting,proto3" json:"enable_ja3_fingerprinting,omitempty"`
	// Populate “JA4“ fingerprint hash using data from the TLS Client Hello packet.
	// “JA4“ is an improved version of “JA3“ that includes TLS version, ciphers, extensions,
	// and ALPN information in a hex format. Default is false.
	EnableJa4Fingerprinting *wrapperspb.BoolValue `protobuf:"bytes,3,opt,name=enable_ja4_fingerprinting,json=enableJa4Fingerprinting,proto3" json:"enable_ja4_fingerprinting,omitempty"`
	// The size in bytes of the initial buffer requested by the tls_inspector.
	// If the filter needs to read additional bytes from the socket, the
	// filter will double the buffer up to it's default maximum of 16KiB.
	// If this size is not defined, defaults to maximum 16KiB that the
	// tls inspector will consume.
	InitialReadBufferSize *wrapperspb.UInt32Value `protobuf:"bytes,2,opt,name=initial_read_buffer_size,json=initialReadBufferSize,proto3" json:"initial_read_buffer_size,omitempty"`
	// Close connection when TLS ClientHello message could not be parsed.
	// This flag should be enabled only if it is known that incoming connections are expected to use
	// TLS protocol, as Envoy does not distinguish between a plain text message or a malformed TLS
	// ClientHello message.
	// By default this flag is false and TLS ClientHello parsing errors are interpreted as a
	// plain text connection.
	// Setting this to true will cause connections to be terminated and the “client_hello_too_large“
	// counter to be incremented if the ClientHello message is over implementation defined limit
	// (currently 16Kb).
	CloseConnectionOnClientHelloParsingErrors bool `protobuf:"varint,4,opt,name=close_connection_on_client_hello_parsing_errors,json=closeConnectionOnClientHelloParsingErrors,proto3" json:"close_connection_on_client_hello_parsing_errors,omitempty"`
	// The maximum size in bytes of the ClientHello that the tls_inspector will
	// process. If the ClientHello is larger than this size, the tls_inspector
	// will stop processing and indicate failure. If not defined, defaults to
	// 16KiB.
	MaxClientHelloSize *wrapperspb.UInt32Value `protobuf:"bytes,5,opt,name=max_client_hello_size,json=maxClientHelloSize,proto3" json:"max_client_hello_size,omitempty"`
	unknownFields      protoimpl.UnknownFields
	sizeCache          protoimpl.SizeCache
}

func (x *TlsInspector) Reset() {
	*x = TlsInspector{}
	mi := &file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TlsInspector) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TlsInspector) ProtoMessage() {}

func (x *TlsInspector) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TlsInspector.ProtoReflect.Descriptor instead.
func (*TlsInspector) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDescGZIP(), []int{0}
}

func (x *TlsInspector) GetEnableJa3Fingerprinting() *wrapperspb.BoolValue {
	if x != nil {
		return x.EnableJa3Fingerprinting
	}
	return nil
}

func (x *TlsInspector) GetEnableJa4Fingerprinting() *wrapperspb.BoolValue {
	if x != nil {
		return x.EnableJa4Fingerprinting
	}
	return nil
}

func (x *TlsInspector) GetInitialReadBufferSize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.InitialReadBufferSize
	}
	return nil
}

func (x *TlsInspector) GetCloseConnectionOnClientHelloParsingErrors() bool {
	if x != nil {
		return x.CloseConnectionOnClientHelloParsingErrors
	}
	return false
}

func (x *TlsInspector) GetMaxClientHelloSize() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MaxClientHelloSize
	}
	return nil
}

var File_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDesc = "" +
	"\n" +
	"Fenvoy/extensions/filters/listener/tls_inspector/v3/tls_inspector.proto\x122envoy.extensions.filters.listener.tls_inspector.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xa9\x04\n" +
	"\fTlsInspector\x12V\n" +
	"\x19enable_ja3_fingerprinting\x18\x01 \x01(\v2\x1a.google.protobuf.BoolValueR\x17enableJa3Fingerprinting\x12V\n" +
	"\x19enable_ja4_fingerprinting\x18\x03 \x01(\v2\x1a.google.protobuf.BoolValueR\x17enableJa4Fingerprinting\x12c\n" +
	"\x18initial_read_buffer_size\x18\x02 \x01(\v2\x1c.google.protobuf.UInt32ValueB\f\xfaB\t*\a\x10\x81\x80\x04 \xff\x01R\x15initialReadBufferSize\x12b\n" +
	"/close_connection_on_client_hello_parsing_errors\x18\x04 \x01(\bR)closeConnectionOnClientHelloParsingErrors\x12]\n" +
	"\x15max_client_hello_size\x18\x05 \x01(\v2\x1c.google.protobuf.UInt32ValueB\f\xfaB\t*\a\x18\x80\x80\x01 \xff\x01R\x12maxClientHelloSize:A\x9aň\x1e<\n" +
	":envoy.config.filter.listener.tls_inspector.v2.TlsInspectorB\xca\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"@io.envoyproxy.envoy.extensions.filters.listener.tls_inspector.v3B\x11TlsInspectorProtoP\x01Zigithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/listener/tls_inspector/v3;tls_inspectorv3b\x06proto3"

var (
	file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDescData []byte
)

func file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDesc), len(file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDescData
}

var file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_goTypes = []any{
	(*TlsInspector)(nil),           // 0: envoy.extensions.filters.listener.tls_inspector.v3.TlsInspector
	(*wrapperspb.BoolValue)(nil),   // 1: google.protobuf.BoolValue
	(*wrapperspb.UInt32Value)(nil), // 2: google.protobuf.UInt32Value
}
var file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.listener.tls_inspector.v3.TlsInspector.enable_ja3_fingerprinting:type_name -> google.protobuf.BoolValue
	1, // 1: envoy.extensions.filters.listener.tls_inspector.v3.TlsInspector.enable_ja4_fingerprinting:type_name -> google.protobuf.BoolValue
	2, // 2: envoy.extensions.filters.listener.tls_inspector.v3.TlsInspector.initial_read_buffer_size:type_name -> google.protobuf.UInt32Value
	2, // 3: envoy.extensions.filters.listener.tls_inspector.v3.TlsInspector.max_client_hello_size:type_name -> google.protobuf.UInt32Value
	4, // [4:4] is the sub-list for method output_type
	4, // [4:4] is the sub-list for method input_type
	4, // [4:4] is the sub-list for extension type_name
	4, // [4:4] is the sub-list for extension extendee
	0, // [0:4] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_init() }
func file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_init() {
	if File_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDesc), len(file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto = out.File
	file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_goTypes = nil
	file_envoy_extensions_filters_listener_tls_inspector_v3_tls_inspector_proto_depIdxs = nil
}
