// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/http/kill_request/v3/kill_request.proto

package kill_requestv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/type/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// On which direction should the filter check for the “kill_request_header“.
// Default to “REQUEST“ if unspecified.
type KillRequest_Direction int32

const (
	KillRequest_REQUEST  KillRequest_Direction = 0
	KillRequest_RESPONSE KillRequest_Direction = 1
)

// Enum value maps for KillRequest_Direction.
var (
	KillRequest_Direction_name = map[int32]string{
		0: "REQUEST",
		1: "RESPONSE",
	}
	KillRequest_Direction_value = map[string]int32{
		"REQUEST":  0,
		"RESPONSE": 1,
	}
)

func (x KillRequest_Direction) Enum() *KillRequest_Direction {
	p := new(KillRequest_Direction)
	*p = x
	return p
}

func (x KillRequest_Direction) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (KillRequest_Direction) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_enumTypes[0].Descriptor()
}

func (KillRequest_Direction) Type() protoreflect.EnumType {
	return &file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_enumTypes[0]
}

func (x KillRequest_Direction) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use KillRequest_Direction.Descriptor instead.
func (KillRequest_Direction) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDescGZIP(), []int{0, 0}
}

// Configuration for KillRequest filter.
type KillRequest struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The probability that a Kill request will be triggered.
	Probability *v3.FractionalPercent `protobuf:"bytes,1,opt,name=probability,proto3" json:"probability,omitempty"`
	// The name of the kill request header. If this field is not empty, it will override the :ref:`default header <config_http_filters_kill_request_http_header>` name. Otherwise the default header name will be used.
	KillRequestHeader string                `protobuf:"bytes,2,opt,name=kill_request_header,json=killRequestHeader,proto3" json:"kill_request_header,omitempty"`
	Direction         KillRequest_Direction `protobuf:"varint,3,opt,name=direction,proto3,enum=envoy.extensions.filters.http.kill_request.v3.KillRequest_Direction" json:"direction,omitempty"`
	unknownFields     protoimpl.UnknownFields
	sizeCache         protoimpl.SizeCache
}

func (x *KillRequest) Reset() {
	*x = KillRequest{}
	mi := &file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *KillRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*KillRequest) ProtoMessage() {}

func (x *KillRequest) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use KillRequest.ProtoReflect.Descriptor instead.
func (*KillRequest) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDescGZIP(), []int{0}
}

func (x *KillRequest) GetProbability() *v3.FractionalPercent {
	if x != nil {
		return x.Probability
	}
	return nil
}

func (x *KillRequest) GetKillRequestHeader() string {
	if x != nil {
		return x.KillRequestHeader
	}
	return ""
}

func (x *KillRequest) GetDirection() KillRequest_Direction {
	if x != nil {
		return x.Direction
	}
	return KillRequest_REQUEST
}

var File_envoy_extensions_filters_http_kill_request_v3_kill_request_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDesc = "" +
	"\n" +
	"@envoy/extensions/filters/http/kill_request/v3/kill_request.proto\x12-envoy.extensions.filters.http.kill_request.v3\x1a\x1benvoy/type/v3/percent.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xa4\x02\n" +
	"\vKillRequest\x12B\n" +
	"\vprobability\x18\x01 \x01(\v2 .envoy.type.v3.FractionalPercentR\vprobability\x12;\n" +
	"\x13kill_request_header\x18\x02 \x01(\tB\v\xfaB\br\x06\xc8\x01\x00\xc0\x01\x02R\x11killRequestHeader\x12l\n" +
	"\tdirection\x18\x03 \x01(\x0e2D.envoy.extensions.filters.http.kill_request.v3.KillRequest.DirectionB\b\xfaB\x05\x82\x01\x02\x10\x01R\tdirection\"&\n" +
	"\tDirection\x12\v\n" +
	"\aREQUEST\x10\x00\x12\f\n" +
	"\bRESPONSE\x10\x01B\xbe\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	";io.envoyproxy.envoy.extensions.filters.http.kill_request.v3B\x10KillRequestProtoP\x01Zcgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/kill_request/v3;kill_requestv3b\x06proto3"

var (
	file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDescData []byte
)

func file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDesc), len(file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDescData
}

var file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_goTypes = []any{
	(KillRequest_Direction)(0),   // 0: envoy.extensions.filters.http.kill_request.v3.KillRequest.Direction
	(*KillRequest)(nil),          // 1: envoy.extensions.filters.http.kill_request.v3.KillRequest
	(*v3.FractionalPercent)(nil), // 2: envoy.type.v3.FractionalPercent
}
var file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.filters.http.kill_request.v3.KillRequest.probability:type_name -> envoy.type.v3.FractionalPercent
	0, // 1: envoy.extensions.filters.http.kill_request.v3.KillRequest.direction:type_name -> envoy.extensions.filters.http.kill_request.v3.KillRequest.Direction
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_init() }
func file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_init() {
	if File_envoy_extensions_filters_http_kill_request_v3_kill_request_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDesc), len(file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_depIdxs,
		EnumInfos:         file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_enumTypes,
		MessageInfos:      file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_http_kill_request_v3_kill_request_proto = out.File
	file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_goTypes = nil
	file_envoy_extensions_filters_http_kill_request_v3_kill_request_proto_depIdxs = nil
}
