// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/thrift_proxy/v3/thrift_proxy.proto

package thrift_proxyv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v31 "github.com/envoyproxy/go-control-plane/envoy/config/accesslog/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Thrift transport types supported by Envoy.
type TransportType int32

const (
	// For downstream connections, the Thrift proxy will attempt to determine which transport to use.
	// For upstream connections, the Thrift proxy will use same transport as the downstream
	// connection.
	TransportType_AUTO_TRANSPORT TransportType = 0
	// The Thrift proxy will use the Thrift framed transport.
	TransportType_FRAMED TransportType = 1
	// The Thrift proxy will use the Thrift unframed transport.
	TransportType_UNFRAMED TransportType = 2
	// The Thrift proxy will assume the client is using the Thrift header transport.
	TransportType_HEADER TransportType = 3
)

// Enum value maps for TransportType.
var (
	TransportType_name = map[int32]string{
		0: "AUTO_TRANSPORT",
		1: "FRAMED",
		2: "UNFRAMED",
		3: "HEADER",
	}
	TransportType_value = map[string]int32{
		"AUTO_TRANSPORT": 0,
		"FRAMED":         1,
		"UNFRAMED":       2,
		"HEADER":         3,
	}
)

func (x TransportType) Enum() *TransportType {
	p := new(TransportType)
	*p = x
	return p
}

func (x TransportType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (TransportType) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_enumTypes[0].Descriptor()
}

func (TransportType) Type() protoreflect.EnumType {
	return &file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_enumTypes[0]
}

func (x TransportType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use TransportType.Descriptor instead.
func (TransportType) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescGZIP(), []int{0}
}

// Thrift Protocol types supported by Envoy.
type ProtocolType int32

const (
	// For downstream connections, the Thrift proxy will attempt to determine which protocol to use.
	// Note that the older, non-strict (or lax) binary protocol is not included in automatic protocol
	// detection. For upstream connections, the Thrift proxy will use the same protocol as the
	// downstream connection.
	ProtocolType_AUTO_PROTOCOL ProtocolType = 0
	// The Thrift proxy will use the Thrift binary protocol.
	ProtocolType_BINARY ProtocolType = 1
	// The Thrift proxy will use Thrift non-strict binary protocol.
	ProtocolType_LAX_BINARY ProtocolType = 2
	// The Thrift proxy will use the Thrift compact protocol.
	ProtocolType_COMPACT ProtocolType = 3
	// The Thrift proxy will use the Thrift "Twitter" protocol implemented by the finagle library.
	//
	// Deprecated: Marked as deprecated in envoy/extensions/filters/network/thrift_proxy/v3/thrift_proxy.proto.
	ProtocolType_TWITTER ProtocolType = 4
)

// Enum value maps for ProtocolType.
var (
	ProtocolType_name = map[int32]string{
		0: "AUTO_PROTOCOL",
		1: "BINARY",
		2: "LAX_BINARY",
		3: "COMPACT",
		4: "TWITTER",
	}
	ProtocolType_value = map[string]int32{
		"AUTO_PROTOCOL": 0,
		"BINARY":        1,
		"LAX_BINARY":    2,
		"COMPACT":       3,
		"TWITTER":       4,
	}
)

func (x ProtocolType) Enum() *ProtocolType {
	p := new(ProtocolType)
	*p = x
	return p
}

func (x ProtocolType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ProtocolType) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_enumTypes[1].Descriptor()
}

func (ProtocolType) Type() protoreflect.EnumType {
	return &file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_enumTypes[1]
}

func (x ProtocolType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ProtocolType.Descriptor instead.
func (ProtocolType) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescGZIP(), []int{1}
}

type Trds struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Configuration source specifier.
	// In case of “api_config_source“ only aggregated “api_type“ is supported.
	ConfigSource *v3.ConfigSource `protobuf:"bytes,1,opt,name=config_source,json=configSource,proto3" json:"config_source,omitempty"`
	// The name of the route configuration. This allows to use different route
	// configurations. Tells which route configuration should be fetched from the configuration source.
	// Leave unspecified is also valid and means the unnamed route configuration.
	RouteConfigName string `protobuf:"bytes,2,opt,name=route_config_name,json=routeConfigName,proto3" json:"route_config_name,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *Trds) Reset() {
	*x = Trds{}
	mi := &file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Trds) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Trds) ProtoMessage() {}

func (x *Trds) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Trds.ProtoReflect.Descriptor instead.
func (*Trds) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescGZIP(), []int{0}
}

func (x *Trds) GetConfigSource() *v3.ConfigSource {
	if x != nil {
		return x.ConfigSource
	}
	return nil
}

func (x *Trds) GetRouteConfigName() string {
	if x != nil {
		return x.RouteConfigName
	}
	return ""
}

// [#next-free-field: 11]
type ThriftProxy struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Supplies the type of transport that the Thrift proxy should use. Defaults to
	// :ref:`AUTO_TRANSPORT<envoy_v3_api_enum_value_extensions.filters.network.thrift_proxy.v3.TransportType.AUTO_TRANSPORT>`.
	Transport TransportType `protobuf:"varint,2,opt,name=transport,proto3,enum=envoy.extensions.filters.network.thrift_proxy.v3.TransportType" json:"transport,omitempty"`
	// Supplies the type of protocol that the Thrift proxy should use. Defaults to
	// :ref:`AUTO_PROTOCOL<envoy_v3_api_enum_value_extensions.filters.network.thrift_proxy.v3.ProtocolType.AUTO_PROTOCOL>`.
	Protocol ProtocolType `protobuf:"varint,3,opt,name=protocol,proto3,enum=envoy.extensions.filters.network.thrift_proxy.v3.ProtocolType" json:"protocol,omitempty"`
	// The human readable prefix to use when emitting statistics.
	StatPrefix string `protobuf:"bytes,1,opt,name=stat_prefix,json=statPrefix,proto3" json:"stat_prefix,omitempty"`
	// The route table for the connection manager is static and is specified in this property.
	// It is invalid to define both “route_config“ and “trds“.
	RouteConfig *RouteConfiguration `protobuf:"bytes,4,opt,name=route_config,json=routeConfig,proto3" json:"route_config,omitempty"`
	// Use xDS to fetch the route configuration. It is invalid to define both “route_config“ and “trds“.
	Trds *Trds `protobuf:"bytes,8,opt,name=trds,proto3" json:"trds,omitempty"`
	// A list of individual Thrift filters that make up the filter chain for requests made to the
	// Thrift proxy. Order matters as the filters are processed sequentially. For backwards
	// compatibility, if no thrift_filters are specified, a default Thrift router filter
	// (“envoy.filters.thrift.router“) is used.
	// [#extension-category: envoy.thrift_proxy.filters]
	ThriftFilters []*ThriftFilter `protobuf:"bytes,5,rep,name=thrift_filters,json=thriftFilters,proto3" json:"thrift_filters,omitempty"`
	// If set to true, Envoy will try to skip decode data after metadata in the Thrift message.
	// This mode will only work if the upstream and downstream protocols are the same and the transports
	// are Framed or Header, and the protocol is not Twitter. Otherwise Envoy will
	// fallback to decode the data.
	PayloadPassthrough bool `protobuf:"varint,6,opt,name=payload_passthrough,json=payloadPassthrough,proto3" json:"payload_passthrough,omitempty"`
	// Optional maximum requests for a single downstream connection. If not specified, there is no limit.
	MaxRequestsPerConnection *wrapperspb.UInt32Value `protobuf:"bytes,7,opt,name=max_requests_per_connection,json=maxRequestsPerConnection,proto3" json:"max_requests_per_connection,omitempty"`
	// Configuration for :ref:`access logs <arch_overview_access_logs>`
	// emitted by Thrift proxy.
	AccessLog []*v31.AccessLog `protobuf:"bytes,9,rep,name=access_log,json=accessLog,proto3" json:"access_log,omitempty"`
	// If set to true, Envoy will preserve the case of Thrift header keys instead of serializing them to
	// lower case as per the default behavior. Note that NUL, CR and LF characters will also be preserved
	// as mandated by the Thrift spec.
	//
	// More info: https://github.com/apache/thrift/commit/e165fa3c85d00cb984f4d9635ed60909a1266ce1.
	HeaderKeysPreserveCase bool `protobuf:"varint,10,opt,name=header_keys_preserve_case,json=headerKeysPreserveCase,proto3" json:"header_keys_preserve_case,omitempty"`
	unknownFields          protoimpl.UnknownFields
	sizeCache              protoimpl.SizeCache
}

func (x *ThriftProxy) Reset() {
	*x = ThriftProxy{}
	mi := &file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ThriftProxy) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ThriftProxy) ProtoMessage() {}

func (x *ThriftProxy) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ThriftProxy.ProtoReflect.Descriptor instead.
func (*ThriftProxy) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescGZIP(), []int{1}
}

func (x *ThriftProxy) GetTransport() TransportType {
	if x != nil {
		return x.Transport
	}
	return TransportType_AUTO_TRANSPORT
}

func (x *ThriftProxy) GetProtocol() ProtocolType {
	if x != nil {
		return x.Protocol
	}
	return ProtocolType_AUTO_PROTOCOL
}

func (x *ThriftProxy) GetStatPrefix() string {
	if x != nil {
		return x.StatPrefix
	}
	return ""
}

func (x *ThriftProxy) GetRouteConfig() *RouteConfiguration {
	if x != nil {
		return x.RouteConfig
	}
	return nil
}

func (x *ThriftProxy) GetTrds() *Trds {
	if x != nil {
		return x.Trds
	}
	return nil
}

func (x *ThriftProxy) GetThriftFilters() []*ThriftFilter {
	if x != nil {
		return x.ThriftFilters
	}
	return nil
}

func (x *ThriftProxy) GetPayloadPassthrough() bool {
	if x != nil {
		return x.PayloadPassthrough
	}
	return false
}

func (x *ThriftProxy) GetMaxRequestsPerConnection() *wrapperspb.UInt32Value {
	if x != nil {
		return x.MaxRequestsPerConnection
	}
	return nil
}

func (x *ThriftProxy) GetAccessLog() []*v31.AccessLog {
	if x != nil {
		return x.AccessLog
	}
	return nil
}

func (x *ThriftProxy) GetHeaderKeysPreserveCase() bool {
	if x != nil {
		return x.HeaderKeysPreserveCase
	}
	return false
}

// ThriftFilter configures a Thrift filter.
type ThriftFilter struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the filter to instantiate. The name must match a supported
	// filter. The built-in filters are:
	//
	// [#comment:TODO(zuercher): Auto generate the following list]
	// * :ref:`envoy.filters.thrift.router <config_thrift_filters_router>`
	// * :ref:`envoy.filters.thrift.rate_limit <config_thrift_filters_rate_limit>`
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Filter specific configuration which depends on the filter being instantiated. See the supported
	// filters for further documentation.
	//
	// Types that are valid to be assigned to ConfigType:
	//
	//	*ThriftFilter_TypedConfig
	ConfigType    isThriftFilter_ConfigType `protobuf_oneof:"config_type"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ThriftFilter) Reset() {
	*x = ThriftFilter{}
	mi := &file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ThriftFilter) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ThriftFilter) ProtoMessage() {}

func (x *ThriftFilter) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ThriftFilter.ProtoReflect.Descriptor instead.
func (*ThriftFilter) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescGZIP(), []int{2}
}

func (x *ThriftFilter) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *ThriftFilter) GetConfigType() isThriftFilter_ConfigType {
	if x != nil {
		return x.ConfigType
	}
	return nil
}

func (x *ThriftFilter) GetTypedConfig() *anypb.Any {
	if x != nil {
		if x, ok := x.ConfigType.(*ThriftFilter_TypedConfig); ok {
			return x.TypedConfig
		}
	}
	return nil
}

type isThriftFilter_ConfigType interface {
	isThriftFilter_ConfigType()
}

type ThriftFilter_TypedConfig struct {
	TypedConfig *anypb.Any `protobuf:"bytes,3,opt,name=typed_config,json=typedConfig,proto3,oneof"`
}

func (*ThriftFilter_TypedConfig) isThriftFilter_ConfigType() {}

// ThriftProtocolOptions specifies Thrift upstream protocol options. This object is used in
// in
// :ref:`typed_extension_protocol_options<envoy_v3_api_field_config.cluster.v3.Cluster.typed_extension_protocol_options>`,
// keyed by the name “envoy.filters.network.thrift_proxy“.
type ThriftProtocolOptions struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Supplies the type of transport that the Thrift proxy should use for upstream connections.
	// Selecting
	// :ref:`AUTO_TRANSPORT<envoy_v3_api_enum_value_extensions.filters.network.thrift_proxy.v3.TransportType.AUTO_TRANSPORT>`,
	// which is the default, causes the proxy to use the same transport as the downstream connection.
	Transport TransportType `protobuf:"varint,1,opt,name=transport,proto3,enum=envoy.extensions.filters.network.thrift_proxy.v3.TransportType" json:"transport,omitempty"`
	// Supplies the type of protocol that the Thrift proxy should use for upstream connections.
	// Selecting
	// :ref:`AUTO_PROTOCOL<envoy_v3_api_enum_value_extensions.filters.network.thrift_proxy.v3.ProtocolType.AUTO_PROTOCOL>`,
	// which is the default, causes the proxy to use the same protocol as the downstream connection.
	Protocol      ProtocolType `protobuf:"varint,2,opt,name=protocol,proto3,enum=envoy.extensions.filters.network.thrift_proxy.v3.ProtocolType" json:"protocol,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ThriftProtocolOptions) Reset() {
	*x = ThriftProtocolOptions{}
	mi := &file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ThriftProtocolOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ThriftProtocolOptions) ProtoMessage() {}

func (x *ThriftProtocolOptions) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ThriftProtocolOptions.ProtoReflect.Descriptor instead.
func (*ThriftProtocolOptions) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescGZIP(), []int{3}
}

func (x *ThriftProtocolOptions) GetTransport() TransportType {
	if x != nil {
		return x.Transport
	}
	return TransportType_AUTO_TRANSPORT
}

func (x *ThriftProtocolOptions) GetProtocol() ProtocolType {
	if x != nil {
		return x.Protocol
	}
	return ProtocolType_AUTO_PROTOCOL
}

var File_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDesc = "" +
	"\n" +
	"Cenvoy/extensions/filters/network/thrift_proxy/v3/thrift_proxy.proto\x120envoy.extensions.filters.network.thrift_proxy.v3\x1a)envoy/config/accesslog/v3/accesslog.proto\x1a(envoy/config/core/v3/config_source.proto\x1a<envoy/extensions/filters/network/thrift_proxy/v3/route.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x85\x01\n" +
	"\x04Trds\x12Q\n" +
	"\rconfig_source\x18\x01 \x01(\v2\".envoy.config.core.v3.ConfigSourceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\fconfigSource\x12*\n" +
	"\x11route_config_name\x18\x02 \x01(\tR\x0frouteConfigName\"\xa9\a\n" +
	"\vThriftProxy\x12g\n" +
	"\ttransport\x18\x02 \x01(\x0e2?.envoy.extensions.filters.network.thrift_proxy.v3.TransportTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\ttransport\x12d\n" +
	"\bprotocol\x18\x03 \x01(\x0e2>.envoy.extensions.filters.network.thrift_proxy.v3.ProtocolTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\bprotocol\x12(\n" +
	"\vstat_prefix\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"statPrefix\x12\x80\x01\n" +
	"\froute_config\x18\x04 \x01(\v2D.envoy.extensions.filters.network.thrift_proxy.v3.RouteConfigurationB\x17\xf2\x98\xfe\x8f\x05\x11\x12\x0froute_specifierR\vrouteConfig\x12c\n" +
	"\x04trds\x18\b \x01(\v26.envoy.extensions.filters.network.thrift_proxy.v3.TrdsB\x17\xf2\x98\xfe\x8f\x05\x11\x12\x0froute_specifierR\x04trds\x12e\n" +
	"\x0ethrift_filters\x18\x05 \x03(\v2>.envoy.extensions.filters.network.thrift_proxy.v3.ThriftFilterR\rthriftFilters\x12/\n" +
	"\x13payload_passthrough\x18\x06 \x01(\bR\x12payloadPassthrough\x12[\n" +
	"\x1bmax_requests_per_connection\x18\a \x01(\v2\x1c.google.protobuf.UInt32ValueR\x18maxRequestsPerConnection\x12C\n" +
	"\n" +
	"access_log\x18\t \x03(\v2$.envoy.config.accesslog.v3.AccessLogR\taccessLog\x129\n" +
	"\x19header_keys_preserve_case\x18\n" +
	" \x01(\bR\x16headerKeysPreserveCase:D\x9aň\x1e?\n" +
	"=envoy.config.filter.network.thrift_proxy.v2alpha1.ThriftProxy\"\xca\x01\n" +
	"\fThriftFilter\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04name\x129\n" +
	"\ftyped_config\x18\x03 \x01(\v2\x14.google.protobuf.AnyH\x00R\vtypedConfig:E\x9aň\x1e@\n" +
	">envoy.config.filter.network.thrift_proxy.v2alpha1.ThriftFilterB\r\n" +
	"\vconfig_typeJ\x04\b\x02\x10\x03R\x06config\"\xb6\x02\n" +
	"\x15ThriftProtocolOptions\x12g\n" +
	"\ttransport\x18\x01 \x01(\x0e2?.envoy.extensions.filters.network.thrift_proxy.v3.TransportTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\ttransport\x12d\n" +
	"\bprotocol\x18\x02 \x01(\x0e2>.envoy.extensions.filters.network.thrift_proxy.v3.ProtocolTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\bprotocol:N\x9aň\x1eI\n" +
	"Genvoy.config.filter.network.thrift_proxy.v2alpha1.ThriftProtocolOptions*I\n" +
	"\rTransportType\x12\x12\n" +
	"\x0eAUTO_TRANSPORT\x10\x00\x12\n" +
	"\n" +
	"\x06FRAMED\x10\x01\x12\f\n" +
	"\bUNFRAMED\x10\x02\x12\n" +
	"\n" +
	"\x06HEADER\x10\x03*d\n" +
	"\fProtocolType\x12\x11\n" +
	"\rAUTO_PROTOCOL\x10\x00\x12\n" +
	"\n" +
	"\x06BINARY\x10\x01\x12\x0e\n" +
	"\n" +
	"LAX_BINARY\x10\x02\x12\v\n" +
	"\aCOMPACT\x10\x03\x12\x18\n" +
	"\aTWITTER\x10\x04\x1a\v\x8a\xf4\x9b\xb3\x05\x033.0\b\x01B\xc4\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	">io.envoyproxy.envoy.extensions.filters.network.thrift_proxy.v3B\x10ThriftProxyProtoP\x01Zfgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/thrift_proxy/v3;thrift_proxyv3b\x06proto3"

var (
	file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDesc), len(file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDescData
}

var file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_enumTypes = make([]protoimpl.EnumInfo, 2)
var file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_msgTypes = make([]protoimpl.MessageInfo, 4)
var file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_goTypes = []any{
	(TransportType)(0),             // 0: envoy.extensions.filters.network.thrift_proxy.v3.TransportType
	(ProtocolType)(0),              // 1: envoy.extensions.filters.network.thrift_proxy.v3.ProtocolType
	(*Trds)(nil),                   // 2: envoy.extensions.filters.network.thrift_proxy.v3.Trds
	(*ThriftProxy)(nil),            // 3: envoy.extensions.filters.network.thrift_proxy.v3.ThriftProxy
	(*ThriftFilter)(nil),           // 4: envoy.extensions.filters.network.thrift_proxy.v3.ThriftFilter
	(*ThriftProtocolOptions)(nil),  // 5: envoy.extensions.filters.network.thrift_proxy.v3.ThriftProtocolOptions
	(*v3.ConfigSource)(nil),        // 6: envoy.config.core.v3.ConfigSource
	(*RouteConfiguration)(nil),     // 7: envoy.extensions.filters.network.thrift_proxy.v3.RouteConfiguration
	(*wrapperspb.UInt32Value)(nil), // 8: google.protobuf.UInt32Value
	(*v31.AccessLog)(nil),          // 9: envoy.config.accesslog.v3.AccessLog
	(*anypb.Any)(nil),              // 10: google.protobuf.Any
}
var file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_depIdxs = []int32{
	6,  // 0: envoy.extensions.filters.network.thrift_proxy.v3.Trds.config_source:type_name -> envoy.config.core.v3.ConfigSource
	0,  // 1: envoy.extensions.filters.network.thrift_proxy.v3.ThriftProxy.transport:type_name -> envoy.extensions.filters.network.thrift_proxy.v3.TransportType
	1,  // 2: envoy.extensions.filters.network.thrift_proxy.v3.ThriftProxy.protocol:type_name -> envoy.extensions.filters.network.thrift_proxy.v3.ProtocolType
	7,  // 3: envoy.extensions.filters.network.thrift_proxy.v3.ThriftProxy.route_config:type_name -> envoy.extensions.filters.network.thrift_proxy.v3.RouteConfiguration
	2,  // 4: envoy.extensions.filters.network.thrift_proxy.v3.ThriftProxy.trds:type_name -> envoy.extensions.filters.network.thrift_proxy.v3.Trds
	4,  // 5: envoy.extensions.filters.network.thrift_proxy.v3.ThriftProxy.thrift_filters:type_name -> envoy.extensions.filters.network.thrift_proxy.v3.ThriftFilter
	8,  // 6: envoy.extensions.filters.network.thrift_proxy.v3.ThriftProxy.max_requests_per_connection:type_name -> google.protobuf.UInt32Value
	9,  // 7: envoy.extensions.filters.network.thrift_proxy.v3.ThriftProxy.access_log:type_name -> envoy.config.accesslog.v3.AccessLog
	10, // 8: envoy.extensions.filters.network.thrift_proxy.v3.ThriftFilter.typed_config:type_name -> google.protobuf.Any
	0,  // 9: envoy.extensions.filters.network.thrift_proxy.v3.ThriftProtocolOptions.transport:type_name -> envoy.extensions.filters.network.thrift_proxy.v3.TransportType
	1,  // 10: envoy.extensions.filters.network.thrift_proxy.v3.ThriftProtocolOptions.protocol:type_name -> envoy.extensions.filters.network.thrift_proxy.v3.ProtocolType
	11, // [11:11] is the sub-list for method output_type
	11, // [11:11] is the sub-list for method input_type
	11, // [11:11] is the sub-list for extension type_name
	11, // [11:11] is the sub-list for extension extendee
	0,  // [0:11] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_init() }
func file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_init() {
	if File_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto != nil {
		return
	}
	file_envoy_extensions_filters_network_thrift_proxy_v3_route_proto_init()
	file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_msgTypes[2].OneofWrappers = []any{
		(*ThriftFilter_TypedConfig)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDesc), len(file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_rawDesc)),
			NumEnums:      2,
			NumMessages:   4,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_depIdxs,
		EnumInfos:         file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_enumTypes,
		MessageInfos:      file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto = out.File
	file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_goTypes = nil
	file_envoy_extensions_filters_network_thrift_proxy_v3_thrift_proxy_proto_depIdxs = nil
}
