// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/dubbo_proxy/v3/dubbo_proxy.proto

package dubbo_proxyv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	anypb "google.golang.org/protobuf/types/known/anypb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Dubbo Protocol types supported by Envoy.
type ProtocolType int32

const (
	// the default protocol.
	ProtocolType_Dubbo ProtocolType = 0
)

// Enum value maps for ProtocolType.
var (
	ProtocolType_name = map[int32]string{
		0: "Dubbo",
	}
	ProtocolType_value = map[string]int32{
		"Dubbo": 0,
	}
)

func (x ProtocolType) Enum() *ProtocolType {
	p := new(ProtocolType)
	*p = x
	return p
}

func (x ProtocolType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (ProtocolType) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_enumTypes[0].Descriptor()
}

func (ProtocolType) Type() protoreflect.EnumType {
	return &file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_enumTypes[0]
}

func (x ProtocolType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use ProtocolType.Descriptor instead.
func (ProtocolType) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDescGZIP(), []int{0}
}

// Dubbo Serialization types supported by Envoy.
type SerializationType int32

const (
	// the default serialization protocol.
	SerializationType_Hessian2 SerializationType = 0
)

// Enum value maps for SerializationType.
var (
	SerializationType_name = map[int32]string{
		0: "Hessian2",
	}
	SerializationType_value = map[string]int32{
		"Hessian2": 0,
	}
)

func (x SerializationType) Enum() *SerializationType {
	p := new(SerializationType)
	*p = x
	return p
}

func (x SerializationType) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (SerializationType) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_enumTypes[1].Descriptor()
}

func (SerializationType) Type() protoreflect.EnumType {
	return &file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_enumTypes[1]
}

func (x SerializationType) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use SerializationType.Descriptor instead.
func (SerializationType) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDescGZIP(), []int{1}
}

type Drds struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Configuration source specifier.
	// In case of “api_config_source“ only aggregated “api_type“ is supported.
	ConfigSource *v3.ConfigSource `protobuf:"bytes,1,opt,name=config_source,json=configSource,proto3" json:"config_source,omitempty"`
	// The name of the multiple route configuration. This allows to use different multiple route
	// configurations. Tells which multiple route configuration should be fetched from the configuration
	// source. Leave unspecified is also valid and means the unnamed multiple route configuration.
	RouteConfigName string `protobuf:"bytes,2,opt,name=route_config_name,json=routeConfigName,proto3" json:"route_config_name,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *Drds) Reset() {
	*x = Drds{}
	mi := &file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Drds) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Drds) ProtoMessage() {}

func (x *Drds) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Drds.ProtoReflect.Descriptor instead.
func (*Drds) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDescGZIP(), []int{0}
}

func (x *Drds) GetConfigSource() *v3.ConfigSource {
	if x != nil {
		return x.ConfigSource
	}
	return nil
}

func (x *Drds) GetRouteConfigName() string {
	if x != nil {
		return x.RouteConfigName
	}
	return ""
}

// [#next-free-field: 8]
type DubboProxy struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The human readable prefix to use when emitting statistics.
	StatPrefix string `protobuf:"bytes,1,opt,name=stat_prefix,json=statPrefix,proto3" json:"stat_prefix,omitempty"`
	// Configure the protocol used.
	ProtocolType ProtocolType `protobuf:"varint,2,opt,name=protocol_type,json=protocolType,proto3,enum=envoy.extensions.filters.network.dubbo_proxy.v3.ProtocolType" json:"protocol_type,omitempty"`
	// Configure the serialization protocol used.
	SerializationType SerializationType `protobuf:"varint,3,opt,name=serialization_type,json=serializationType,proto3,enum=envoy.extensions.filters.network.dubbo_proxy.v3.SerializationType" json:"serialization_type,omitempty"`
	// The route table for the connection manager is static and is specified in this property.
	//
	// .. note::
	//
	//	This field is deprecated. Please use ``drds`` or ``multiple_route_config`` first.
	//
	// Deprecated: Marked as deprecated in envoy/extensions/filters/network/dubbo_proxy/v3/dubbo_proxy.proto.
	RouteConfig []*RouteConfiguration `protobuf:"bytes,4,rep,name=route_config,json=routeConfig,proto3" json:"route_config,omitempty"`
	// Types that are valid to be assigned to RouteSpecifier:
	//
	//	*DubboProxy_Drds
	//	*DubboProxy_MultipleRouteConfig
	RouteSpecifier isDubboProxy_RouteSpecifier `protobuf_oneof:"route_specifier"`
	// A list of individual Dubbo filters that make up the filter chain for requests made to the
	// Dubbo proxy. Order matters as the filters are processed sequentially. For backwards
	// compatibility, if no dubbo_filters are specified, a default Dubbo router filter
	// (“envoy.filters.dubbo.router“) is used.
	DubboFilters  []*DubboFilter `protobuf:"bytes,5,rep,name=dubbo_filters,json=dubboFilters,proto3" json:"dubbo_filters,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DubboProxy) Reset() {
	*x = DubboProxy{}
	mi := &file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DubboProxy) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DubboProxy) ProtoMessage() {}

func (x *DubboProxy) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DubboProxy.ProtoReflect.Descriptor instead.
func (*DubboProxy) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDescGZIP(), []int{1}
}

func (x *DubboProxy) GetStatPrefix() string {
	if x != nil {
		return x.StatPrefix
	}
	return ""
}

func (x *DubboProxy) GetProtocolType() ProtocolType {
	if x != nil {
		return x.ProtocolType
	}
	return ProtocolType_Dubbo
}

func (x *DubboProxy) GetSerializationType() SerializationType {
	if x != nil {
		return x.SerializationType
	}
	return SerializationType_Hessian2
}

// Deprecated: Marked as deprecated in envoy/extensions/filters/network/dubbo_proxy/v3/dubbo_proxy.proto.
func (x *DubboProxy) GetRouteConfig() []*RouteConfiguration {
	if x != nil {
		return x.RouteConfig
	}
	return nil
}

func (x *DubboProxy) GetRouteSpecifier() isDubboProxy_RouteSpecifier {
	if x != nil {
		return x.RouteSpecifier
	}
	return nil
}

func (x *DubboProxy) GetDrds() *Drds {
	if x != nil {
		if x, ok := x.RouteSpecifier.(*DubboProxy_Drds); ok {
			return x.Drds
		}
	}
	return nil
}

func (x *DubboProxy) GetMultipleRouteConfig() *MultipleRouteConfiguration {
	if x != nil {
		if x, ok := x.RouteSpecifier.(*DubboProxy_MultipleRouteConfig); ok {
			return x.MultipleRouteConfig
		}
	}
	return nil
}

func (x *DubboProxy) GetDubboFilters() []*DubboFilter {
	if x != nil {
		return x.DubboFilters
	}
	return nil
}

type isDubboProxy_RouteSpecifier interface {
	isDubboProxy_RouteSpecifier()
}

type DubboProxy_Drds struct {
	// Use xDS to fetch the route configuration. It is invalid to define both “route_config“ and “drds“.
	Drds *Drds `protobuf:"bytes,6,opt,name=drds,proto3,oneof"`
}

type DubboProxy_MultipleRouteConfig struct {
	MultipleRouteConfig *MultipleRouteConfiguration `protobuf:"bytes,7,opt,name=multiple_route_config,json=multipleRouteConfig,proto3,oneof"`
}

func (*DubboProxy_Drds) isDubboProxy_RouteSpecifier() {}

func (*DubboProxy_MultipleRouteConfig) isDubboProxy_RouteSpecifier() {}

// DubboFilter configures a Dubbo filter.
type DubboFilter struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The name of the filter to instantiate. The name must match a supported
	// filter.
	Name string `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	// Filter specific configuration which depends on the filter being
	// instantiated. See the supported filters for further documentation.
	Config        *anypb.Any `protobuf:"bytes,2,opt,name=config,proto3" json:"config,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *DubboFilter) Reset() {
	*x = DubboFilter{}
	mi := &file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *DubboFilter) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*DubboFilter) ProtoMessage() {}

func (x *DubboFilter) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use DubboFilter.ProtoReflect.Descriptor instead.
func (*DubboFilter) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDescGZIP(), []int{2}
}

func (x *DubboFilter) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *DubboFilter) GetConfig() *anypb.Any {
	if x != nil {
		return x.Config
	}
	return nil
}

var File_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDesc = "" +
	"\n" +
	"Aenvoy/extensions/filters/network/dubbo_proxy/v3/dubbo_proxy.proto\x12/envoy.extensions.filters.network.dubbo_proxy.v3\x1a(envoy/config/core/v3/config_source.proto\x1a;envoy/extensions/filters/network/dubbo_proxy/v3/route.proto\x1a\x19google/protobuf/any.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x85\x01\n" +
	"\x04Drds\x12Q\n" +
	"\rconfig_source\x18\x01 \x01(\v2\".envoy.config.core.v3.ConfigSourceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\fconfigSource\x12*\n" +
	"\x11route_config_name\x18\x02 \x01(\tR\x0frouteConfigName\"\xba\x06\n" +
	"\n" +
	"DubboProxy\x12(\n" +
	"\vstat_prefix\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"statPrefix\x12l\n" +
	"\rprotocol_type\x18\x02 \x01(\x0e2=.envoy.extensions.filters.network.dubbo_proxy.v3.ProtocolTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\fprotocolType\x12{\n" +
	"\x12serialization_type\x18\x03 \x01(\x0e2B.envoy.extensions.filters.network.dubbo_proxy.v3.SerializationTypeB\b\xfaB\x05\x82\x01\x02\x10\x01R\x11serializationType\x12s\n" +
	"\froute_config\x18\x04 \x03(\v2C.envoy.extensions.filters.network.dubbo_proxy.v3.RouteConfigurationB\v\x92ǆ\xd8\x04\x033.0\x18\x01R\vrouteConfig\x12d\n" +
	"\x04drds\x18\x06 \x01(\v25.envoy.extensions.filters.network.dubbo_proxy.v3.DrdsB\x17\xf2\x98\xfe\x8f\x05\x11\x12\x0froute_specifierH\x00R\x04drds\x12\x81\x01\n" +
	"\x15multiple_route_config\x18\a \x01(\v2K.envoy.extensions.filters.network.dubbo_proxy.v3.MultipleRouteConfigurationH\x00R\x13multipleRouteConfig\x12a\n" +
	"\rdubbo_filters\x18\x05 \x03(\v2<.envoy.extensions.filters.network.dubbo_proxy.v3.DubboFilterR\fdubboFilters:B\x9aň\x1e=\n" +
	";envoy.config.filter.network.dubbo_proxy.v2alpha1.DubboProxyB\x11\n" +
	"\x0froute_specifier\"\x9d\x01\n" +
	"\vDubboFilter\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04name\x12,\n" +
	"\x06config\x18\x02 \x01(\v2\x14.google.protobuf.AnyR\x06config:C\x9aň\x1e>\n" +
	"<envoy.config.filter.network.dubbo_proxy.v2alpha1.DubboFilter*\x19\n" +
	"\fProtocolType\x12\t\n" +
	"\x05Dubbo\x10\x00*!\n" +
	"\x11SerializationType\x12\f\n" +
	"\bHessian2\x10\x00B\xc0\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"=io.envoyproxy.envoy.extensions.filters.network.dubbo_proxy.v3B\x0fDubboProxyProtoP\x01Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/dubbo_proxy/v3;dubbo_proxyv3b\x06proto3"

var (
	file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDesc), len(file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDescData
}

var file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_enumTypes = make([]protoimpl.EnumInfo, 2)
var file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_msgTypes = make([]protoimpl.MessageInfo, 3)
var file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_goTypes = []any{
	(ProtocolType)(0),                  // 0: envoy.extensions.filters.network.dubbo_proxy.v3.ProtocolType
	(SerializationType)(0),             // 1: envoy.extensions.filters.network.dubbo_proxy.v3.SerializationType
	(*Drds)(nil),                       // 2: envoy.extensions.filters.network.dubbo_proxy.v3.Drds
	(*DubboProxy)(nil),                 // 3: envoy.extensions.filters.network.dubbo_proxy.v3.DubboProxy
	(*DubboFilter)(nil),                // 4: envoy.extensions.filters.network.dubbo_proxy.v3.DubboFilter
	(*v3.ConfigSource)(nil),            // 5: envoy.config.core.v3.ConfigSource
	(*RouteConfiguration)(nil),         // 6: envoy.extensions.filters.network.dubbo_proxy.v3.RouteConfiguration
	(*MultipleRouteConfiguration)(nil), // 7: envoy.extensions.filters.network.dubbo_proxy.v3.MultipleRouteConfiguration
	(*anypb.Any)(nil),                  // 8: google.protobuf.Any
}
var file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_depIdxs = []int32{
	5, // 0: envoy.extensions.filters.network.dubbo_proxy.v3.Drds.config_source:type_name -> envoy.config.core.v3.ConfigSource
	0, // 1: envoy.extensions.filters.network.dubbo_proxy.v3.DubboProxy.protocol_type:type_name -> envoy.extensions.filters.network.dubbo_proxy.v3.ProtocolType
	1, // 2: envoy.extensions.filters.network.dubbo_proxy.v3.DubboProxy.serialization_type:type_name -> envoy.extensions.filters.network.dubbo_proxy.v3.SerializationType
	6, // 3: envoy.extensions.filters.network.dubbo_proxy.v3.DubboProxy.route_config:type_name -> envoy.extensions.filters.network.dubbo_proxy.v3.RouteConfiguration
	2, // 4: envoy.extensions.filters.network.dubbo_proxy.v3.DubboProxy.drds:type_name -> envoy.extensions.filters.network.dubbo_proxy.v3.Drds
	7, // 5: envoy.extensions.filters.network.dubbo_proxy.v3.DubboProxy.multiple_route_config:type_name -> envoy.extensions.filters.network.dubbo_proxy.v3.MultipleRouteConfiguration
	4, // 6: envoy.extensions.filters.network.dubbo_proxy.v3.DubboProxy.dubbo_filters:type_name -> envoy.extensions.filters.network.dubbo_proxy.v3.DubboFilter
	8, // 7: envoy.extensions.filters.network.dubbo_proxy.v3.DubboFilter.config:type_name -> google.protobuf.Any
	8, // [8:8] is the sub-list for method output_type
	8, // [8:8] is the sub-list for method input_type
	8, // [8:8] is the sub-list for extension type_name
	8, // [8:8] is the sub-list for extension extendee
	0, // [0:8] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_init() }
func file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_init() {
	if File_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto != nil {
		return
	}
	file_envoy_extensions_filters_network_dubbo_proxy_v3_route_proto_init()
	file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_msgTypes[1].OneofWrappers = []any{
		(*DubboProxy_Drds)(nil),
		(*DubboProxy_MultipleRouteConfig)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDesc), len(file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_rawDesc)),
			NumEnums:      2,
			NumMessages:   3,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_depIdxs,
		EnumInfos:         file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_enumTypes,
		MessageInfos:      file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto = out.File
	file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_goTypes = nil
	file_envoy_extensions_filters_network_dubbo_proxy_v3_dubbo_proxy_proto_depIdxs = nil
}
