// Code generated by go-swagger; DO NOT EDIT.

// Copyright Authors of Cilium
// SPDX-License-Identifier: Apache-2.0

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"context"

	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/swag"
)

// BgpRoutePolicyPrefix Matches a CIDR prefix in a BGP route policy
//
// swagger:model BgpRoutePolicyPrefix
type BgpRoutePolicyPrefix struct {

	// CIDR prefix to match with
	Cidr string `json:"cidr,omitempty"`

	// Maximal prefix length that will match if it falls under CIDR
	PrefixLenMax int64 `json:"prefix-len-max,omitempty"`

	// Minimal prefix length that will match if it falls under CIDR
	PrefixLenMin int64 `json:"prefix-len-min,omitempty"`
}

// Validate validates this bgp route policy prefix
func (m *BgpRoutePolicyPrefix) Validate(formats strfmt.Registry) error {
	return nil
}

// ContextValidate validates this bgp route policy prefix based on context it is used
func (m *BgpRoutePolicyPrefix) ContextValidate(ctx context.Context, formats strfmt.Registry) error {
	return nil
}

// MarshalBinary interface implementation
func (m *BgpRoutePolicyPrefix) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *BgpRoutePolicyPrefix) UnmarshalBinary(b []byte) error {
	var res BgpRoutePolicyPrefix
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}
