// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.6
// 	protoc        v5.29.3
// source: cilium/api/network_filter.proto

package cilium

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type NetworkFilter struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Path to the proxylib to be opened
	Proxylib string `protobuf:"bytes,1,opt,name=proxylib,proto3" json:"proxylib,omitempty"`
	// Transparent set of parameters provided for proxylib initialization
	ProxylibParams map[string]string `protobuf:"bytes,2,rep,name=proxylib_params,json=proxylibParams,proto3" json:"proxylib_params,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	// Path to the unix domain socket for the cilium access log.
	AccessLogPath string `protobuf:"bytes,5,opt,name=access_log_path,json=accessLogPath,proto3" json:"access_log_path,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *NetworkFilter) Reset() {
	*x = NetworkFilter{}
	mi := &file_cilium_api_network_filter_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *NetworkFilter) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*NetworkFilter) ProtoMessage() {}

func (x *NetworkFilter) ProtoReflect() protoreflect.Message {
	mi := &file_cilium_api_network_filter_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use NetworkFilter.ProtoReflect.Descriptor instead.
func (*NetworkFilter) Descriptor() ([]byte, []int) {
	return file_cilium_api_network_filter_proto_rawDescGZIP(), []int{0}
}

func (x *NetworkFilter) GetProxylib() string {
	if x != nil {
		return x.Proxylib
	}
	return ""
}

func (x *NetworkFilter) GetProxylibParams() map[string]string {
	if x != nil {
		return x.ProxylibParams
	}
	return nil
}

func (x *NetworkFilter) GetAccessLogPath() string {
	if x != nil {
		return x.AccessLogPath
	}
	return ""
}

var File_cilium_api_network_filter_proto protoreflect.FileDescriptor

const file_cilium_api_network_filter_proto_rawDesc = "" +
	"\n" +
	"\x1fcilium/api/network_filter.proto\x12\x06cilium\"\xea\x01\n" +
	"\rNetworkFilter\x12\x1a\n" +
	"\bproxylib\x18\x01 \x01(\tR\bproxylib\x12R\n" +
	"\x0fproxylib_params\x18\x02 \x03(\v2).cilium.NetworkFilter.ProxylibParamsEntryR\x0eproxylibParams\x12&\n" +
	"\x0faccess_log_path\x18\x05 \x01(\tR\raccessLogPath\x1aA\n" +
	"\x13ProxylibParamsEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01B.Z,github.com/cilium/proxy/go/cilium/api;ciliumb\x06proto3"

var (
	file_cilium_api_network_filter_proto_rawDescOnce sync.Once
	file_cilium_api_network_filter_proto_rawDescData []byte
)

func file_cilium_api_network_filter_proto_rawDescGZIP() []byte {
	file_cilium_api_network_filter_proto_rawDescOnce.Do(func() {
		file_cilium_api_network_filter_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_cilium_api_network_filter_proto_rawDesc), len(file_cilium_api_network_filter_proto_rawDesc)))
	})
	return file_cilium_api_network_filter_proto_rawDescData
}

var file_cilium_api_network_filter_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_cilium_api_network_filter_proto_goTypes = []any{
	(*NetworkFilter)(nil), // 0: cilium.NetworkFilter
	nil,                   // 1: cilium.NetworkFilter.ProxylibParamsEntry
}
var file_cilium_api_network_filter_proto_depIdxs = []int32{
	1, // 0: cilium.NetworkFilter.proxylib_params:type_name -> cilium.NetworkFilter.ProxylibParamsEntry
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_cilium_api_network_filter_proto_init() }
func file_cilium_api_network_filter_proto_init() {
	if File_cilium_api_network_filter_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_cilium_api_network_filter_proto_rawDesc), len(file_cilium_api_network_filter_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_cilium_api_network_filter_proto_goTypes,
		DependencyIndexes: file_cilium_api_network_filter_proto_depIdxs,
		MessageInfos:      file_cilium_api_network_filter_proto_msgTypes,
	}.Build()
	File_cilium_api_network_filter_proto = out.File
	file_cilium_api_network_filter_proto_goTypes = nil
	file_cilium_api_network_filter_proto_depIdxs = nil
}
