// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/service/route/v3/rds.proto

package routev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/service/discovery/v3"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#not-implemented-hide:] Not configuration. Workaround c++ protobuf issue with importing
// services: https://github.com/google/protobuf/issues/4221 and protoxform to upgrade the file.
type RdsDummy struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RdsDummy) Reset() {
	*x = RdsDummy{}
	mi := &file_envoy_service_route_v3_rds_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RdsDummy) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RdsDummy) ProtoMessage() {}

func (x *RdsDummy) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_route_v3_rds_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RdsDummy.ProtoReflect.Descriptor instead.
func (*RdsDummy) Descriptor() ([]byte, []int) {
	return file_envoy_service_route_v3_rds_proto_rawDescGZIP(), []int{0}
}

var File_envoy_service_route_v3_rds_proto protoreflect.FileDescriptor

const file_envoy_service_route_v3_rds_proto_rawDesc = "" +
	"\n" +
	" envoy/service/route/v3/rds.proto\x12\x16envoy.service.route.v3\x1a*envoy/service/discovery/v3/discovery.proto\x1a\x1cgoogle/api/annotations.proto\x1a envoy/annotations/resource.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"(\n" +
	"\bRdsDummy:\x1c\x9aň\x1e\x17\n" +
	"\x15envoy.api.v2.RdsDummy2\xc6\x03\n" +
	"\x15RouteDiscoveryService\x12q\n" +
	"\fStreamRoutes\x12,.envoy.service.discovery.v3.DiscoveryRequest\x1a-.envoy.service.discovery.v3.DiscoveryResponse\"\x00(\x010\x01\x12z\n" +
	"\vDeltaRoutes\x121.envoy.service.discovery.v3.DeltaDiscoveryRequest\x1a2.envoy.service.discovery.v3.DeltaDiscoveryResponse\"\x00(\x010\x01\x12\x8b\x01\n" +
	"\vFetchRoutes\x12,.envoy.service.discovery.v3.DiscoveryRequest\x1a-.envoy.service.discovery.v3.DiscoveryResponse\"\x1f\x82\xd3\xe4\x93\x02\x19:\x01*\"\x14/v3/discovery:routes\x1a0\x8a\xa4\x96\xf3\a*\n" +
	"(envoy.config.route.v3.RouteConfiguration2\xcb\x01\n" +
	"\x1bVirtualHostDiscoveryService\x12\x80\x01\n" +
	"\x11DeltaVirtualHosts\x121.envoy.service.discovery.v3.DeltaDiscoveryRequest\x1a2.envoy.service.discovery.v3.DeltaDiscoveryResponse\"\x00(\x010\x01\x1a)\x8a\xa4\x96\xf3\a#\n" +
	"!envoy.config.route.v3.VirtualHostB\x81\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"$io.envoyproxy.envoy.service.route.v3B\bRdsProtoP\x01ZEgithub.com/envoyproxy/go-control-plane/envoy/service/route/v3;routev3b\x06proto3"

var (
	file_envoy_service_route_v3_rds_proto_rawDescOnce sync.Once
	file_envoy_service_route_v3_rds_proto_rawDescData []byte
)

func file_envoy_service_route_v3_rds_proto_rawDescGZIP() []byte {
	file_envoy_service_route_v3_rds_proto_rawDescOnce.Do(func() {
		file_envoy_service_route_v3_rds_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_service_route_v3_rds_proto_rawDesc), len(file_envoy_service_route_v3_rds_proto_rawDesc)))
	})
	return file_envoy_service_route_v3_rds_proto_rawDescData
}

var file_envoy_service_route_v3_rds_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_service_route_v3_rds_proto_goTypes = []any{
	(*RdsDummy)(nil),                  // 0: envoy.service.route.v3.RdsDummy
	(*v3.DiscoveryRequest)(nil),       // 1: envoy.service.discovery.v3.DiscoveryRequest
	(*v3.DeltaDiscoveryRequest)(nil),  // 2: envoy.service.discovery.v3.DeltaDiscoveryRequest
	(*v3.DiscoveryResponse)(nil),      // 3: envoy.service.discovery.v3.DiscoveryResponse
	(*v3.DeltaDiscoveryResponse)(nil), // 4: envoy.service.discovery.v3.DeltaDiscoveryResponse
}
var file_envoy_service_route_v3_rds_proto_depIdxs = []int32{
	1, // 0: envoy.service.route.v3.RouteDiscoveryService.StreamRoutes:input_type -> envoy.service.discovery.v3.DiscoveryRequest
	2, // 1: envoy.service.route.v3.RouteDiscoveryService.DeltaRoutes:input_type -> envoy.service.discovery.v3.DeltaDiscoveryRequest
	1, // 2: envoy.service.route.v3.RouteDiscoveryService.FetchRoutes:input_type -> envoy.service.discovery.v3.DiscoveryRequest
	2, // 3: envoy.service.route.v3.VirtualHostDiscoveryService.DeltaVirtualHosts:input_type -> envoy.service.discovery.v3.DeltaDiscoveryRequest
	3, // 4: envoy.service.route.v3.RouteDiscoveryService.StreamRoutes:output_type -> envoy.service.discovery.v3.DiscoveryResponse
	4, // 5: envoy.service.route.v3.RouteDiscoveryService.DeltaRoutes:output_type -> envoy.service.discovery.v3.DeltaDiscoveryResponse
	3, // 6: envoy.service.route.v3.RouteDiscoveryService.FetchRoutes:output_type -> envoy.service.discovery.v3.DiscoveryResponse
	4, // 7: envoy.service.route.v3.VirtualHostDiscoveryService.DeltaVirtualHosts:output_type -> envoy.service.discovery.v3.DeltaDiscoveryResponse
	4, // [4:8] is the sub-list for method output_type
	0, // [0:4] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_service_route_v3_rds_proto_init() }
func file_envoy_service_route_v3_rds_proto_init() {
	if File_envoy_service_route_v3_rds_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_service_route_v3_rds_proto_rawDesc), len(file_envoy_service_route_v3_rds_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   2,
		},
		GoTypes:           file_envoy_service_route_v3_rds_proto_goTypes,
		DependencyIndexes: file_envoy_service_route_v3_rds_proto_depIdxs,
		MessageInfos:      file_envoy_service_route_v3_rds_proto_msgTypes,
	}.Build()
	File_envoy_service_route_v3_rds_proto = out.File
	file_envoy_service_route_v3_rds_proto_goTypes = nil
	file_envoy_service_route_v3_rds_proto_depIdxs = nil
}
