// Code generated by go-swagger; DO NOT EDIT.

// Copyright Authors of Cilium
// SPDX-License-Identifier: Apache-2.0

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"context"

	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/swag"
)

// IPV4BigTCP Status of IPv4 BIG TCP
//
// swagger:model IPV4BigTCP
type IPV4BigTCP struct {

	// Is IPv4 BIG TCP enabled
	Enabled bool `json:"enabled,omitempty"`

	// Maximum IPv4 GRO size
	MaxGRO int64 `json:"maxGRO,omitempty"`

	// Maximum IPv4 GSO size
	MaxGSO int64 `json:"maxGSO,omitempty"`
}

// Validate validates this IP v4 big TCP
func (m *IPV4BigTCP) Validate(formats strfmt.Registry) error {
	return nil
}

// ContextValidate validates this IP v4 big TCP based on context it is used
func (m *IPV4BigTCP) ContextValidate(ctx context.Context, formats strfmt.Registry) error {
	return nil
}

// MarshalBinary interface implementation
func (m *IPV4BigTCP) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *IPV4BigTCP) UnmarshalBinary(b []byte) error {
	var res IPV4BigTCP
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}
