// Code generated by go-swagger; DO NOT EDIT.

// Copyright Authors of Cilium
// SPDX-License-Identifier: Apache-2.0

package daemon

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"net/http"

	"github.com/go-openapi/runtime"

	"github.com/cilium/cilium/api/v1/models"
)

// GetCgroupDumpMetadataOKCode is the HTTP code returned for type GetCgroupDumpMetadataOK
const GetCgroupDumpMetadataOKCode int = 200

/*
GetCgroupDumpMetadataOK Success

swagger:response getCgroupDumpMetadataOK
*/
type GetCgroupDumpMetadataOK struct {

	/*
	  In: Body
	*/
	Payload *models.CgroupDumpMetadata `json:"body,omitempty"`
}

// NewGetCgroupDumpMetadataOK creates GetCgroupDumpMetadataOK with default headers values
func NewGetCgroupDumpMetadataOK() *GetCgroupDumpMetadataOK {

	return &GetCgroupDumpMetadataOK{}
}

// WithPayload adds the payload to the get cgroup dump metadata o k response
func (o *GetCgroupDumpMetadataOK) WithPayload(payload *models.CgroupDumpMetadata) *GetCgroupDumpMetadataOK {
	o.Payload = payload
	return o
}

// SetPayload sets the payload to the get cgroup dump metadata o k response
func (o *GetCgroupDumpMetadataOK) SetPayload(payload *models.CgroupDumpMetadata) {
	o.Payload = payload
}

// WriteResponse to the client
func (o *GetCgroupDumpMetadataOK) WriteResponse(rw http.ResponseWriter, producer runtime.Producer) {

	rw.WriteHeader(200)
	if o.Payload != nil {
		payload := o.Payload
		if err := producer.Produce(rw, payload); err != nil {
			panic(err) // let the recovery middleware deal with this
		}
	}
}

// GetCgroupDumpMetadataFailureCode is the HTTP code returned for type GetCgroupDumpMetadataFailure
const GetCgroupDumpMetadataFailureCode int = 500

/*
GetCgroupDumpMetadataFailure CgroupDumpMetadata get failed

swagger:response getCgroupDumpMetadataFailure
*/
type GetCgroupDumpMetadataFailure struct {

	/*
	  In: Body
	*/
	Payload models.Error `json:"body,omitempty"`
}

// NewGetCgroupDumpMetadataFailure creates GetCgroupDumpMetadataFailure with default headers values
func NewGetCgroupDumpMetadataFailure() *GetCgroupDumpMetadataFailure {

	return &GetCgroupDumpMetadataFailure{}
}

// WithPayload adds the payload to the get cgroup dump metadata failure response
func (o *GetCgroupDumpMetadataFailure) WithPayload(payload models.Error) *GetCgroupDumpMetadataFailure {
	o.Payload = payload
	return o
}

// SetPayload sets the payload to the get cgroup dump metadata failure response
func (o *GetCgroupDumpMetadataFailure) SetPayload(payload models.Error) {
	o.Payload = payload
}

// WriteResponse to the client
func (o *GetCgroupDumpMetadataFailure) WriteResponse(rw http.ResponseWriter, producer runtime.Producer) {

	rw.WriteHeader(500)
	payload := o.Payload
	if err := producer.Produce(rw, payload); err != nil {
		panic(err) // let the recovery middleware deal with this
	}
}
