//go:build !ignore_autogenerated
// +build !ignore_autogenerated

// SPDX-License-Identifier: Apache-2.0
// Copyright Authors of Cilium

// Code generated by deepcopy-gen. DO NOT EDIT.

package node

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *LocalNode) DeepCopyInto(out *LocalNode) {
	*out = *in
	in.Node.DeepCopyInto(&out.Node)
	if in.Local != nil {
		in, out := &in.Local, &out.Local
		*out = (*in).DeepCopy()
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new LocalNode.
func (in *LocalNode) DeepCopy() *LocalNode {
	if in == nil {
		return nil
	}
	out := new(LocalNode)
	in.DeepCopyInto(out)
	return out
}
