// SPDX-License-Identifier: Apache-2.0
// Copyright Authors of Cilium

// Code generated by client-gen. DO NOT EDIT.

package fake

import (
	v2 "github.com/cilium/cilium/pkg/k8s/client/clientset/versioned/typed/cilium.io/v2"
	rest "k8s.io/client-go/rest"
	testing "k8s.io/client-go/testing"
)

type FakeCiliumV2 struct {
	*testing.Fake
}

func (c *FakeCiliumV2) CiliumBGPAdvertisements() v2.CiliumBGPAdvertisementInterface {
	return newFakeCiliumBGPAdvertisements(c)
}

func (c *FakeCiliumV2) CiliumBGPClusterConfigs() v2.CiliumBGPClusterConfigInterface {
	return newFakeCiliumBGPClusterConfigs(c)
}

func (c *FakeCiliumV2) CiliumBGPNodeConfigs() v2.CiliumBGPNodeConfigInterface {
	return newFakeCiliumBGPNodeConfigs(c)
}

func (c *FakeCiliumV2) CiliumBGPNodeConfigOverrides() v2.CiliumBGPNodeConfigOverrideInterface {
	return newFakeCiliumBGPNodeConfigOverrides(c)
}

func (c *FakeCiliumV2) CiliumBGPPeerConfigs() v2.CiliumBGPPeerConfigInterface {
	return newFakeCiliumBGPPeerConfigs(c)
}

func (c *FakeCiliumV2) CiliumCIDRGroups() v2.CiliumCIDRGroupInterface {
	return newFakeCiliumCIDRGroups(c)
}

func (c *FakeCiliumV2) CiliumClusterwideEnvoyConfigs() v2.CiliumClusterwideEnvoyConfigInterface {
	return newFakeCiliumClusterwideEnvoyConfigs(c)
}

func (c *FakeCiliumV2) CiliumClusterwideNetworkPolicies() v2.CiliumClusterwideNetworkPolicyInterface {
	return newFakeCiliumClusterwideNetworkPolicies(c)
}

func (c *FakeCiliumV2) CiliumEgressGatewayPolicies() v2.CiliumEgressGatewayPolicyInterface {
	return newFakeCiliumEgressGatewayPolicies(c)
}

func (c *FakeCiliumV2) CiliumEndpoints(namespace string) v2.CiliumEndpointInterface {
	return newFakeCiliumEndpoints(c, namespace)
}

func (c *FakeCiliumV2) CiliumEnvoyConfigs(namespace string) v2.CiliumEnvoyConfigInterface {
	return newFakeCiliumEnvoyConfigs(c, namespace)
}

func (c *FakeCiliumV2) CiliumIdentities() v2.CiliumIdentityInterface {
	return newFakeCiliumIdentities(c)
}

func (c *FakeCiliumV2) CiliumLoadBalancerIPPools() v2.CiliumLoadBalancerIPPoolInterface {
	return newFakeCiliumLoadBalancerIPPools(c)
}

func (c *FakeCiliumV2) CiliumLocalRedirectPolicies(namespace string) v2.CiliumLocalRedirectPolicyInterface {
	return newFakeCiliumLocalRedirectPolicies(c, namespace)
}

func (c *FakeCiliumV2) CiliumNetworkPolicies(namespace string) v2.CiliumNetworkPolicyInterface {
	return newFakeCiliumNetworkPolicies(c, namespace)
}

func (c *FakeCiliumV2) CiliumNodes() v2.CiliumNodeInterface {
	return newFakeCiliumNodes(c)
}

func (c *FakeCiliumV2) CiliumNodeConfigs(namespace string) v2.CiliumNodeConfigInterface {
	return newFakeCiliumNodeConfigs(c, namespace)
}

// RESTClient returns a RESTClient that is used to communicate
// with API server by this client implementation.
func (c *FakeCiliumV2) RESTClient() rest.Interface {
	var ret *rest.RESTClient
	return ret
}
