// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/thrift_proxy/router/v3/router.proto

package routerv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Router struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Close downstream connection in case of routing or upstream connection problem. Default: true
	CloseDownstreamOnUpstreamError *wrapperspb.BoolValue `protobuf:"bytes,1,opt,name=close_downstream_on_upstream_error,json=closeDownstreamOnUpstreamError,proto3" json:"close_downstream_on_upstream_error,omitempty"`
	unknownFields                  protoimpl.UnknownFields
	sizeCache                      protoimpl.SizeCache
}

func (x *Router) Reset() {
	*x = Router{}
	mi := &file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Router) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Router) ProtoMessage() {}

func (x *Router) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Router.ProtoReflect.Descriptor instead.
func (*Router) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDescGZIP(), []int{0}
}

func (x *Router) GetCloseDownstreamOnUpstreamError() *wrapperspb.BoolValue {
	if x != nil {
		return x.CloseDownstreamOnUpstreamError
	}
	return nil
}

var File_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDesc = "" +
	"\n" +
	"Denvoy/extensions/filters/network/thrift_proxy/router/v3/router.proto\x127envoy.extensions.filters.network.thrift_proxy.router.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\xaa\x01\n" +
	"\x06Router\x12f\n" +
	"\"close_downstream_on_upstream_error\x18\x01 \x01(\v2\x1a.google.protobuf.BoolValueR\x1ecloseDownstreamOnUpstreamError:8\x9aň\x1e3\n" +
	"1envoy.config.filter.thrift.router.v2alpha1.RouterB\xc7\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Eio.envoyproxy.envoy.extensions.filters.network.thrift_proxy.router.v3B\vRouterProtoP\x01Zggithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/thrift_proxy/router/v3;routerv3b\x06proto3"

var (
	file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDesc), len(file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDescData
}

var file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_goTypes = []any{
	(*Router)(nil),               // 0: envoy.extensions.filters.network.thrift_proxy.router.v3.Router
	(*wrapperspb.BoolValue)(nil), // 1: google.protobuf.BoolValue
}
var file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.filters.network.thrift_proxy.router.v3.Router.close_downstream_on_upstream_error:type_name -> google.protobuf.BoolValue
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_init() }
func file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_init() {
	if File_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDesc), len(file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto = out.File
	file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_goTypes = nil
	file_envoy_extensions_filters_network_thrift_proxy_router_v3_router_proto_depIdxs = nil
}
