// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/matching/input_matchers/runtime_fraction/v3/runtime_fraction.proto

package runtime_fractionv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// The runtime fraction matchers computes a hash from the input and matches if runtime feature is enabled
// for the the resulting hash. Every time the input is considered for a match, its hash must fall within
// the percentage of matches indicated by this field. For a fraction N/D, a number is computed as a hash
// of the input on a field in the range [0,D). If the number is less than or equal to the value of the
// numerator N, the matcher evaluates to true. A runtime_fraction input matcher can be used to gradually
// roll out matcher changes without requiring full code or configuration deployments.
// Note that distribution of matching results is only as good as one of the input.
type RuntimeFraction struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Match the input against the given runtime key. The specified default value is used if key is not
	// present in the runtime configuration.
	RuntimeFraction *v3.RuntimeFractionalPercent `protobuf:"bytes,1,opt,name=runtime_fraction,json=runtimeFraction,proto3" json:"runtime_fraction,omitempty"`
	// Optional seed passed through the hash function. This allows using additional information when computing
	// the hash value: by changing the seed value, a potentially different outcome can be achieved for the same input.
	Seed          uint64 `protobuf:"varint,2,opt,name=seed,proto3" json:"seed,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RuntimeFraction) Reset() {
	*x = RuntimeFraction{}
	mi := &file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RuntimeFraction) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RuntimeFraction) ProtoMessage() {}

func (x *RuntimeFraction) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RuntimeFraction.ProtoReflect.Descriptor instead.
func (*RuntimeFraction) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDescGZIP(), []int{0}
}

func (x *RuntimeFraction) GetRuntimeFraction() *v3.RuntimeFractionalPercent {
	if x != nil {
		return x.RuntimeFraction
	}
	return nil
}

func (x *RuntimeFraction) GetSeed() uint64 {
	if x != nil {
		return x.Seed
	}
	return 0
}

var File_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto protoreflect.FileDescriptor

const file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDesc = "" +
	"\n" +
	"Senvoy/extensions/matching/input_matchers/runtime_fraction/v3/runtime_fraction.proto\x12<envoy.extensions.matching.input_matchers.runtime_fraction.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x8a\x01\n" +
	"\x0fRuntimeFraction\x12c\n" +
	"\x10runtime_fraction\x18\x01 \x01(\v2..envoy.config.core.v3.RuntimeFractionalPercentB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x0fruntimeFraction\x12\x12\n" +
	"\x04seed\x18\x02 \x01(\x04R\x04seedB\xe4\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Jio.envoyproxy.envoy.extensions.matching.input_matchers.runtime_fraction.v3B\x14RuntimeFractionProtoP\x01Zvgithub.com/envoyproxy/go-control-plane/envoy/extensions/matching/input_matchers/runtime_fraction/v3;runtime_fractionv3b\x06proto3"

var (
	file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDescOnce sync.Once
	file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDescData []byte
)

func file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDescGZIP() []byte {
	file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDesc), len(file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDesc)))
	})
	return file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDescData
}

var file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_goTypes = []any{
	(*RuntimeFraction)(nil),             // 0: envoy.extensions.matching.input_matchers.runtime_fraction.v3.RuntimeFraction
	(*v3.RuntimeFractionalPercent)(nil), // 1: envoy.config.core.v3.RuntimeFractionalPercent
}
var file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.matching.input_matchers.runtime_fraction.v3.RuntimeFraction.runtime_fraction:type_name -> envoy.config.core.v3.RuntimeFractionalPercent
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_init()
}
func file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_init() {
	if File_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDesc), len(file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_msgTypes,
	}.Build()
	File_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto = out.File
	file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_goTypes = nil
	file_envoy_extensions_matching_input_matchers_runtime_fraction_v3_runtime_fraction_proto_depIdxs = nil
}
