// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/service/runtime/v3/rtds.proto

package runtimev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/go-control-plane/envoy/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/service/discovery/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	structpb "google.golang.org/protobuf/types/known/structpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#not-implemented-hide:] Not configuration. Workaround c++ protobuf issue with importing
// services: https://github.com/google/protobuf/issues/4221
type RtdsDummy struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *RtdsDummy) Reset() {
	*x = RtdsDummy{}
	mi := &file_envoy_service_runtime_v3_rtds_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RtdsDummy) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RtdsDummy) ProtoMessage() {}

func (x *RtdsDummy) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_runtime_v3_rtds_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RtdsDummy.ProtoReflect.Descriptor instead.
func (*RtdsDummy) Descriptor() ([]byte, []int) {
	return file_envoy_service_runtime_v3_rtds_proto_rawDescGZIP(), []int{0}
}

// RTDS resource type. This describes a layer in the runtime virtual filesystem.
type Runtime struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Runtime resource name. This makes the Runtime a self-describing xDS
	// resource.
	Name          string           `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Layer         *structpb.Struct `protobuf:"bytes,2,opt,name=layer,proto3" json:"layer,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Runtime) Reset() {
	*x = Runtime{}
	mi := &file_envoy_service_runtime_v3_rtds_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Runtime) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Runtime) ProtoMessage() {}

func (x *Runtime) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_service_runtime_v3_rtds_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Runtime.ProtoReflect.Descriptor instead.
func (*Runtime) Descriptor() ([]byte, []int) {
	return file_envoy_service_runtime_v3_rtds_proto_rawDescGZIP(), []int{1}
}

func (x *Runtime) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

func (x *Runtime) GetLayer() *structpb.Struct {
	if x != nil {
		return x.Layer
	}
	return nil
}

var File_envoy_service_runtime_v3_rtds_proto protoreflect.FileDescriptor

const file_envoy_service_runtime_v3_rtds_proto_rawDesc = "" +
	"\n" +
	"#envoy/service/runtime/v3/rtds.proto\x12\x18envoy.service.runtime.v3\x1a*envoy/service/discovery/v3/discovery.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a envoy/annotations/resource.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"8\n" +
	"\tRtdsDummy:+\x9aň\x1e&\n" +
	"$envoy.service.discovery.v2.RtdsDummy\"\x80\x01\n" +
	"\aRuntime\x12\x1b\n" +
	"\x04name\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04name\x12-\n" +
	"\x05layer\x18\x02 \x01(\v2\x17.google.protobuf.StructR\x05layer:)\x9aň\x1e$\n" +
	"\"envoy.service.discovery.v2.Runtime2\xc4\x03\n" +
	"\x17RuntimeDiscoveryService\x12r\n" +
	"\rStreamRuntime\x12,.envoy.service.discovery.v3.DiscoveryRequest\x1a-.envoy.service.discovery.v3.DiscoveryResponse\"\x00(\x010\x01\x12{\n" +
	"\fDeltaRuntime\x121.envoy.service.discovery.v3.DeltaDiscoveryRequest\x1a2.envoy.service.discovery.v3.DeltaDiscoveryResponse\"\x00(\x010\x01\x12\x8d\x01\n" +
	"\fFetchRuntime\x12,.envoy.service.discovery.v3.DiscoveryRequest\x1a-.envoy.service.discovery.v3.DiscoveryResponse\" \x82\xd3\xe4\x93\x02\x1a:\x01*\"\x15/v3/discovery:runtime\x1a(\x8a\xa4\x96\xf3\a\"\n" +
	" envoy.service.runtime.v3.RuntimeB\x88\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"&io.envoyproxy.envoy.service.runtime.v3B\tRtdsProtoP\x01ZIgithub.com/envoyproxy/go-control-plane/envoy/service/runtime/v3;runtimev3b\x06proto3"

var (
	file_envoy_service_runtime_v3_rtds_proto_rawDescOnce sync.Once
	file_envoy_service_runtime_v3_rtds_proto_rawDescData []byte
)

func file_envoy_service_runtime_v3_rtds_proto_rawDescGZIP() []byte {
	file_envoy_service_runtime_v3_rtds_proto_rawDescOnce.Do(func() {
		file_envoy_service_runtime_v3_rtds_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_service_runtime_v3_rtds_proto_rawDesc), len(file_envoy_service_runtime_v3_rtds_proto_rawDesc)))
	})
	return file_envoy_service_runtime_v3_rtds_proto_rawDescData
}

var file_envoy_service_runtime_v3_rtds_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_service_runtime_v3_rtds_proto_goTypes = []any{
	(*RtdsDummy)(nil),                 // 0: envoy.service.runtime.v3.RtdsDummy
	(*Runtime)(nil),                   // 1: envoy.service.runtime.v3.Runtime
	(*structpb.Struct)(nil),           // 2: google.protobuf.Struct
	(*v3.DiscoveryRequest)(nil),       // 3: envoy.service.discovery.v3.DiscoveryRequest
	(*v3.DeltaDiscoveryRequest)(nil),  // 4: envoy.service.discovery.v3.DeltaDiscoveryRequest
	(*v3.DiscoveryResponse)(nil),      // 5: envoy.service.discovery.v3.DiscoveryResponse
	(*v3.DeltaDiscoveryResponse)(nil), // 6: envoy.service.discovery.v3.DeltaDiscoveryResponse
}
var file_envoy_service_runtime_v3_rtds_proto_depIdxs = []int32{
	2, // 0: envoy.service.runtime.v3.Runtime.layer:type_name -> google.protobuf.Struct
	3, // 1: envoy.service.runtime.v3.RuntimeDiscoveryService.StreamRuntime:input_type -> envoy.service.discovery.v3.DiscoveryRequest
	4, // 2: envoy.service.runtime.v3.RuntimeDiscoveryService.DeltaRuntime:input_type -> envoy.service.discovery.v3.DeltaDiscoveryRequest
	3, // 3: envoy.service.runtime.v3.RuntimeDiscoveryService.FetchRuntime:input_type -> envoy.service.discovery.v3.DiscoveryRequest
	5, // 4: envoy.service.runtime.v3.RuntimeDiscoveryService.StreamRuntime:output_type -> envoy.service.discovery.v3.DiscoveryResponse
	6, // 5: envoy.service.runtime.v3.RuntimeDiscoveryService.DeltaRuntime:output_type -> envoy.service.discovery.v3.DeltaDiscoveryResponse
	5, // 6: envoy.service.runtime.v3.RuntimeDiscoveryService.FetchRuntime:output_type -> envoy.service.discovery.v3.DiscoveryResponse
	4, // [4:7] is the sub-list for method output_type
	1, // [1:4] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_service_runtime_v3_rtds_proto_init() }
func file_envoy_service_runtime_v3_rtds_proto_init() {
	if File_envoy_service_runtime_v3_rtds_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_service_runtime_v3_rtds_proto_rawDesc), len(file_envoy_service_runtime_v3_rtds_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_envoy_service_runtime_v3_rtds_proto_goTypes,
		DependencyIndexes: file_envoy_service_runtime_v3_rtds_proto_depIdxs,
		MessageInfos:      file_envoy_service_runtime_v3_rtds_proto_msgTypes,
	}.Build()
	File_envoy_service_runtime_v3_rtds_proto = out.File
	file_envoy_service_runtime_v3_rtds_proto_goTypes = nil
	file_envoy_service_runtime_v3_rtds_proto_depIdxs = nil
}
