// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/network/dns_resolver/apple/v3/apple_dns_resolver.proto

package applev3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for apple DNS resolver.
type AppleDnsResolverConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The resolver will avoid the system's heuristics to only return
	// IPv4 or IPv6 addresses that it considers to be "routable", instead
	// returning all possible IPv4 or IPv6 addresses. This setting is
	// ignored if the DNS lookup family is set to v4-only or v6-only.
	// This should remain false in the vast majority of cases, but may be
	// useful when performing custom filtering of addresses, such as with
	// Happy Eyeballs.
	IncludeUnroutableFamilies bool `protobuf:"varint,1,opt,name=include_unroutable_families,json=includeUnroutableFamilies,proto3" json:"include_unroutable_families,omitempty"`
	unknownFields             protoimpl.UnknownFields
	sizeCache                 protoimpl.SizeCache
}

func (x *AppleDnsResolverConfig) Reset() {
	*x = AppleDnsResolverConfig{}
	mi := &file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *AppleDnsResolverConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*AppleDnsResolverConfig) ProtoMessage() {}

func (x *AppleDnsResolverConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use AppleDnsResolverConfig.ProtoReflect.Descriptor instead.
func (*AppleDnsResolverConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDescGZIP(), []int{0}
}

func (x *AppleDnsResolverConfig) GetIncludeUnroutableFamilies() bool {
	if x != nil {
		return x.IncludeUnroutableFamilies
	}
	return false
}

var File_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto protoreflect.FileDescriptor

const file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDesc = "" +
	"\n" +
	"Genvoy/extensions/network/dns_resolver/apple/v3/apple_dns_resolver.proto\x12.envoy.extensions.network.dns_resolver.apple.v3\x1a\x1dudpa/annotations/status.proto\"X\n" +
	"\x16AppleDnsResolverConfig\x12>\n" +
	"\x1binclude_unroutable_families\x18\x01 \x01(\bR\x19includeUnroutableFamiliesB\xbe\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"<io.envoyproxy.envoy.extensions.network.dns_resolver.apple.v3B\x15AppleDnsResolverProtoP\x01Z]github.com/envoyproxy/go-control-plane/envoy/extensions/network/dns_resolver/apple/v3;applev3b\x06proto3"

var (
	file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDescOnce sync.Once
	file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDescData []byte
)

func file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDescGZIP() []byte {
	file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDesc), len(file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDesc)))
	})
	return file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDescData
}

var file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_goTypes = []any{
	(*AppleDnsResolverConfig)(nil), // 0: envoy.extensions.network.dns_resolver.apple.v3.AppleDnsResolverConfig
}
var file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_init() }
func file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_init() {
	if File_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDesc), len(file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_msgTypes,
	}.Build()
	File_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto = out.File
	file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_goTypes = nil
	file_envoy_extensions_network_dns_resolver_apple_v3_apple_dns_resolver_proto_depIdxs = nil
}
