// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/config/ratelimit/v3/rls.proto

package ratelimitv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Rate limit :ref:`configuration overview <config_rate_limit_service>`.
type RateLimitServiceConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Specifies the gRPC service that hosts the rate limit service. The client
	// will connect to this cluster when it needs to make rate limit service
	// requests.
	GrpcService *v3.GrpcService `protobuf:"bytes,2,opt,name=grpc_service,json=grpcService,proto3" json:"grpc_service,omitempty"`
	// API version for rate limit transport protocol. This describes the rate limit gRPC endpoint and
	// version of messages used on the wire.
	TransportApiVersion v3.ApiVersion `protobuf:"varint,4,opt,name=transport_api_version,json=transportApiVersion,proto3,enum=envoy.config.core.v3.ApiVersion" json:"transport_api_version,omitempty"`
	unknownFields       protoimpl.UnknownFields
	sizeCache           protoimpl.SizeCache
}

func (x *RateLimitServiceConfig) Reset() {
	*x = RateLimitServiceConfig{}
	mi := &file_envoy_config_ratelimit_v3_rls_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *RateLimitServiceConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RateLimitServiceConfig) ProtoMessage() {}

func (x *RateLimitServiceConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_config_ratelimit_v3_rls_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RateLimitServiceConfig.ProtoReflect.Descriptor instead.
func (*RateLimitServiceConfig) Descriptor() ([]byte, []int) {
	return file_envoy_config_ratelimit_v3_rls_proto_rawDescGZIP(), []int{0}
}

func (x *RateLimitServiceConfig) GetGrpcService() *v3.GrpcService {
	if x != nil {
		return x.GrpcService
	}
	return nil
}

func (x *RateLimitServiceConfig) GetTransportApiVersion() v3.ApiVersion {
	if x != nil {
		return x.TransportApiVersion
	}
	return v3.ApiVersion(0)
}

var File_envoy_config_ratelimit_v3_rls_proto protoreflect.FileDescriptor

const file_envoy_config_ratelimit_v3_rls_proto_rawDesc = "" +
	"\n" +
	"#envoy/config/ratelimit/v3/rls.proto\x12\x19envoy.config.ratelimit.v3\x1a(envoy/config/core/v3/config_source.proto\x1a'envoy/config/core/v3/grpc_service.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x8d\x02\n" +
	"\x16RateLimitServiceConfig\x12N\n" +
	"\fgrpc_service\x18\x02 \x01(\v2!.envoy.config.core.v3.GrpcServiceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\vgrpcService\x12^\n" +
	"\x15transport_api_version\x18\x04 \x01(\x0e2 .envoy.config.core.v3.ApiVersionB\b\xfaB\x05\x82\x01\x02\x10\x01R\x13transportApiVersion:7\x9aň\x1e2\n" +
	"0envoy.config.ratelimit.v2.RateLimitServiceConfigJ\x04\b\x01\x10\x02J\x04\b\x03\x10\x04B\x8b\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"'io.envoyproxy.envoy.config.ratelimit.v3B\bRlsProtoP\x01ZLgithub.com/envoyproxy/go-control-plane/envoy/config/ratelimit/v3;ratelimitv3b\x06proto3"

var (
	file_envoy_config_ratelimit_v3_rls_proto_rawDescOnce sync.Once
	file_envoy_config_ratelimit_v3_rls_proto_rawDescData []byte
)

func file_envoy_config_ratelimit_v3_rls_proto_rawDescGZIP() []byte {
	file_envoy_config_ratelimit_v3_rls_proto_rawDescOnce.Do(func() {
		file_envoy_config_ratelimit_v3_rls_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_config_ratelimit_v3_rls_proto_rawDesc), len(file_envoy_config_ratelimit_v3_rls_proto_rawDesc)))
	})
	return file_envoy_config_ratelimit_v3_rls_proto_rawDescData
}

var file_envoy_config_ratelimit_v3_rls_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_config_ratelimit_v3_rls_proto_goTypes = []any{
	(*RateLimitServiceConfig)(nil), // 0: envoy.config.ratelimit.v3.RateLimitServiceConfig
	(*v3.GrpcService)(nil),         // 1: envoy.config.core.v3.GrpcService
	(v3.ApiVersion)(0),             // 2: envoy.config.core.v3.ApiVersion
}
var file_envoy_config_ratelimit_v3_rls_proto_depIdxs = []int32{
	1, // 0: envoy.config.ratelimit.v3.RateLimitServiceConfig.grpc_service:type_name -> envoy.config.core.v3.GrpcService
	2, // 1: envoy.config.ratelimit.v3.RateLimitServiceConfig.transport_api_version:type_name -> envoy.config.core.v3.ApiVersion
	2, // [2:2] is the sub-list for method output_type
	2, // [2:2] is the sub-list for method input_type
	2, // [2:2] is the sub-list for extension type_name
	2, // [2:2] is the sub-list for extension extendee
	0, // [0:2] is the sub-list for field type_name
}

func init() { file_envoy_config_ratelimit_v3_rls_proto_init() }
func file_envoy_config_ratelimit_v3_rls_proto_init() {
	if File_envoy_config_ratelimit_v3_rls_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_config_ratelimit_v3_rls_proto_rawDesc), len(file_envoy_config_ratelimit_v3_rls_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_config_ratelimit_v3_rls_proto_goTypes,
		DependencyIndexes: file_envoy_config_ratelimit_v3_rls_proto_depIdxs,
		MessageInfos:      file_envoy_config_ratelimit_v3_rls_proto_msgTypes,
	}.Build()
	File_envoy_config_ratelimit_v3_rls_proto = out.File
	file_envoy_config_ratelimit_v3_rls_proto_goTypes = nil
	file_envoy_config_ratelimit_v3_rls_proto_depIdxs = nil
}
