// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.6
// 	protoc        v5.29.3
// source: cilium/api/health_check_sink.proto

package cilium

import (
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Health check event pipe sink.
// The health check event will be streamed as binary protobufs.
type HealthCheckEventPipeSink struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Unix domain socket path where to connect to send health check events to.
	Path          string `protobuf:"bytes,1,opt,name=path,proto3" json:"path,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *HealthCheckEventPipeSink) Reset() {
	*x = HealthCheckEventPipeSink{}
	mi := &file_cilium_api_health_check_sink_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *HealthCheckEventPipeSink) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*HealthCheckEventPipeSink) ProtoMessage() {}

func (x *HealthCheckEventPipeSink) ProtoReflect() protoreflect.Message {
	mi := &file_cilium_api_health_check_sink_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use HealthCheckEventPipeSink.ProtoReflect.Descriptor instead.
func (*HealthCheckEventPipeSink) Descriptor() ([]byte, []int) {
	return file_cilium_api_health_check_sink_proto_rawDescGZIP(), []int{0}
}

func (x *HealthCheckEventPipeSink) GetPath() string {
	if x != nil {
		return x.Path
	}
	return ""
}

var File_cilium_api_health_check_sink_proto protoreflect.FileDescriptor

const file_cilium_api_health_check_sink_proto_rawDesc = "" +
	"\n" +
	"\"cilium/api/health_check_sink.proto\x12\x06cilium\x1a\x17validate/validate.proto\"7\n" +
	"\x18HealthCheckEventPipeSink\x12\x1b\n" +
	"\x04path\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04pathB.Z,github.com/cilium/proxy/go/cilium/api;ciliumb\x06proto3"

var (
	file_cilium_api_health_check_sink_proto_rawDescOnce sync.Once
	file_cilium_api_health_check_sink_proto_rawDescData []byte
)

func file_cilium_api_health_check_sink_proto_rawDescGZIP() []byte {
	file_cilium_api_health_check_sink_proto_rawDescOnce.Do(func() {
		file_cilium_api_health_check_sink_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_cilium_api_health_check_sink_proto_rawDesc), len(file_cilium_api_health_check_sink_proto_rawDesc)))
	})
	return file_cilium_api_health_check_sink_proto_rawDescData
}

var file_cilium_api_health_check_sink_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_cilium_api_health_check_sink_proto_goTypes = []any{
	(*HealthCheckEventPipeSink)(nil), // 0: cilium.HealthCheckEventPipeSink
}
var file_cilium_api_health_check_sink_proto_depIdxs = []int32{
	0, // [0:0] is the sub-list for method output_type
	0, // [0:0] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_cilium_api_health_check_sink_proto_init() }
func file_cilium_api_health_check_sink_proto_init() {
	if File_cilium_api_health_check_sink_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_cilium_api_health_check_sink_proto_rawDesc), len(file_cilium_api_health_check_sink_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_cilium_api_health_check_sink_proto_goTypes,
		DependencyIndexes: file_cilium_api_health_check_sink_proto_depIdxs,
		MessageInfos:      file_cilium_api_health_check_sink_proto_msgTypes,
	}.Build()
	File_cilium_api_health_check_sink_proto = out.File
	file_cilium_api_health_check_sink_proto_goTypes = nil
	file_cilium_api_health_check_sink_proto_depIdxs = nil
}
