// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/transport_sockets/quic/v3/quic_transport.proto

package quicv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	wrapperspb "google.golang.org/protobuf/types/known/wrapperspb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for Downstream QUIC transport socket. This provides Google's implementation of Google QUIC and IETF QUIC to Envoy.
type QuicDownstreamTransport struct {
	state                protoimpl.MessageState   `protogen:"open.v1"`
	DownstreamTlsContext *v3.DownstreamTlsContext `protobuf:"bytes,1,opt,name=downstream_tls_context,json=downstreamTlsContext,proto3" json:"downstream_tls_context,omitempty"`
	// If false, QUIC will tell TLS to reject any early data and to stop issuing 0-RTT credentials with resumption session tickets. This will prevent clients from sending 0-RTT requests.
	// Default to true.
	EnableEarlyData *wrapperspb.BoolValue `protobuf:"bytes,2,opt,name=enable_early_data,json=enableEarlyData,proto3" json:"enable_early_data,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *QuicDownstreamTransport) Reset() {
	*x = QuicDownstreamTransport{}
	mi := &file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *QuicDownstreamTransport) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*QuicDownstreamTransport) ProtoMessage() {}

func (x *QuicDownstreamTransport) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use QuicDownstreamTransport.ProtoReflect.Descriptor instead.
func (*QuicDownstreamTransport) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDescGZIP(), []int{0}
}

func (x *QuicDownstreamTransport) GetDownstreamTlsContext() *v3.DownstreamTlsContext {
	if x != nil {
		return x.DownstreamTlsContext
	}
	return nil
}

func (x *QuicDownstreamTransport) GetEnableEarlyData() *wrapperspb.BoolValue {
	if x != nil {
		return x.EnableEarlyData
	}
	return nil
}

// Configuration for Upstream QUIC transport socket. This provides Google's implementation of Google QUIC and IETF QUIC to Envoy.
type QuicUpstreamTransport struct {
	state              protoimpl.MessageState `protogen:"open.v1"`
	UpstreamTlsContext *v3.UpstreamTlsContext `protobuf:"bytes,1,opt,name=upstream_tls_context,json=upstreamTlsContext,proto3" json:"upstream_tls_context,omitempty"`
	unknownFields      protoimpl.UnknownFields
	sizeCache          protoimpl.SizeCache
}

func (x *QuicUpstreamTransport) Reset() {
	*x = QuicUpstreamTransport{}
	mi := &file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *QuicUpstreamTransport) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*QuicUpstreamTransport) ProtoMessage() {}

func (x *QuicUpstreamTransport) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use QuicUpstreamTransport.ProtoReflect.Descriptor instead.
func (*QuicUpstreamTransport) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDescGZIP(), []int{1}
}

func (x *QuicUpstreamTransport) GetUpstreamTlsContext() *v3.UpstreamTlsContext {
	if x != nil {
		return x.UpstreamTlsContext
	}
	return nil
}

var File_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto protoreflect.FileDescriptor

const file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDesc = "" +
	"\n" +
	"?envoy/extensions/transport_sockets/quic/v3/quic_transport.proto\x12*envoy.extensions.transport_sockets.quic.v3\x1a3envoy/extensions/transport_sockets/tls/v3/tls.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xe2\x01\n" +
	"\x17QuicDownstreamTransport\x12\x7f\n" +
	"\x16downstream_tls_context\x18\x01 \x01(\v2?.envoy.extensions.transport_sockets.tls.v3.DownstreamTlsContextB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x14downstreamTlsContext\x12F\n" +
	"\x11enable_early_data\x18\x02 \x01(\v2\x1a.google.protobuf.BoolValueR\x0fenableEarlyData\"\x92\x01\n" +
	"\x15QuicUpstreamTransport\x12y\n" +
	"\x14upstream_tls_context\x18\x01 \x01(\v2=.envoy.extensions.transport_sockets.tls.v3.UpstreamTlsContextB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x12upstreamTlsContextB\xb2\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"8io.envoyproxy.envoy.extensions.transport_sockets.quic.v3B\x12QuicTransportProtoP\x01ZXgithub.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/quic/v3;quicv3b\x06proto3"

var (
	file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDescOnce sync.Once
	file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDescData []byte
)

func file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDescGZIP() []byte {
	file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDesc), len(file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDesc)))
	})
	return file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDescData
}

var file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_goTypes = []any{
	(*QuicDownstreamTransport)(nil), // 0: envoy.extensions.transport_sockets.quic.v3.QuicDownstreamTransport
	(*QuicUpstreamTransport)(nil),   // 1: envoy.extensions.transport_sockets.quic.v3.QuicUpstreamTransport
	(*v3.DownstreamTlsContext)(nil), // 2: envoy.extensions.transport_sockets.tls.v3.DownstreamTlsContext
	(*wrapperspb.BoolValue)(nil),    // 3: google.protobuf.BoolValue
	(*v3.UpstreamTlsContext)(nil),   // 4: envoy.extensions.transport_sockets.tls.v3.UpstreamTlsContext
}
var file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.transport_sockets.quic.v3.QuicDownstreamTransport.downstream_tls_context:type_name -> envoy.extensions.transport_sockets.tls.v3.DownstreamTlsContext
	3, // 1: envoy.extensions.transport_sockets.quic.v3.QuicDownstreamTransport.enable_early_data:type_name -> google.protobuf.BoolValue
	4, // 2: envoy.extensions.transport_sockets.quic.v3.QuicUpstreamTransport.upstream_tls_context:type_name -> envoy.extensions.transport_sockets.tls.v3.UpstreamTlsContext
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() { file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_init() }
func file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_init() {
	if File_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDesc), len(file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_msgTypes,
	}.Build()
	File_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto = out.File
	file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_goTypes = nil
	file_envoy_extensions_transport_sockets_quic_v3_quic_transport_proto_depIdxs = nil
}
