// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/watchdog/profile_action/v3/profile_action.proto

package profile_actionv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the profile watchdog action.
type ProfileActionConfig struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// How long the profile should last. If not set defaults to 5 seconds.
	ProfileDuration *durationpb.Duration `protobuf:"bytes,1,opt,name=profile_duration,json=profileDuration,proto3" json:"profile_duration,omitempty"`
	// File path to the directory to output profiles.
	ProfilePath string `protobuf:"bytes,2,opt,name=profile_path,json=profilePath,proto3" json:"profile_path,omitempty"`
	// Limits the max number of profiles that can be generated by this action
	// over its lifetime to avoid filling the disk.
	// If not set (i.e. it's 0), a default of 10 will be used.
	MaxProfiles   uint64 `protobuf:"varint,3,opt,name=max_profiles,json=maxProfiles,proto3" json:"max_profiles,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ProfileActionConfig) Reset() {
	*x = ProfileActionConfig{}
	mi := &file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ProfileActionConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ProfileActionConfig) ProtoMessage() {}

func (x *ProfileActionConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ProfileActionConfig.ProtoReflect.Descriptor instead.
func (*ProfileActionConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDescGZIP(), []int{0}
}

func (x *ProfileActionConfig) GetProfileDuration() *durationpb.Duration {
	if x != nil {
		return x.ProfileDuration
	}
	return nil
}

func (x *ProfileActionConfig) GetProfilePath() string {
	if x != nil {
		return x.ProfilePath
	}
	return ""
}

func (x *ProfileActionConfig) GetMaxProfiles() uint64 {
	if x != nil {
		return x.MaxProfiles
	}
	return 0
}

var File_envoy_extensions_watchdog_profile_action_v3_profile_action_proto protoreflect.FileDescriptor

const file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDesc = "" +
	"\n" +
	"@envoy/extensions/watchdog/profile_action/v3/profile_action.proto\x12+envoy.extensions.watchdog.profile_action.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xaa\x01\n" +
	"\x13ProfileActionConfig\x12D\n" +
	"\x10profile_duration\x18\x01 \x01(\v2\x19.google.protobuf.DurationR\x0fprofileDuration\x12*\n" +
	"\fprofile_path\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\vprofilePath\x12!\n" +
	"\fmax_profiles\x18\x03 \x01(\x04R\vmaxProfilesB\xbe\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"9io.envoyproxy.envoy.extensions.watchdog.profile_action.v3B\x12ProfileActionProtoP\x01Zcgithub.com/envoyproxy/go-control-plane/envoy/extensions/watchdog/profile_action/v3;profile_actionv3b\x06proto3"

var (
	file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDescOnce sync.Once
	file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDescData []byte
)

func file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDescGZIP() []byte {
	file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDesc), len(file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDesc)))
	})
	return file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDescData
}

var file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_goTypes = []any{
	(*ProfileActionConfig)(nil), // 0: envoy.extensions.watchdog.profile_action.v3.ProfileActionConfig
	(*durationpb.Duration)(nil), // 1: google.protobuf.Duration
}
var file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.watchdog.profile_action.v3.ProfileActionConfig.profile_duration:type_name -> google.protobuf.Duration
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_init() }
func file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_init() {
	if File_envoy_extensions_watchdog_profile_action_v3_profile_action_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDesc), len(file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_msgTypes,
	}.Build()
	File_envoy_extensions_watchdog_profile_action_v3_profile_action_proto = out.File
	file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_goTypes = nil
	file_envoy_extensions_watchdog_profile_action_v3_profile_action_proto_depIdxs = nil
}
