// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/resource_monitors/cpu_utilization/v3/cpu_utilization.proto

package cpu_utilizationv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type CpuUtilizationConfig_UtilizationComputeStrategy int32

const (
	// Reports CPU Utilization of the entire Linux Host machine
	CpuUtilizationConfig_HOST CpuUtilizationConfig_UtilizationComputeStrategy = 0
	// Reports CPU Utilization of Envoy Container using CGROUP stats in Linux K8s environment
	CpuUtilizationConfig_CONTAINER CpuUtilizationConfig_UtilizationComputeStrategy = 1
)

// Enum value maps for CpuUtilizationConfig_UtilizationComputeStrategy.
var (
	CpuUtilizationConfig_UtilizationComputeStrategy_name = map[int32]string{
		0: "HOST",
		1: "CONTAINER",
	}
	CpuUtilizationConfig_UtilizationComputeStrategy_value = map[string]int32{
		"HOST":      0,
		"CONTAINER": 1,
	}
)

func (x CpuUtilizationConfig_UtilizationComputeStrategy) Enum() *CpuUtilizationConfig_UtilizationComputeStrategy {
	p := new(CpuUtilizationConfig_UtilizationComputeStrategy)
	*p = x
	return p
}

func (x CpuUtilizationConfig_UtilizationComputeStrategy) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (CpuUtilizationConfig_UtilizationComputeStrategy) Descriptor() protoreflect.EnumDescriptor {
	return file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_enumTypes[0].Descriptor()
}

func (CpuUtilizationConfig_UtilizationComputeStrategy) Type() protoreflect.EnumType {
	return &file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_enumTypes[0]
}

func (x CpuUtilizationConfig_UtilizationComputeStrategy) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use CpuUtilizationConfig_UtilizationComputeStrategy.Descriptor instead.
func (CpuUtilizationConfig_UtilizationComputeStrategy) EnumDescriptor() ([]byte, []int) {
	return file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDescGZIP(), []int{0, 0}
}

// The CPU utilization resource monitor reports the Envoy process the CPU Utilization across different platforms.
type CpuUtilizationConfig struct {
	state         protoimpl.MessageState                          `protogen:"open.v1"`
	Mode          CpuUtilizationConfig_UtilizationComputeStrategy `protobuf:"varint,1,opt,name=mode,proto3,enum=envoy.extensions.resource_monitors.cpu_utilization.v3.CpuUtilizationConfig_UtilizationComputeStrategy" json:"mode,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *CpuUtilizationConfig) Reset() {
	*x = CpuUtilizationConfig{}
	mi := &file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *CpuUtilizationConfig) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*CpuUtilizationConfig) ProtoMessage() {}

func (x *CpuUtilizationConfig) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use CpuUtilizationConfig.ProtoReflect.Descriptor instead.
func (*CpuUtilizationConfig) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDescGZIP(), []int{0}
}

func (x *CpuUtilizationConfig) GetMode() CpuUtilizationConfig_UtilizationComputeStrategy {
	if x != nil {
		return x.Mode
	}
	return CpuUtilizationConfig_HOST
}

var File_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto protoreflect.FileDescriptor

const file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDesc = "" +
	"\n" +
	"Kenvoy/extensions/resource_monitors/cpu_utilization/v3/cpu_utilization.proto\x125envoy.extensions.resource_monitors.cpu_utilization.v3\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xd4\x01\n" +
	"\x14CpuUtilizationConfig\x12\x84\x01\n" +
	"\x04mode\x18\x01 \x01(\x0e2f.envoy.extensions.resource_monitors.cpu_utilization.v3.CpuUtilizationConfig.UtilizationComputeStrategyB\b\xfaB\x05\x82\x01\x02\x10\x01R\x04mode\"5\n" +
	"\x1aUtilizationComputeStrategy\x12\b\n" +
	"\x04HOST\x10\x00\x12\r\n" +
	"\tCONTAINER\x10\x01B\xd4\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Cio.envoyproxy.envoy.extensions.resource_monitors.cpu_utilization.v3B\x13CpuUtilizationProtoP\x01Zngithub.com/envoyproxy/go-control-plane/envoy/extensions/resource_monitors/cpu_utilization/v3;cpu_utilizationv3b\x06proto3"

var (
	file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDescOnce sync.Once
	file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDescData []byte
)

func file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDescGZIP() []byte {
	file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDesc), len(file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDesc)))
	})
	return file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDescData
}

var file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_enumTypes = make([]protoimpl.EnumInfo, 1)
var file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_goTypes = []any{
	(CpuUtilizationConfig_UtilizationComputeStrategy)(0), // 0: envoy.extensions.resource_monitors.cpu_utilization.v3.CpuUtilizationConfig.UtilizationComputeStrategy
	(*CpuUtilizationConfig)(nil),                         // 1: envoy.extensions.resource_monitors.cpu_utilization.v3.CpuUtilizationConfig
}
var file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_depIdxs = []int32{
	0, // 0: envoy.extensions.resource_monitors.cpu_utilization.v3.CpuUtilizationConfig.mode:type_name -> envoy.extensions.resource_monitors.cpu_utilization.v3.CpuUtilizationConfig.UtilizationComputeStrategy
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_init() }
func file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_init() {
	if File_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDesc), len(file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_rawDesc)),
			NumEnums:      1,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_depIdxs,
		EnumInfos:         file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_enumTypes,
		MessageInfos:      file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_msgTypes,
	}.Build()
	File_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto = out.File
	file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_goTypes = nil
	file_envoy_extensions_resource_monitors_cpu_utilization_v3_cpu_utilization_proto_depIdxs = nil
}
