// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/filters/network/generic_proxy/v3/generic_proxy.proto

package generic_proxyv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	v32 "github.com/envoyproxy/go-control-plane/envoy/config/accesslog/v3"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v31 "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/http_connection_manager/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// [#next-free-field: 8]
type GenericProxy struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// The human readable prefix to use when emitting statistics.
	StatPrefix string `protobuf:"bytes,1,opt,name=stat_prefix,json=statPrefix,proto3" json:"stat_prefix,omitempty"`
	// The codec which encodes and decodes the application protocol.
	// [#extension-category: envoy.generic_proxy.codecs]
	CodecConfig *v3.TypedExtensionConfig `protobuf:"bytes,2,opt,name=codec_config,json=codecConfig,proto3" json:"codec_config,omitempty"`
	// Types that are valid to be assigned to RouteSpecifier:
	//
	//	*GenericProxy_GenericRds
	//	*GenericProxy_RouteConfig
	RouteSpecifier isGenericProxy_RouteSpecifier `protobuf_oneof:"route_specifier"`
	// A list of individual Layer-7 filters that make up the filter chain for requests made to the
	// proxy. Order matters as the filters are processed sequentially as request events
	// happen.
	// [#extension-category: envoy.generic_proxy.filters]
	Filters []*v3.TypedExtensionConfig `protobuf:"bytes,5,rep,name=filters,proto3" json:"filters,omitempty"`
	// Tracing configuration for the generic proxy.
	Tracing *v31.HttpConnectionManager_Tracing `protobuf:"bytes,6,opt,name=tracing,proto3" json:"tracing,omitempty"`
	// Configuration for :ref:`access logs <arch_overview_access_logs>` emitted by generic proxy.
	AccessLog     []*v32.AccessLog `protobuf:"bytes,7,rep,name=access_log,json=accessLog,proto3" json:"access_log,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *GenericProxy) Reset() {
	*x = GenericProxy{}
	mi := &file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GenericProxy) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GenericProxy) ProtoMessage() {}

func (x *GenericProxy) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GenericProxy.ProtoReflect.Descriptor instead.
func (*GenericProxy) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDescGZIP(), []int{0}
}

func (x *GenericProxy) GetStatPrefix() string {
	if x != nil {
		return x.StatPrefix
	}
	return ""
}

func (x *GenericProxy) GetCodecConfig() *v3.TypedExtensionConfig {
	if x != nil {
		return x.CodecConfig
	}
	return nil
}

func (x *GenericProxy) GetRouteSpecifier() isGenericProxy_RouteSpecifier {
	if x != nil {
		return x.RouteSpecifier
	}
	return nil
}

func (x *GenericProxy) GetGenericRds() *GenericRds {
	if x != nil {
		if x, ok := x.RouteSpecifier.(*GenericProxy_GenericRds); ok {
			return x.GenericRds
		}
	}
	return nil
}

func (x *GenericProxy) GetRouteConfig() *RouteConfiguration {
	if x != nil {
		if x, ok := x.RouteSpecifier.(*GenericProxy_RouteConfig); ok {
			return x.RouteConfig
		}
	}
	return nil
}

func (x *GenericProxy) GetFilters() []*v3.TypedExtensionConfig {
	if x != nil {
		return x.Filters
	}
	return nil
}

func (x *GenericProxy) GetTracing() *v31.HttpConnectionManager_Tracing {
	if x != nil {
		return x.Tracing
	}
	return nil
}

func (x *GenericProxy) GetAccessLog() []*v32.AccessLog {
	if x != nil {
		return x.AccessLog
	}
	return nil
}

type isGenericProxy_RouteSpecifier interface {
	isGenericProxy_RouteSpecifier()
}

type GenericProxy_GenericRds struct {
	// The generic proxies route table will be dynamically loaded via the meta RDS API.
	GenericRds *GenericRds `protobuf:"bytes,3,opt,name=generic_rds,json=genericRds,proto3,oneof"`
}

type GenericProxy_RouteConfig struct {
	// The route table for the generic proxy is static and is specified in this property.
	RouteConfig *RouteConfiguration `protobuf:"bytes,4,opt,name=route_config,json=routeConfig,proto3,oneof"`
}

func (*GenericProxy_GenericRds) isGenericProxy_RouteSpecifier() {}

func (*GenericProxy_RouteConfig) isGenericProxy_RouteSpecifier() {}

type GenericRds struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Configuration source specifier for RDS.
	ConfigSource *v3.ConfigSource `protobuf:"bytes,1,opt,name=config_source,json=configSource,proto3" json:"config_source,omitempty"`
	// The name of the route configuration. This name will be passed to the RDS API. This allows an
	// Envoy configuration with multiple generic proxies to use different route configurations.
	RouteConfigName string `protobuf:"bytes,2,opt,name=route_config_name,json=routeConfigName,proto3" json:"route_config_name,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *GenericRds) Reset() {
	*x = GenericRds{}
	mi := &file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *GenericRds) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*GenericRds) ProtoMessage() {}

func (x *GenericRds) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use GenericRds.ProtoReflect.Descriptor instead.
func (*GenericRds) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDescGZIP(), []int{1}
}

func (x *GenericRds) GetConfigSource() *v3.ConfigSource {
	if x != nil {
		return x.ConfigSource
	}
	return nil
}

func (x *GenericRds) GetRouteConfigName() string {
	if x != nil {
		return x.RouteConfigName
	}
	return ""
}

var File_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto protoreflect.FileDescriptor

const file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDesc = "" +
	"\n" +
	"Eenvoy/extensions/filters/network/generic_proxy/v3/generic_proxy.proto\x121envoy.extensions.filters.network.generic_proxy.v3\x1a)envoy/config/accesslog/v3/accesslog.proto\x1a(envoy/config/core/v3/config_source.proto\x1a$envoy/config/core/v3/extension.proto\x1a=envoy/extensions/filters/network/generic_proxy/v3/route.proto\x1aYenvoy/extensions/filters/network/http_connection_manager/v3/http_connection_manager.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xf8\x04\n" +
	"\fGenericProxy\x12(\n" +
	"\vstat_prefix\x18\x01 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\n" +
	"statPrefix\x12W\n" +
	"\fcodec_config\x18\x02 \x01(\v2*.envoy.config.core.v3.TypedExtensionConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\vcodecConfig\x12`\n" +
	"\vgeneric_rds\x18\x03 \x01(\v2=.envoy.extensions.filters.network.generic_proxy.v3.GenericRdsH\x00R\n" +
	"genericRds\x12j\n" +
	"\froute_config\x18\x04 \x01(\v2E.envoy.extensions.filters.network.generic_proxy.v3.RouteConfigurationH\x00R\vrouteConfig\x12D\n" +
	"\afilters\x18\x05 \x03(\v2*.envoy.config.core.v3.TypedExtensionConfigR\afilters\x12t\n" +
	"\atracing\x18\x06 \x01(\v2Z.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.TracingR\atracing\x12C\n" +
	"\n" +
	"access_log\x18\a \x03(\v2$.envoy.config.accesslog.v3.AccessLogR\taccessLogB\x16\n" +
	"\x0froute_specifier\x12\x03\xf8B\x01\"\x94\x01\n" +
	"\n" +
	"GenericRds\x12Q\n" +
	"\rconfig_source\x18\x01 \x01(\v2\".envoy.config.core.v3.ConfigSourceB\b\xfaB\x05\x8a\x01\x02\x10\x01R\fconfigSource\x123\n" +
	"\x11route_config_name\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x0frouteConfigNameB\xd0\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"?io.envoyproxy.envoy.extensions.filters.network.generic_proxy.v3B\x11GenericProxyProtoP\x01Zhgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/generic_proxy/v3;generic_proxyv3b\x06proto3"

var (
	file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDescOnce sync.Once
	file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDescData []byte
)

func file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDescGZIP() []byte {
	file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDesc), len(file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDesc)))
	})
	return file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDescData
}

var file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_goTypes = []any{
	(*GenericProxy)(nil),                      // 0: envoy.extensions.filters.network.generic_proxy.v3.GenericProxy
	(*GenericRds)(nil),                        // 1: envoy.extensions.filters.network.generic_proxy.v3.GenericRds
	(*v3.TypedExtensionConfig)(nil),           // 2: envoy.config.core.v3.TypedExtensionConfig
	(*RouteConfiguration)(nil),                // 3: envoy.extensions.filters.network.generic_proxy.v3.RouteConfiguration
	(*v31.HttpConnectionManager_Tracing)(nil), // 4: envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.Tracing
	(*v32.AccessLog)(nil),                     // 5: envoy.config.accesslog.v3.AccessLog
	(*v3.ConfigSource)(nil),                   // 6: envoy.config.core.v3.ConfigSource
}
var file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_depIdxs = []int32{
	2, // 0: envoy.extensions.filters.network.generic_proxy.v3.GenericProxy.codec_config:type_name -> envoy.config.core.v3.TypedExtensionConfig
	1, // 1: envoy.extensions.filters.network.generic_proxy.v3.GenericProxy.generic_rds:type_name -> envoy.extensions.filters.network.generic_proxy.v3.GenericRds
	3, // 2: envoy.extensions.filters.network.generic_proxy.v3.GenericProxy.route_config:type_name -> envoy.extensions.filters.network.generic_proxy.v3.RouteConfiguration
	2, // 3: envoy.extensions.filters.network.generic_proxy.v3.GenericProxy.filters:type_name -> envoy.config.core.v3.TypedExtensionConfig
	4, // 4: envoy.extensions.filters.network.generic_proxy.v3.GenericProxy.tracing:type_name -> envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.Tracing
	5, // 5: envoy.extensions.filters.network.generic_proxy.v3.GenericProxy.access_log:type_name -> envoy.config.accesslog.v3.AccessLog
	6, // 6: envoy.extensions.filters.network.generic_proxy.v3.GenericRds.config_source:type_name -> envoy.config.core.v3.ConfigSource
	7, // [7:7] is the sub-list for method output_type
	7, // [7:7] is the sub-list for method input_type
	7, // [7:7] is the sub-list for extension type_name
	7, // [7:7] is the sub-list for extension extendee
	0, // [0:7] is the sub-list for field type_name
}

func init() { file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_init() }
func file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_init() {
	if File_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto != nil {
		return
	}
	file_envoy_extensions_filters_network_generic_proxy_v3_route_proto_init()
	file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_msgTypes[0].OneofWrappers = []any{
		(*GenericProxy_GenericRds)(nil),
		(*GenericProxy_RouteConfig)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDesc), len(file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_msgTypes,
	}.Build()
	File_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto = out.File
	file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_goTypes = nil
	file_envoy_extensions_filters_network_generic_proxy_v3_generic_proxy_proto_depIdxs = nil
}
