// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v6.33.2
// source: envoy/extensions/transport_sockets/internal_upstream/v3/internal_upstream.proto

package internal_upstreamv3

import (
	_ "github.com/cncf/xds/go/udpa/annotations"
	v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
	v31 "github.com/envoyproxy/go-control-plane/envoy/type/metadata/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Configuration for the internal upstream address. An internal address defines
// a loopback user space socket residing in the same proxy instance. This
// extension allows passing additional structured state across the user space
// socket in addition to the regular byte stream. The purpose is to facilitate
// communication between filters on the downstream and the upstream internal
// connections. All filter state objects that are shared with the upstream
// connection are also shared with the downstream internal connection using
// this transport socket.
type InternalUpstreamTransport struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Specifies the metadata namespaces and values to insert into the downstream
	// internal connection dynamic metadata when an internal address is used as a
	// host. If the destination name is repeated across two metadata source
	// locations, and both locations contain the metadata with the given name,
	// then the latter in the list overrides the former.
	PassthroughMetadata []*InternalUpstreamTransport_MetadataValueSource `protobuf:"bytes,1,rep,name=passthrough_metadata,json=passthroughMetadata,proto3" json:"passthrough_metadata,omitempty"`
	// The underlying transport socket being wrapped.
	TransportSocket *v3.TransportSocket `protobuf:"bytes,3,opt,name=transport_socket,json=transportSocket,proto3" json:"transport_socket,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *InternalUpstreamTransport) Reset() {
	*x = InternalUpstreamTransport{}
	mi := &file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *InternalUpstreamTransport) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*InternalUpstreamTransport) ProtoMessage() {}

func (x *InternalUpstreamTransport) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use InternalUpstreamTransport.ProtoReflect.Descriptor instead.
func (*InternalUpstreamTransport) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDescGZIP(), []int{0}
}

func (x *InternalUpstreamTransport) GetPassthroughMetadata() []*InternalUpstreamTransport_MetadataValueSource {
	if x != nil {
		return x.PassthroughMetadata
	}
	return nil
}

func (x *InternalUpstreamTransport) GetTransportSocket() *v3.TransportSocket {
	if x != nil {
		return x.TransportSocket
	}
	return nil
}

// Describes the location of the imported metadata value.
// If the metadata with the given name is not present at the source location,
// then no metadata is passed through for this particular instance.
type InternalUpstreamTransport_MetadataValueSource struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Specifies what kind of metadata.
	Kind *v31.MetadataKind `protobuf:"bytes,1,opt,name=kind,proto3" json:"kind,omitempty"`
	// Name is the filter namespace used in the dynamic metadata.
	Name          string `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *InternalUpstreamTransport_MetadataValueSource) Reset() {
	*x = InternalUpstreamTransport_MetadataValueSource{}
	mi := &file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *InternalUpstreamTransport_MetadataValueSource) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*InternalUpstreamTransport_MetadataValueSource) ProtoMessage() {}

func (x *InternalUpstreamTransport_MetadataValueSource) ProtoReflect() protoreflect.Message {
	mi := &file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use InternalUpstreamTransport_MetadataValueSource.ProtoReflect.Descriptor instead.
func (*InternalUpstreamTransport_MetadataValueSource) Descriptor() ([]byte, []int) {
	return file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDescGZIP(), []int{0, 0}
}

func (x *InternalUpstreamTransport_MetadataValueSource) GetKind() *v31.MetadataKind {
	if x != nil {
		return x.Kind
	}
	return nil
}

func (x *InternalUpstreamTransport_MetadataValueSource) GetName() string {
	if x != nil {
		return x.Name
	}
	return ""
}

var File_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto protoreflect.FileDescriptor

const file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDesc = "" +
	"\n" +
	"Oenvoy/extensions/transport_sockets/internal_upstream/v3/internal_upstream.proto\x127envoy.extensions.transport_sockets.internal_upstream.v3\x1a\x1fenvoy/config/core/v3/base.proto\x1a%envoy/type/metadata/v3/metadata.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x8b\x03\n" +
	"\x19InternalUpstreamTransport\x12\x99\x01\n" +
	"\x14passthrough_metadata\x18\x01 \x03(\v2f.envoy.extensions.transport_sockets.internal_upstream.v3.InternalUpstreamTransport.MetadataValueSourceR\x13passthroughMetadata\x12Z\n" +
	"\x10transport_socket\x18\x03 \x01(\v2%.envoy.config.core.v3.TransportSocketB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x0ftransportSocket\x1av\n" +
	"\x13MetadataValueSource\x12B\n" +
	"\x04kind\x18\x01 \x01(\v2$.envoy.type.metadata.v3.MetadataKindB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x04kind\x12\x1b\n" +
	"\x04name\x18\x02 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\x04nameB\xdc\x01\xba\x80\xc8\xd1\x06\x02\x10\x02\n" +
	"Eio.envoyproxy.envoy.extensions.transport_sockets.internal_upstream.v3B\x15InternalUpstreamProtoP\x01Zrgithub.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/internal_upstream/v3;internal_upstreamv3b\x06proto3"

var (
	file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDescOnce sync.Once
	file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDescData []byte
)

func file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDescGZIP() []byte {
	file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDescOnce.Do(func() {
		file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDesc), len(file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDesc)))
	})
	return file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDescData
}

var file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_goTypes = []any{
	(*InternalUpstreamTransport)(nil),                     // 0: envoy.extensions.transport_sockets.internal_upstream.v3.InternalUpstreamTransport
	(*InternalUpstreamTransport_MetadataValueSource)(nil), // 1: envoy.extensions.transport_sockets.internal_upstream.v3.InternalUpstreamTransport.MetadataValueSource
	(*v3.TransportSocket)(nil),                            // 2: envoy.config.core.v3.TransportSocket
	(*v31.MetadataKind)(nil),                              // 3: envoy.type.metadata.v3.MetadataKind
}
var file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_depIdxs = []int32{
	1, // 0: envoy.extensions.transport_sockets.internal_upstream.v3.InternalUpstreamTransport.passthrough_metadata:type_name -> envoy.extensions.transport_sockets.internal_upstream.v3.InternalUpstreamTransport.MetadataValueSource
	2, // 1: envoy.extensions.transport_sockets.internal_upstream.v3.InternalUpstreamTransport.transport_socket:type_name -> envoy.config.core.v3.TransportSocket
	3, // 2: envoy.extensions.transport_sockets.internal_upstream.v3.InternalUpstreamTransport.MetadataValueSource.kind:type_name -> envoy.type.metadata.v3.MetadataKind
	3, // [3:3] is the sub-list for method output_type
	3, // [3:3] is the sub-list for method input_type
	3, // [3:3] is the sub-list for extension type_name
	3, // [3:3] is the sub-list for extension extendee
	0, // [0:3] is the sub-list for field type_name
}

func init() {
	file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_init()
}
func file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_init() {
	if File_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDesc), len(file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_goTypes,
		DependencyIndexes: file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_depIdxs,
		MessageInfos:      file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_msgTypes,
	}.Build()
	File_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto = out.File
	file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_goTypes = nil
	file_envoy_extensions_transport_sockets_internal_upstream_v3_internal_upstream_proto_depIdxs = nil
}
